/**
* QSortExample
*
* This class illustrates a simple example of using the
* NetSolve Java API.  An array of integers is initialized, 
* sorted, and the resulting array is printed to the screen.
*
* @version 0.0, 24 Sep 1997
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

import java.util.Vector;

public class QSortExample extends Example {
  public static void main (String [] args) {
    QSortExample qs = new QSortExample();
    NetsolveSession ns;
    Netsolve n;
    int [] blah = {765,653,534,67,212,987,43,8,543,8357,2};
    int [] sorted, i;
    Vector out;

    // Since these variables are all set within the 'try'
    // block, the compiler can't confirm that they are 
    // initialized.  To avoid the subsequent complaints from
    // the compiler, we initialize them to null here.

    n = null;
    ns = null;
    out = null;

    // Almost all of the methods in Netsolve and NetsolveSession
    // throw NetSolveException upon some error condition, so it's
    // easier to just enclose the whole transaction in a try-catch
    // block than to try each one individually (although that is
    // possible, too).

    try {
      // First, we create a new session, specifying the hostname
      // of the agent we want to contact.
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      // To initialize a new Netsolve object, pass in the session
      // object.
      n = new Netsolve(ns);

      // Specify which problem to solve (no need for the whole
      // pathname of the problem).
      n.setProblem("iqsort");

      // There's only one argument to qsort, so push it and submit
      // this problem to be solved.
      n.pushArg(blah);

      n.submitProb();    

      // submitProb is the blocking call to NetSolve, so by the
      // time we reach this point, the problem has been solved
      // and we can access the data through the getOutput() method.
      // Note that the data is returned in a Vector, each element
      // containing one of the output items.  See below.
      out = n.getOutput();

    }catch(NetSolveException e) {
      // An error has been detected sometime during the problem 
      // specification, submission, or solving.  Here we just print
      // the error message and quit, however you don't necessarily
      // have to exit the program, depending on your application.
 
      System.err.println(e.getMessage());
      System.exit(1);
    }
   
    // Now if everything went well, we can get the output item
    // from the vector.  elementAt() returns an Object, so we must
    // cast the return value to the appropriate type.

    sorted = (int []) (out.elementAt(0));

    // Print the results to verify that it worked.

    System.out.println("Unsorted:");
    qs.printVec(blah);

    System.out.println("Sorted:");
    qs.printVec(sorted);
  }
}
