/**
* TotaltestExample
*
* This program illustrates how to initialize and retrieve every 
* data type that NetSolve currently supports except for the
* User Provided Function.  The 'totaltest' problem takes one item
* of every data type and increments each element by one.  In summary,
* we initialize all the variables, submit the data, and then
* print the results.
*
* @version 0.0, 24 Sep 1997
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

import java.util.Vector;

public class TotaltestExample extends Example {
  public static void main (String [] args) {
    TotaltestExample tt = new TotaltestExample();
    NetsolveSession ns;
    Netsolve n;
    Vector vec;
    int i,j;

    int int_scalar = 1;
    int [] int_vector = {1,2,3,4};
    int [][] int_matrix = {
      {1,2,3,4},
      {5,6,7,8},
      {9,10,11,12},
      {13,14,15,16}
    };

    char char_scalar = 'A';
    char [] char_vector = {'A', 'B', 'C', 'D'};
    char [][] char_matrix = {    
      {'A', 'B', 'C', 'D'},
      {'E', 'F', 'G', 'H'},
      {'I', 'J', 'K', 'L'},
      {'M', 'N', 'O', 'P'}
    };

    // We must cast the constants to float to make the compiler happy
    float float_scalar = (float) 1.1;
    float [] float_vector = {(float) 1.1, (float) 2.2, (float) 3.3, (float) 4.4};
    float [][] float_matrix = {  
      {(float) 1.1, (float)   2.2, (float)  3.3, (float)  4.4},
      {(float) 5.1,  (float)  6.2, (float)  7.3,  (float) 8.4},
      {(float) 9.1, (float)  10.2, (float) 11.3, (float) 12.4},
      {(float) 13.1, (float) 14.2, (float) 15.3, (float) 16.4}
    };

    double double_scalar = 1.1;
    double [] double_vector = {1.1, 2.2, 3.3, 4.4};
    double [][] double_matrix = {  
      {1.1,   2.2,  3.3,  4.4},
      {5.1,   6.2,  7.3,  8.4},
      {9.1,  10.2, 11.3, 12.4},
      {13.1, 14.2, 15.3, 16.4}
    };

    // *** NOTICE *** Complex scalars do not have their own data type.
    //                They are sent as an array, the first element being
    //                the real part and the second element the imaginary.
    //
    // No explicit casts needed here since I didn't use floating point 
    // constants.
    float [] scomplex_scalar = {1,0};
    float [] scomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    float [][] scomplex_matrix = {
      {1,0 , 2,0 , 3,0 , 4,0},
      {5,0 , 6,0 , 7,0 , 8,0},
      {9,0 , 10,0 , 11,0 , 12,0},
      {13,0 , 14,0 , 15,0 , 16,0},
    };

    // Double precision complex numbers are initialized the same way
    // as single precision.
    double [] dcomplex_scalar = {1,0};
    double [] dcomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    double [][] dcomplex_matrix = {
      {1,0 , 2,0 , 3,0 , 4,0},
      {5,0 , 6,0 , 7,0 , 8,0},
      {9,0 , 10,0 , 11,0 , 12,0},
      {13,0 , 14,0 , 15,0 , 16,0},
    };

    byte byte_scalar = 1;
    byte [] byte_vector = {1,2,3,4};
    byte [][] byte_matrix = {
      {1,2,3,4},
      {5,6,7,8},
      {9,10,11,12},
      {13,14,15,16}
    };

    // As usual, we initialize the variables that will be set inside
    // the try/catch block since the compiler complains otherwise.
    ns = null;
    n = null;
    vec = null;

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);       // Specify 'session'
      n.setProblem("totaltest");  // Specify the problem to solve

      n.pushArg(int_scalar);      // Push all the args
      n.pushArg(int_vector);
      n.pushArg(int_matrix);
      n.pushArg(char_scalar);
      n.pushArg(char_vector);
      n.pushArg(char_matrix);
      n.pushArg(float_scalar);
      n.pushArg(float_vector);
      n.pushArg(float_matrix);
      n.pushArg(double_scalar);
      n.pushArg(double_vector);
      n.pushArg(double_matrix);
      n.pushArg(scomplex_scalar);
      n.pushArg(scomplex_vector);
      n.pushArg(scomplex_matrix);
      n.pushArg(dcomplex_scalar);
      n.pushArg(dcomplex_vector);
      n.pushArg(dcomplex_matrix);
      n.pushArg(byte_scalar);      // Push all the args
      n.pushArg(byte_vector);
      n.pushArg(byte_matrix);
      n.submitProb();           // Submit this problem
      vec = n.getOutput();
    }catch(NetSolveException e) {
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Now that we have the results, we can print all this
    // data out.  Since elementAt() returns Object, we must
    // add in all the explicit casts.  This means that you
    // have to know ahead of time what kind of data the
    // problem is going to return so that you know what
    // type to cast it as.

    int_scalar = ((Integer) (vec.elementAt(0))).intValue();
    System.out.println("Integer Scalar:");
    System.out.println( int_scalar );

    int_vector = (int []) (vec.elementAt(1));
    System.out.println("Integer Vector:");
    tt.printVec(int_vector);

    int_matrix = (int [][]) (vec.elementAt(2));
    System.out.println("Integer Matrix:");
    tt.printMat(int_matrix);
    
    char_scalar = ((Character) (vec.elementAt(3))).charValue();
    System.out.println("Character Scalar:");
    System.out.println( char_scalar );

    char_vector = (char []) (vec.elementAt(4));
    System.out.println("Character Vector:");
    tt.printVec(char_vector);

    char_matrix = (char [][]) (vec.elementAt(5));
    System.out.println("Character Matrix:");
    tt.printMat(char_matrix);
    
    float_scalar = ((Float) (vec.elementAt(6))).floatValue();
    System.out.println("Float Scalar:");
    System.out.println( float_scalar );

    float_vector = (float []) (vec.elementAt(7));
    System.out.println("Float Vector:");
    tt.printVec(float_vector);

    float_matrix = (float [][]) (vec.elementAt(8));
    System.out.println("Float Matrix:");
    tt.printMat(float_matrix);
    
    double_scalar = ((Double) (vec.elementAt(9))).doubleValue();
    System.out.println("Double Scalar:");
    System.out.println( double_scalar );

    double_vector = (double []) (vec.elementAt(10));
    System.out.println("Double Vector:");
    tt.printVec(double_vector);

    double_matrix = (double [][]) (vec.elementAt(11));
    System.out.println("Double Matrix:");
    tt.printMat(double_matrix);
    
    scomplex_scalar = (float []) (vec.elementAt(12));
    System.out.println("Scomplex Scalar:");
    tt.printComplex(scomplex_scalar);

    scomplex_vector = (float []) (vec.elementAt(13));
    System.out.println("Scomplex Vector:");
    tt.printComplexVec(scomplex_vector);

    scomplex_matrix = (float [][]) (vec.elementAt(14));
    System.out.println("Scomplex Matrix:");
    tt.printComplexMat(scomplex_matrix);
    
    dcomplex_scalar = (double []) (vec.elementAt(15));
    System.out.println("Dcomplex Scalar:");
    tt.printComplex(dcomplex_scalar);

    dcomplex_vector = (double []) (vec.elementAt(16));
    System.out.println("Dcomplex Vector:");
    tt.printComplexVec(dcomplex_vector);

    dcomplex_matrix = (double [][]) (vec.elementAt(17));
    System.out.println("Dcomplex Matrix:");
    tt.printComplexMat(dcomplex_matrix);

    byte_scalar = (byte) ((Integer) (vec.elementAt(18))).intValue();
    System.out.println("Byte Scalar:");
    System.out.println( byte_scalar );

    byte_vector = (byte []) (vec.elementAt(19));
    System.out.println("Byte Vector:");
    tt.printVec(byte_vector);

    byte_matrix = (byte [][]) (vec.elementAt(20));
    System.out.println("Byte Matrix:");
    tt.printMat(byte_matrix);
    
    // That's it.
  }
}
