/**
* TotaltestExampleColMajor
*
* This program illustrates how to initialize and retrieve every 
* data type that NetSolve currently supports except for the
* User Provided Function.  The 'totaltest' problem takes one item
* of every data type and increments each element by one.  In summary,
* we initialize all the variables, submit the data, and then
* print the results.
*
* @version 0.0, 24 Sep 1997
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

import java.util.Vector;

public class TotaltestExampleColMajor extends Example {
  public static void main (String [] args) {
    TotaltestExampleColMajor tt = new TotaltestExampleColMajor();
    NetsolveSession ns;
    Netsolve n;
    Vector vec;
    int i,j;

    int int_scalar = 1;
    int [] int_vector = {1,2,3,4};
    int [][] int_matrix = {
      {1,5},
      {2,6},
      {3,7},
      {4,8}
    };

    char char_scalar = 'A';
    char [] char_vector = {'A', 'B', 'C', 'D'};
    char [][] char_matrix = {    
      {'A', 'E', 'I', 'M'},
      {'B', 'F', 'J', 'N'}
    };

    // We must cast the constants to float to make the compiler happy
    float float_scalar = (float) 1.1;
    float [] float_vector = {(float) 1.1, (float) 2.2, (float) 3.3, (float) 4.4};
    float [][] float_matrix = {  
      {(float) 1.4, (float) 3.3, (float) 5.2, (float) 7.1},
      {(float) 2.5, (float) 4.6, (float) 6.7, (float) 8.8}
    };

    double double_scalar = 1.1;
    double [] double_vector = {1.1, 2.2, 3.3, 4.4};
    double [][] double_matrix = {  
      {1.1},
      {2.2},
      {3.3}
    };

    // *** NOTICE *** Complex scalars do not have their own data type.
    //                They are sent as an array, the first element being
    //                the real part and the second element the imaginary.
    //
    // No explicit casts needed here since I didn't use floating point 
    // constants.
    float [] scomplex_scalar = {1,0};
    float [] scomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    float [][] scomplex_matrix = {
      {1,16 , 5,12 , 9,8  , 13,4 },
      {2,15 , 6,11 , 10,7 , 14,3 },
      {3,14 , 7,10 , 11,6 , 15,2}
    };

    // Double precision complex numbers are initialized the same way
    // as single precision.
    double [] dcomplex_scalar = {1,0};
    double [] dcomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    double [][] dcomplex_matrix = {
      {1,0 , 5,0},
      {2,0 , 6,0},
      {3,0 , 7,0},
      {4,0 , 8,0}
    };

    byte byte_scalar = 1;
    byte [] byte_vector = {1,2,3,4};
    byte [][] byte_matrix = {
      {1,3,5,7},
      {2,4,6,8}
    };

    // As usual, we initialize the variables that will be set inside
    // the try/catch block since the compiler complains otherwise.
    ns = null;
    n = null;
    vec = null;

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);       // Specify 'session'
      n.setProblem("totaltest");  // Specify the problem to solve
      n.setMajor("col");          // Specify column-major order

      n.pushArg(int_scalar);      // Push all the args
      n.pushArg(int_vector);
      n.pushArg(int_matrix);
      n.pushArg(char_scalar);
      n.pushArg(char_vector);
      n.pushArg(char_matrix);
      n.pushArg(float_scalar);
      n.pushArg(float_vector);
      n.pushArg(float_matrix);
      n.pushArg(double_scalar);
      n.pushArg(double_vector);
      n.pushArg(double_matrix);
      n.pushArg(scomplex_scalar);
      n.pushArg(scomplex_vector);
      n.pushArg(scomplex_matrix);
      n.pushArg(dcomplex_scalar);
      n.pushArg(dcomplex_vector);
      n.pushArg(dcomplex_matrix);
      n.pushArg(byte_scalar);      // Push all the args
      n.pushArg(byte_vector);
      n.pushArg(byte_matrix);
      n.submitProb();           // Submit this problem
      vec = n.getOutput();
    }catch(NetSolveException e) {
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Now that we have the results, we can print all this
    // data out.  Since elementAt() returns Object, we must
    // add in all the explicit casts.  This means that you
    // have to know ahead of time what kind of data the
    // problem is going to return so that you know what
    // type to cast it as.

    int_scalar = ((Integer) (vec.elementAt(0))).intValue();
    System.out.println("Integer Scalar:");
    System.out.println( int_scalar );

    int_vector = (int []) (vec.elementAt(1));
    System.out.println("Integer Vector:");
    tt.printVec(int_vector);

    int_matrix = (int [][]) (vec.elementAt(2));
    System.out.println("Integer Matrix:");
    tt.printMatColMajor(int_matrix);
    
    char_scalar = ((Character) (vec.elementAt(3))).charValue();
    System.out.println("Character Scalar:");
    System.out.println( char_scalar );

    char_vector = (char []) (vec.elementAt(4));
    System.out.println("Character Vector:");
    tt.printVec(char_vector);

    char_matrix = (char [][]) (vec.elementAt(5));
    System.out.println("Character Matrix:");
    tt.printMatColMajor(char_matrix);
    
    float_scalar = ((Float) (vec.elementAt(6))).floatValue();
    System.out.println("Float Scalar:");
    System.out.println( float_scalar );

    float_vector = (float []) (vec.elementAt(7));
    System.out.println("Float Vector:");
    tt.printVec(float_vector);

    float_matrix = (float [][]) (vec.elementAt(8));
    System.out.println("Float Matrix:");
    tt.printMatColMajor(float_matrix);
    
    double_scalar = ((Double) (vec.elementAt(9))).doubleValue();
    System.out.println("Double Scalar:");
    System.out.println( double_scalar );

    double_vector = (double []) (vec.elementAt(10));
    System.out.println("Double Vector:");
    tt.printVec(double_vector);

    double_matrix = (double [][]) (vec.elementAt(11));
    System.out.println("Double Matrix:");
    tt.printMatColMajor(double_matrix);
    
    scomplex_scalar = (float []) (vec.elementAt(12));
    System.out.println("Scomplex Scalar:");
    tt.printComplex(scomplex_scalar);

    scomplex_vector = (float []) (vec.elementAt(13));
    System.out.println("Scomplex Vector:");
    tt.printComplexVec(scomplex_vector);

    scomplex_matrix = (float [][]) (vec.elementAt(14));
    System.out.println("Scomplex Matrix:");
    tt.printComplexMatColMajor(scomplex_matrix);
    
    dcomplex_scalar = (double []) (vec.elementAt(15));
    System.out.println("Dcomplex Scalar:");
    tt.printComplex(dcomplex_scalar);

    dcomplex_vector = (double []) (vec.elementAt(16));
    System.out.println("Dcomplex Vector:");
    tt.printComplexVec(dcomplex_vector);

    dcomplex_matrix = (double [][]) (vec.elementAt(17));
    System.out.println("Dcomplex Matrix:");
    tt.printComplexMatColMajor(dcomplex_matrix);

    byte_scalar = (byte) ((Integer) (vec.elementAt(18))).intValue();
    System.out.println("Byte Scalar:");
    System.out.println( byte_scalar );

    byte_vector = (byte []) (vec.elementAt(19));
    System.out.println("Byte Vector:");
    tt.printVec(byte_vector);

    byte_matrix = (byte [][]) (vec.elementAt(20));
    System.out.println("Byte Matrix:");
    tt.printMatColMajor(byte_matrix);
    
    // That's it.
  }
}
