/**
* ASCII_Example
*
* This program illustrates how to submit a problem which requires ASCII
* text via the NetSolve Java API.  The problem submission procedure is 
* essentially the same for ASCII and other data items such as vectors.
*
* @version 0.1, 17 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
* @see GlobalDefs
**/

public class ASCII_Example {
  public static void main (String [] args) {
    NetsolveSession ns       = null;
    NetSolveObject [] output = null;
    Netsolve n               = null;

    String text = "here is an\nexample of\nsending\nsome text\n";

    NetSolveASCII answer;

    System.out.println("Before, text is: ");
    System.out.println(text);

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);      // Specify 'session'
      n.setProblem("filetest");  // Specify the problem to solve

      n.pushArg(text);           // Pass the text to 'filetest'

      n.submitProbNB();          // Submit this problem

      n.waitFor();
      output = n.getOutputObject();
    }catch (NetSolveException e) {
      System.out.println(e.getMessage());
      System.exit(1);
    }

    // Check to see that we got an answer
    if(output == null) {
      System.out.println("NO answer");
    } else {
      answer = (NetSolveASCII) output[0];
      System.out.println("After, text is: ");
      answer.printData();
    }
  }
}
