/**
* QSortExample
*
* This class illustrates a simple example of using the
* NetSolve Java API.  An array of integers is initialized, 
* sorted, and the resulting array is printed to the screen.
*
* @version 0.1, 17 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

public class QSortExample {
  public static void main (String [] args) {
    NetsolveSession ns;
    Netsolve n;
    int [] blah = {765,653,534,67,212,987,43,8,543,8357,2};
    int i;
    NetSolveIntVector unsorted;
    NetSolveIntVector sorted;
    NetSolveObject [] out;

    // Since these variables are all set within the 'try'
    // block, the compiler can't confirm that they are 
    // initialized.  To avoid the subsequent complaints from
    // the compiler, we initialize them to null here.

    n = null;
    ns = null;
    out = null;
    unsorted = null; 

    // Almost all of the methods in Netsolve and NetsolveSession
    // throw NetSolveException upon some error condition, so it's
    // easier to just enclose the whole transaction in a try-catch
    // block than to try each one individually (although that is
    // possible, too).

    try {
      unsorted = new NetSolveIntVector(blah);

      // First, we create a new session, specifying the hostname
      // of the agent we want to contact.
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      // To initialize a new Netsolve object, pass in the session
      // object.
      n = new Netsolve(ns);

      // Specify which problem to solve (no need for the whole
      // pathname of the problem).
      n.setProblem("iqsort");

      // There's only one argument to qsort, so push it and submit
      // this problem to be solved.
      n.pushArg(blah);

      n.submitProb();    

      // submitProb is the blocking call to NetSolve, so by the
      // time we reach this point, the problem has been solved
      // and we can access the data through the getOutputObject() 
      // method.  Note that the data is returned in an array, each 
      // element containing one of the output items.  See below.
      out = n.getOutputObject();

    }catch(NetSolveException e) {
      // An error has been detected sometime during the problem 
      // specification, submission, or solving.  Here we just print
      // the error message and quit, however you don't necessarily
      // have to exit the program, depending on your application.
 
      System.err.println(e.getMessage());
      System.exit(1);
    }
   
    // Now if everything went well, we can get the output item
    // from the array.  Since the array is of type NetSolveObject,
    // we must cast the return value to the appropriate type.

    sorted = (NetSolveIntVector) out[0];

    // Print the results to verify that it worked.

    System.out.println("Unsorted:");
    unsorted.printData();

    System.out.println("Sorted:");
    sorted.printData();
  }
}
