/**
* TotaltestExample
*
* This program illustrates how to initialize and retrieve every 
* data type that NetSolve currently supports except for the
* User Provided Function.  The 'totaltest' problem takes one item
* of every data type and increments each element by one.  In summary,
* we initialize all the variables, submit the data, and then
* print the results.
*
* @version 0.1, 17 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

public class TotaltestExample {
  public static void main (String [] args) {
    NetsolveSession ns;
    Netsolve n;
    NetSolveObject [] obj;
    int i,j;

    int int_scalar = 1;
    int [] int_vector = {1,2,3,4};
    int [][] int_matrix = {
      {1,2,3,4},
      {5,6,7,8},
      {9,10,11,12},
      {13,14,15,16}
    };

    NetSolveIntScalar intOutputScalar;
    NetSolveIntVector intOutputVector;
    NetSolveIntMatrix intOutputMatrix;

    char char_scalar = 'A';
    char [] char_vector = {'A', 'B', 'C', 'D'};
    char [][] char_matrix = {    
      {'A', 'B', 'C', 'D'},
      {'E', 'F', 'G', 'H'},
      {'I', 'J', 'K', 'L'},
      {'M', 'N', 'O', 'P'}
    };

    NetSolveCharScalar charOutputScalar;
    NetSolveCharVector charOutputVector;
    NetSolveCharMatrix charOutputMatrix;

    // We must cast the constants to float to make the compiler happy
    float float_scalar = (float) 1.1;
    float [] float_vector = {(float) 1.1, (float) 2.2, (float) 3.3, (float) 4.4};
    float [][] float_matrix = {  
      {(float) 1.1, (float)   2.2, (float)  3.3, (float)  4.4},
      {(float) 5.1,  (float)  6.2, (float)  7.3,  (float) 8.4},
      {(float) 9.1, (float)  10.2, (float) 11.3, (float) 12.4},
      {(float) 13.1, (float) 14.2, (float) 15.3, (float) 16.4}
    };

    NetSolveFloatScalar floatOutputScalar;
    NetSolveFloatVector floatOutputVector;
    NetSolveFloatMatrix floatOutputMatrix;

    double double_scalar = 1.1;
    double [] double_vector = {1.1, 2.2, 3.3, 4.4};
    double [][] double_matrix = {  
      {1.1,   2.2,  3.3,  4.4},
      {5.1,   6.2,  7.3,  8.4},
      {9.1,  10.2, 11.3, 12.4},
      {13.1, 14.2, 15.3, 16.4}
    };

    NetSolveDoubleScalar doubleOutputScalar;
    NetSolveDoubleVector doubleOutputVector;
    NetSolveDoubleMatrix doubleOutputMatrix;

    // *** NOTICE *** Complex scalars do not have their own data type.
    //                They are sent as an array, the first element being
    //                the real part and the second element the imaginary.
    //
    // No explicit casts needed here since I didn't use floating point 
    // constants.
    float [] scomplex_scalar = {1,0};
    float [] scomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    float [][] scomplex_matrix = {
      {1,0 , 2,0 , 3,0 , 4,0},
      {5,0 , 6,0 , 7,0 , 8,0},
      {9,0 , 10,0 , 11,0 , 12,0},
      {13,0 , 14,0 , 15,0 , 16,0},
    };

    NetSolveScomplexScalar scompOutputScalar;
    NetSolveScomplexVector scompOutputVector;
    NetSolveScomplexMatrix scompOutputMatrix;

    // Double precision complex numbers are initialized the same way
    // as single precision.
    double [] dcomplex_scalar = {1,0};
    double [] dcomplex_vector = {1,0 , 2,0 , 3,0 , 4,0};
    double [][] dcomplex_matrix = {
      {1,0 , 2,0 , 3,0 , 4,0},
      {5,0 , 6,0 , 7,0 , 8,0},
      {9,0 , 10,0 , 11,0 , 12,0},
      {13,0 , 14,0 , 15,0 , 16,0},
    };

    NetSolveDcomplexScalar dcompOutputScalar;
    NetSolveDcomplexVector dcompOutputVector;
    NetSolveDcomplexMatrix dcompOutputMatrix;

    byte byte_scalar = 1;
    byte [] byte_vector = {1,2,3,4};
    byte [][] byte_matrix = {
      {1,2,3,4},
      {5,6,7,8},
      {9,10,11,12},
      {13,14,15,16}
    };

    NetSolveByteScalar byteOutputScalar;
    NetSolveByteVector byteOutputVector;
    NetSolveByteMatrix byteOutputMatrix;

    // As usual, we initialize the variables that will be set inside
    // the try/catch block since the compiler complains if they aren't.
    ns = null;
    n = null;
    obj = null;

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);       // Specify 'session'
      n.setProblem("totaltest");  // Specify the problem to solve

      n.pushArg(int_scalar);      // Push all the args
      n.pushArg(int_vector);
      n.pushArg(int_matrix);
      n.pushArg(char_scalar);
      n.pushArg(char_vector);
      n.pushArg(char_matrix);
      n.pushArg(float_scalar);
      n.pushArg(float_vector);
      n.pushArg(float_matrix);
      n.pushArg(double_scalar);
      n.pushArg(double_vector);
      n.pushArg(double_matrix);
      n.pushArg(scomplex_scalar);
      n.pushArg(scomplex_vector);
      n.pushArg(scomplex_matrix);
      n.pushArg(dcomplex_scalar);
      n.pushArg(dcomplex_vector);
      n.pushArg(dcomplex_matrix);
      n.pushArg(byte_scalar);
      n.pushArg(byte_vector);
      n.pushArg(byte_matrix);
      n.submitProb();           // Submit this problem
      obj = n.getOutputObject();
    }catch(NetSolveException e) {
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Now that we have the results, we can print all this
    // data out.  Since the array is NetSolveObject, we must
    // add in all the explicit casts.  This means that you
    // have to know ahead of time what kind of data the
    // problem is going to return so that you know what
    // type to cast it as.

    intOutputScalar = (NetSolveIntScalar) obj[0];
    System.out.println("Integer Scalar:");
    intOutputScalar.printData();

    intOutputVector = (NetSolveIntVector) obj[1];
    System.out.println("Integer Vector:");
    intOutputVector.printData();

    intOutputMatrix = (NetSolveIntMatrix) obj[2];
    System.out.println("Integer Matrix:");
    intOutputMatrix.printData();
    
    charOutputScalar = (NetSolveCharScalar) obj[3];
    System.out.println("Character Scalar:");
    charOutputScalar.printData();

    charOutputVector = (NetSolveCharVector) obj[4];
    System.out.println("Character Vector:");
    charOutputVector.printData();

    charOutputMatrix = (NetSolveCharMatrix) obj[5];
    System.out.println("Character Matrix:");
    charOutputMatrix.printData();
    
    floatOutputScalar = (NetSolveFloatScalar) obj[6];
    System.out.println("Float Scalar:");
    floatOutputScalar.printData();

    floatOutputVector = (NetSolveFloatVector) obj[7];
    System.out.println("Float Vector:");
    floatOutputVector.printData();

    floatOutputMatrix = (NetSolveFloatMatrix) obj[8];
    System.out.println("Float Matrix:");
    floatOutputMatrix.printData();
    
    doubleOutputScalar = (NetSolveDoubleScalar) obj[9];
    System.out.println("Double Scalar:");
    doubleOutputScalar.printData();

    doubleOutputVector = (NetSolveDoubleVector) obj[10];
    System.out.println("Double Vector:");
    doubleOutputVector.printData();

    doubleOutputMatrix = (NetSolveDoubleMatrix) obj[11];
    System.out.println("Double Matrix:");
    doubleOutputMatrix.printData();
    
    scompOutputScalar = (NetSolveScomplexScalar) obj[12];
    System.out.println("Scomplex Scalar:");
    scompOutputScalar.printData();

    scompOutputVector = (NetSolveScomplexVector) obj[13];
    System.out.println("Scomplex Vector:");
    scompOutputVector.printData();

    scompOutputMatrix = (NetSolveScomplexMatrix) obj[14];
    System.out.println("Scomplex Matrix:");
    scompOutputMatrix.printData();
    
    dcompOutputScalar = (NetSolveDcomplexScalar) obj[15];
    System.out.println("Dcomplex Scalar:");
    dcompOutputScalar.printData();

    dcompOutputVector = (NetSolveDcomplexVector) obj[16];
    System.out.println("Dcomplex Vector:");
    dcompOutputVector.printData();

    dcompOutputMatrix = (NetSolveDcomplexMatrix) obj[17];
    System.out.println("Dcomplex Matrix:");
    dcompOutputMatrix.printData();

    byteOutputScalar = (NetSolveByteScalar) obj[18];
    System.out.println("Byte Scalar:");
    byteOutputScalar.printData();

    byteOutputVector = (NetSolveByteVector) obj[19];
    System.out.println("Byte Vector:");
    byteOutputVector.printData();

    byteOutputMatrix = (NetSolveByteMatrix) obj[20];
    System.out.println("Byte Matrix:");
    byteOutputMatrix.printData();
    
    // That's it.
  }
}
