/****************************************************************/
/*      global.h                                                */
/*      Version 0.0                      Henri Casanova         */
/****************************************************************/

/*
 * Debugging information
 */

#ifdef DEBUG
#ifndef VIEW
#define VIEW 1
#endif
#endif

/*
 *  Included files
 */

#ifdef THREADS
#include <pthread.h>
#define THREAD_ERROR 2
#define THREAD_RUNNING 1
#define THREAD_DONE    0
#endif /* THREADS */

#ifdef PLAT_TITN
#include <bsd/sys/types.h>
#endif
#include <sys/types.h>
#if defined(PLAT_RS6K) || defined(PLAT_POWER4)
#include <sys/select.h>
#endif
#ifdef PLAT_RS6K
#include <sys/socketvar.h>
#endif
#include <signal.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdio.h>
#include <sys/time.h>
#ifndef NOWAIT3
#include <sys/resource.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/stat.h>
#include <stdlib.h>
#if defined(PLAT_SUN4SOL2) || defined(PLAT_T3E)
#include <fcntl.h>
#endif
#ifdef PLAT_NEXT
#include <sys/file.h>
#else
#include <sys/file.h>
#include <unistd.h>
#endif

#include "bfunc.h"

/* some simple macros for the LLD computation */
#ifndef MIN
#define MIN(a,b) (a)<(b) ? (a):(b)
#endif
#ifndef MAX
#define MAX(a,b) (a)>(b) ? (a):(b)
#endif


#define LEN_RETURN_LIST 200 /* Number of possible servers 
                               returned by the local server */
#define FILE_LENGTH 256     /* Max length of problem code   */
#define CODE_LENGTH 20      /* Max length of problem code   */
#define PATH_LENGTH 256     /* Max length of problem path   */
#define DESC_LENGTH 4096    /* Max length of description    */
#define MAX_HOST_NAME 256   /* host name                    */
#define ARRAY_MAX 5000000
#define MAX_STRING 4096


/*
 * IP-addresses nice print
 */

#define BYTE(x,y) (((unsigned char *)x)[(y)]&0xff)

/* Host and Server Status */

#define UP               1   /* Host is up        */
#define DOWN             0   /* Host is down      */

#define RUNNING          1   /* Server running     */
#define STOPPED          0   /* Server Stopped     */
#define FAILED           -1  /* Running but failed */



/*
 * Different types of client
 */

#define MATLAB	  1
#define C_PROGRAM 2
#define F_PROGRAM 3
#define JAVA      4

/*
 * Port definition
 */

#define SV_PORT            2222
#define AG_PORT            2223
#define HBM_INTERFACE_PORT 2227

#define MAXCONN 7

/* 
 * Different messages identifier
 */

#define OK               1

#define INVALID_CODE     -1
#define NO_SUCH_PB       -2
#define BAD_INPUT        -3
#define BAD_OUTPUT       -4
#define BAD_UPF          -5
#define DM_MISMATCH      -6
#define BAD_COMPLEXITY   -7
#define NO_SUCH_FORMAT   -8
#define NOT_AUTHORIZED   -9
#define TOKENS_OVERFLOW  -10
#define CORRECT_PB       0
#define PROBLEM_REQUEST	 1
#define PROBLEM_SOLVE  	 2
#define FAILURE_REPORT   3
#define I_AM_NEW         4
#define I_EXIST          5
#define I_DIE            6
#define YOU_ARE_NEW      7
#define WRONG_PROBLEM    8
#define PB_REFUSED       9
#define SV_FAILURE       10
#define PB_ACCEPTED      11
#define PB_NO_SOL        12
#define SOLUTION         13
#define WORKLOAD         14
#define DISTANCE_REPORT  15
#define DISTANCE_REQUEST 16
#define CLIENT_F_REPORT  17
#define INFO_REQUEST     18
#define END_COMP         19
#define JOB_COMPLETED    20
#define ALIVE            21
#define ARE_YOU_ALIVE    22
#define DESC_REQUEST     23
#define IS_ALIVE         24
#define WHO_ARE_YOU      25
#define UPF_OK           26
#define UPF_FAILED       27
#define UPF_UNSAFE       28
#define PONG_LATENCY     29
#define PONG_BANDWIDTH   30
#define RESTRICTION_MISMATCH 31
#define POLL_CONTACT     100
#define POLL_LIST_PB     110
#define POLL_LIST_SV     120
#define POLL_GET_PB      130
#define POLL_GET_SV      140
#define POLL_STAT        150
#define POLL_KILL        160

/*
 * Data Types
 */
#define NETSOLVE_I        1 /* Integer        */
#define NETSOLVE_S        2 /* Simple         */
#define NETSOLVE_D        3 /* Double         */
#define NETSOLVE_C        4 /* Simple complex */
#define NETSOLVE_Z        5 /* Double complex */
#define NETSOLVE_CHAR     6 /* Unsigned char  */
#define NETSOLVE_EXTERNAL 7 /* UPF type       */
#define NETSOLVE_B        8 /* Byte (no XDR)  */
#define NETSOLVE_ASCII    9 /* ASCII FILE     */

/*
 * Objects
 */
#define NETSOLVE_MATRIX 1
#define NETSOLVE_VECTOR 2
#define NETSOLVE_SCALAR 3
#define NETSOLVE_UPF    4
#define NETSOLVE_FILE   5

/*
 * Row or Column
 */

#define COL_MAJOR 0
#define ROW_MAJOR 1

/*
 * Complex Numbers
 */

typedef struct {
  float r;
  float i;
} scomplex;

typedef struct {
  double r;
  double i;
} dcomplex;

/*
 * Protocols
 */
#define DATA_RAW 0
#define DATA_XDR 1

/* 
 * Results from the linear-algebra module
 */
 
#define FAILURE -1
#define BAD_DIMENSION -2
#define NO_SOLUTION   -3
#define BAD_VALUES    -4

#define LATENCY_TIMES 5
#define BANDWIDTH_TIME 1000000            /* (user-)time, in useconds */


/*
 * Data type sizes
 */

#define INTL    (sizeof(int)/sizeof(char))
#define LONGL   (sizeof(long)/sizeof(char))
#define FLOATL  (sizeof(float)/sizeof(char))
#define DOUBL   (sizeof(double)/sizeof(char))
#define SCOMPL  (sizeof(scomplex)/sizeof(char))
#define DCOMPL  (sizeof(dcomplex)/sizeof(char))
#define UCHARL  (sizeof(unsigned char)/sizeof(char))
#define CHARL   1

/*
 * Structure containing the sizes of the objects
 */

typedef struct InputSizes input_sizes;
struct InputSizes {
  int m; /* 1 for a scalar             */
  int n; /* 1 for a vector or a scalar */
  int l; /* Leading Dimension : only on the client side, only for matrices */
  int *d; /* descriptor for a matrix, only fo SCALAPACK server, it is a vector */
};

typedef struct OutputSizes output_sizes;
struct OutputSizes {
  int *m; /* 1 for a scalar             */
  int *n; /* 1 for a vector or a scalar */
  int l; /* Leading Dimension : only on the client side, only for matrices */
  int *d; /* descriptor for a matrix, only fo SCALAPACK server */
};

/* Communicator structure to be thread safe !! */
typedef struct{
  int current_mode;
  int current_desc;
  XDR current_xdr;
  FILE *current_stream;
}communicator;
