/*****************************************************************/
/*      netsolve.h                                               */
/*      Version 1.1                      Henri Casanova          */
/*****************************************************************/


#include "global.h"
#include "object.h"
#include "interface.h"
#include <sys/ioctl.h>
#ifdef PLAT_SUN4SOL2
#include <sys/filio.h>
#endif

#define WAIT 1
#define PROBE 2

#define IGNORE -1
#define INTEGER 1
#define POINTER 2

#define INPUT 1
#define OUTPUT 2

#define MAX_REQUEST 50
char upf_directory[256];
int Xmajor;

#ifdef __STDC__
void sigpipe_handler(int);
#else
void sigpipe_handler();
#endif

#ifdef THREADS

/****************/
/* Thread Stuff */
/****************/

/* Pthread Structure */
typedef struct {
  pthread_t thread_id;
  int thread_num;
  int thread_status;
  char *command_name;
  int NetSolve_error_code;
}thread_list;

/* Pthread argument */
typedef struct {
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input;
  void **data_output;
  int my_slot;
}thread_input;

thread_list *request[MAX_REQUEST];  /* List of threads */

pthread_mutex_t thread_mutex;

#endif /* THREADS */

#ifdef NOTHREADS

char Xcode[MAX_REQUEST][CODE_LENGTH];

unsigned int XIPaddr[MAX_REQUEST];
int Xserver_pid[MAX_REQUEST];
int  Xdesc[MAX_REQUEST];

problem_desc *Xpb_desc[MAX_REQUEST];

input_sizes *Xsizes_input[MAX_REQUEST];
output_sizes *Xsizes_output[MAX_REQUEST];

void **Xdata_input[MAX_REQUEST];
void **Xdata_output[MAX_REQUEST];


int request[MAX_REQUEST];

#endif /* NOTHREADS */

/***********************/
/* Function prototpyes */
/***********************/

#ifdef __STDC__
int parse_constants(problem_desc*, input_sizes*,
                    output_sizes*);
int parse_formulas(problem_desc*,input_sizes*,
                   output_sizes*,void**);

int netsldir(char*);
 
#ifdef ADD_
void fnetsldir_(char *);
#endif
#ifdef UPCASE
void FNETSLDIR(char *);
#endif
#ifdef NOCHANGE
void fnetsldir(char *);
#endif

int netslmajor(char*);

#ifdef ADD_
int fnetslmajor_(char*);
#endif
#ifdef UPCASE
int FNETSLMAJOR(char*);
#endif
#ifdef NOCHANGE
int fnetslmajor(char*);
#endif


void netslerr(int);

#ifdef ADD_
void fnetslerr_(int*);
#endif
#ifdef UPCASE
void FNETSLERR(int*);
#endif
#ifdef __STDC__
void fnetslerr(int*);
#endif

int netslpr(int);

#ifdef ADD_
void fnetslpr_(int*,int*);
#endif
#ifdef UPCASE
void FNETSLPR(int*,int*);
#endif
#ifdef NOCHANGE
void fnetslpr(int*,int*);
#endif


int netslwt(int);

#ifdef ADD_
void fnetslwt_(int*,int*);
#endif
#ifdef UPCASE
void FNETSLWT(int*,int*);
#endif
#ifdef NOCHANGE
void fnetslwt(int*,int*);
#endif

int netsl(char*,...);

#ifdef ADD_
void fnetsl_(char*,...);
#endif
#ifdef UPCASE
void FNETSL(char*,...);
#endif
#ifdef NOCHANGE
void fnetsl(char*,...);
#endif

int netslnb(char *,...);

#ifdef ADD_
void fnetslnb_(char *,...);
#endif
#ifdef UPCASE
void FNETSLNB(char *,...);
#endif
#ifdef NOCHANGE
void fnetslnb(char *,...);
#endif


int netsldirintern();
int mexnetslmajor(int);
int netslmajorintern(int);
int mexnetslmajorintern(int);
int nothread_get_result(int,int);
int nothread_solve(int,problem_desc*,input_sizes*,output_sizes*,void**,void**);
int thread_get_result(communicator*,char*,int,int,problem_desc*,output_sizes*,void**);
int thread_solve(problem_desc*,input_sizes*,output_sizes*,void**,void**);

int client_f_report(int,char*,unsigned int);
int job_completed(char*,unsigned int);
int parse_arg(char*,int*,int*,int*);
int eval(problem_desc*,input_sizes*,void**,char*);
int get_op_arg1_arg2(char*,char**,char**,char**);
int get_array_index(char*,char**,char**);
int uppercase(char*);
int get_members(char*,char**,char**,char**,char**);
void handler(int);
int netsolve_info(char*,problem_desc*);
int submit_job(char*,int*,char**,unsigned int*,int *,
               int*,int,int,int,int,problem_desc*);
int free_pb_desc(problem_desc*);
int arch_to_format(char*);
int is_number(char*);
int tread(int,char*,int);
communicator *init_send(int,int);
communicator *init_recv(int);
int end_message(communicator *);
int end_com(int);
int xdr_scomplex(XDR*,scomplex*);
int xdr_dcomplex(XDR*,dcomplex*);
int read_string(communicator*,char*);
int read_int(communicator*,int *);
int read_char_array(communicator*,char*,int);
int read_int_array(communicator*,int*,int);
int read_IPaddr(communicator*,unsigned int*);
int read_long(communicator*,long*);
int read_single_array(communicator*,float*,int);
int read_double_array(communicator*,double*,int);
int read_byte_array(communicator*,void*,int);
int write_byte_array(communicator*,void*,int);
int read_u_char_array(communicator*,unsigned char*,int);
int read_scomplex_array(communicator*,scomplex*,int);
int read_dcomplex_array(communicator*,dcomplex*,int);
int write_string(communicator*,char*);
int write_int(communicator*,int*);
int write_int_array(communicator*,int*,int);
int write_char_array(communicator*,char*,int);
int write_IPaddr(communicator*,unsigned int*);
int write_long(communicator*,long*);
int write_single_array(communicator*,float*,int);
int write_double_array(communicator*,double*,int);
int write_u_char_array(communicator*,unsigned char*,int);
int write_scomplex_array(communicator*,scomplex*,int);
int write_dcomplex_array(communicator*,dcomplex*,int);
int read_char_array_file(communicator*,int,int);
int read_int_array_file(communicator*,int,int);
int read_single_array_file(communicator*,int,int);
int read_double_array_file(communicator*,int,int);
int read_scomplex_array_file(communicator*,int,int);
int read_dcomplex_array_file(communicator*,int,int);
int write_char_array_file(communicator*,int,int);
int write_int_array_file(communicator*,int,int);
int write_single_array_file(communicator*,int,int);
int write_double_array_file(communicator*,int,int);
int write_scomplex_array_file(communicator*,int,int);
int write_dcomplex_array_file(communicator*,int,int);

int establish_socket(int*);
int connect_to_agent(char*,int);
int connect_to_server(char*,int);
int connect_to_socket(char*,unsigned int,int);


#else

int parse_constants();
int parse_formulas();

int netsldir();

#ifdef ADD_
void fnetsldir_();
#endif
#ifdef UPCASE
void FNETSLDIR();
#endif
#ifdef NOCHANGE
void fnetsldir();
#endif

int netslmajor();

#ifdef ADD_
int fnetslmajor_();
#endif
#ifdef UPCASE
int FNETSLMAJOR();
#endif
#ifdef NOCHANGE
int fnetslmajor();
#endif

void netslerr();

#ifdef ADD_
void fnetslerr_();
#endif
#ifdef UPCASE
void FNETSLERR();
#endif
#ifdef NOCHANGE
void fnetslerr();
#endif

int netslpr();

#ifdef ADD_
void fnetslpr_();
#endif
#ifdef UPCASE
void FNETSLPR();
#endif
#ifdef NOCHANGE
void fnetslpr();
#endif

int netslwt();

#ifdef ADD_
void fnetslwt_();
#endif
#ifdef UPCASE
void FNETSLWT();
#endif
#ifdef NOCHANGE
void fnetslwt();
#endif

int netsl();

#ifdef ADD_
void fnetsl_();
#endif
#ifdef UPCASE
void FNETSLWT();
#endif
#ifdef NOCHANGE
void fnetslwt();
#endif

int netslnb();

#ifdef ADD_
void fnetslnb_();
#endif
#ifdef UPCASE
void FNETSLNB();
#endif
#ifdef NOCHANGE
void fnetslnb();
#endif

int netsldirintern();
int mexnetslmajor();
int mexnetslmajorintern();

int nothread_get_result();
int nothread_solve();
int thread_get_result();
int thread_solve();

int client_f_report();
int job_completed();
int parse_arg();
int eval();
int get_op_arg1_arg2();
int get_array_index();
int uppercase();
int get_members();
void handler();
int netsolve_info();
int submit_job();

int free_pb_desc();
int arch_to_format();
int is_number();
int tread();
communicator *init_send();
communicator *init_recv();
int end_message();
int end_com();
int xdr_scomplex();
int xdr_dcomplex();
int read_string();
int read_int();
int read_char_array();
int read_int_array();
int read_IPaddr();
int read_long();
int read_single_array();
int read_double_array();
int read_u_char_array();
int read_scomplex_array();
int read_dcomplex_array();
int write_string();
int write_int();
int write_int_array();
int write_char_array();
int write_IPaddr();
int write_long();
int write_single_array();
int write_double_array();
int write_u_char_array();
int write_scomplex_array();
int write_dcomplex_array();
int read_char_array_file();
int read_int_array_file();
int read_single_array_file();
int read_double_array_file();
int read_scomplex_array_file();
int read_dcomplex_array_file();
int write_char_array_file();
int write_int_array_file();
int write_single_array_file();
int write_double_array_file();
int write_scomplex_array_file();
int write_dcomplex_array_file();

int establish_socket();
int connect_to_agent();
int connect_to_server();
int connect_to_socket();

#endif
