/*****************************************************************/
/*      object.h                                                 */
/*      Version 0.0                      Henri Casanova          */
/*****************************************************************/

#define OBJ_LENGTH 4096 /* Max length of description */

typedef struct Constant constant; /* list of constants descriptions */
struct Constant
{
  char *mnemonic; /* Mnemonic                                       */
  int value;      /* Actual value of the constant                   */
  constant *next; /* Pointer to the next element of the linked list */
};

typedef struct Formula formula; /* list of formula descriptions */
struct Formula
{
  char *mnemonic;   /* Mnemonic                                       */
  char *expression; /* Actual value of the constant                   */
  formula *next;    /* Pointer to the next element of the linked list */
};


typedef struct FormatSpec format_spec; /* List of format */
struct FormatSpec
{
  int nb_args; /* number of args required */
  char **args_desc; /* xxx,xxx,xxx */
  int nb_constants;
  constant *consts;    
  int nb_formulae;
  formula *forms;
};

typedef struct ObJect object; /* List of objects */
struct ObJect
{
  int data_type; /* NETSOLVE_I : integer
                    NETSOLVE_S : single precision real
                    NETSOLVE_D : double precision real
                    NETSOLVE_C : single precision real
                    NETSOLVE_Z : double precision real */

  int object_type; /* NETSOLVE_MATRIX
                      NETSOLVE_VECTOR
                      NETSOLVE_SCALAR
                      NETSOLVE_UPF    */

  char object_desc[OBJ_LENGTH]; /* Textual description of the object */
};

typedef struct MatlabMerge matlab_merge;
struct MatlabMerge{
  int output1;
  int output2;
  matlab_merge *next;
};

#define LANG_C       0
#define LANG_FORTRAN 1

typedef struct UpfStruct upfstruct;
struct UpfStruct{
  int lang; /* LANG_C or LANG_F */
};

typedef struct ProblemDesc problem_desc; /* Problem Descriptor */
struct ProblemDesc {
  char code[CODE_LENGTH];  /* Code                */
  char path[PATH_LENGTH];  /* Path                */
  char desc[DESC_LENGTH];  /* Textual description */
  char file[FILE_LENGTH];  /* file to fork()      */

  int nb_obj_input;    /*                     */
  int nb_obj_output;   /*   Input and Output  */
  object *input_obj;   /*       Objects       */
  object *output_obj;  /*                     */

  int nb_matlab;
  matlab_merge *matlab;/* Merging ?           */

  char *complexity;    /* Complexity of the algorithm */

  int nb_formats;      /*    Formats          */
  format_spec *formats;/*                     */
};
