/*****************************************************************/
/*      calling_sequence.c                                       */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  parse_constants()                                            */
/*  parse_formulas()                                             */
/*****************************************************************/

#include "netsolve.h"

/*
 * int parse_constants()
 */

#ifdef __STDC__
int parse_constants(problem_desc *pb_desc,
                    input_sizes *sizes_input,
                    output_sizes *sizes_output)
#else
int parse_constants(pb_desc,sizes_input,sizes_output)
problem_desc *pb_desc;
input_sizes *sizes_input;
output_sizes *sizes_output;
#endif
{
  constant *current_constant;

  /****************************/
  /* Processing the constants */
  /****************************/

  current_constant = pb_desc->formats[0].consts;
  while(current_constant != NULL)
  {
    int arg_index;
    arg_index = atoi((current_constant->mnemonic)+2);
    switch((current_constant->mnemonic)[0])
    {

      case 'O' :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
                "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
      case 'I' :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
                "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
      case 'm' :
      {
         if (((current_constant->mnemonic)[1]) == 'O')
         {
            free_pb_desc(pb_desc);
            sprintf(error_message,"%s \n\t\t[%s,%d]",
                "Internal NetSolve error",__FILE__,__LINE__);
            return NetSolveFailure;
         }
          /* From here, the object is in input */
          sizes_input[arg_index].m = current_constant->value;
      break;
      }
      case 'n' :
      {
        if (((current_constant->mnemonic)[1]) == 'O')
        {
          free_pb_desc(pb_desc);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
                "Internal NetSolve error",__FILE__,__LINE__);
          return NetSolveFailure;
        }
        /* From here, the object is in input */
        sizes_input[arg_index].n = current_constant->value;
        break;
      }
      case 'l' :
      {
        if (((current_constant->mnemonic)[1]) == 'O')
        {
          free_pb_desc(pb_desc);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
                "Internal NetSolve error",__FILE__,__LINE__);
          return NetSolveFailure;
        }
        /* From here, the object is in input */
        sizes_input[arg_index].l = current_constant->value;
        break;
      }
      default :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
    }
    current_constant = current_constant->next;
  }
  return 1;
}


#ifdef __STDC__
int parse_formulas(problem_desc *pb_desc,
                   input_sizes *sizes_input,
                   output_sizes *sizes_output,
                   void **data_input)
#else
int parse_formulas(pb_desc, sizes_input,sizes_output,data_input)
problem_desc *pb_desc;
input_sizes *sizes_input;
output_sizes *sizes_output;
void **data_input;
#endif
{
  formula *current_form;

  current_form = pb_desc->formats[0].forms;
  while(current_form != NULL)
  {
    int arg_index;
    arg_index = atoi((current_form->mnemonic)+2);
    switch((current_form->mnemonic)[0])
    {

      case 'O' :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
      case 'I' :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
      case 'm' :
      {
        if (((current_form->mnemonic)[1]) == 'O')
        {
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          free_pb_desc(pb_desc);
          return NetSolveFailure;
        }
        /* From here, the object is in input */
        sizes_input[arg_index].m = 
             eval(pb_desc,sizes_input,data_input,current_form->expression);
        if (sizes_input[arg_index].m <0)
        { 
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          free_pb_desc(pb_desc);
          return NetSolveFailure;
        }
        break;
      }
      case 'n' :
      {
        if (((current_form->mnemonic)[1]) == 'O')
        {
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          free_pb_desc(pb_desc);
          return NetSolveFailure;
        }
        /* From here, the object is in input */
        sizes_input[arg_index].n = eval(pb_desc,sizes_input,data_input,current_form->expression);
        if (sizes_input[arg_index].n < 0)
        {
          free_pb_desc(pb_desc);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          return NetSolveFailure;
        }
        break;
      }
      case 'l' :
      {
        if (((current_form->mnemonic)[1]) == 'O')
        {
          free_pb_desc(pb_desc);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          return NetSolveFailure;
        }
        /* From here, the object is in input */
        sizes_input[arg_index].l = eval(pb_desc,sizes_input,data_input,current_form->expression);
        if (sizes_input[arg_index].l < 0)
        {
          free_pb_desc(pb_desc);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Internal NetSolve error",__FILE__,__LINE__);
          return NetSolveFailure;
        }
        break;
      }
      default :
      {
        free_pb_desc(pb_desc);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Internal NetSolve error",__FILE__,__LINE__);
        return NetSolveFailure;
      }
    }
    current_form = current_form->next;
  }
  return 1;
}

