/*****************************************************************/
/*      fnetsl.c                                                 */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netsl()                                                      */
/*****************************************************************/

/*
 * netsl() : FORTRAN interface to gateway()
 */

#include "netsolve.h"
#include <stdio.h>
#ifdef  __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif


#ifdef __STDC__

#ifdef ADD_
void fnetsl_(char *code,...)
#endif
#ifdef UPCASE
void FNETSL(char *code,...)
#endif
#ifdef NOCHANGE
void fnetsl(char *code,...)
#endif

#else /* not STDC */
 
#ifdef ADD_
void fnetsl_(va_alist)
va_dcl
#endif
#ifdef UPCASE
void FNETSL(va_alist)
va_dcl
#endif
#ifdef NOCHANGE
void fnetsl(va_alist)
va_dcl
#endif

#endif /* STDC */
{
#ifndef __STDC__
  char *code;
#endif
  char *char_pointer[50];
  int index_char_pointer=0;
  int info,status;
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input, **data_output;
  char *argument;
  int *return_value;
  char tmp[CODE_LENGTH+1];
  char *real_code;

  /* The shadow arguments inserted by the FORTRAN    */
  /* to describe the length of the character strings */
  int shadow;
 
  int j;

  static already_been_here = 0;
  int i;
  va_list argptr;

  /*********************/
  /* Initialization    */
  /*********************/

  netslmajorintern(COL_MAJOR);

  /***********************/
  /* Getting the code    */
  /***********************/

#ifdef __STDC__
  va_start(argptr,code);
#else
  va_start(argptr);
  code = va_arg(argptr,char *);
#endif

  /**************************/
  /* Possibly pop something */
  /**************************/

#ifdef STACK_SCHEME_CRAY
  shadow = va_arg(argptr, int);
#endif

  /******************/
  /* Return value   */
  /******************/

  return_value = (int *)va_arg(argptr,int *);

  /*******************/
  /* Finding the '(' */
  /*******************/
 
  for (i=0;i<CODE_LENGTH+1;i++)
    if (code[i] == '(')
      break;
  if (i == CODE_LENGTH+1)
  {
    *return_value = NetSolveBadCode;
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return;
  }
  if (code[i+1] != ')')
  {
    *return_value = NetSolveBadCode;
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return;
  }
  strncpy(tmp,code,i);
  tmp[i] = '\0';
  real_code = tmp;

  /***********************/
  /* Getting the number  */
  /* of parameters       */
  /***********************/

  pb_desc = (problem_desc *)calloc(1,sizeof(problem_desc));
  status = netsolve_info(real_code,pb_desc);

  if (status <0)
  {
    free_pb_desc(pb_desc);
    /* the error message is already set */
    *return_value = status;
    return;
  }

  /************************/
  /* Allocating the space */
  /************************/

  sizes_input =
     (input_sizes *)calloc(pb_desc->nb_obj_input,sizeof(input_sizes));
  sizes_output =
     (output_sizes *)calloc(pb_desc->nb_obj_output,sizeof(output_sizes));
  data_input = (void **)calloc(pb_desc->nb_obj_input,sizeof(void *));
  data_output = (void **)calloc(pb_desc->nb_obj_output,sizeof(void *));

  /****************************/
  /* Processing the constants */
  /****************************/

  status = parse_constants(pb_desc,sizes_input,sizes_output);
  if (status != 1)
  {
    free_pb_desc(pb_desc);
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    /* The error message is already set */
    *return_value = status;
    return;
  }

  /*************************/
  /* Parsing the arguments */
  /* Using the format      */
  /*************************/

  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    int arg_type;
    int arg_i_o;
    int arg_index;
    void *arg_value_ptr;
    char *mnemonic;

    argument = pb_desc->formats[0].args_desc[i];

    parse_arg(argument,&arg_type,&arg_index,&arg_i_o);

    arg_value_ptr = (void *)va_arg(argptr,void *);

    mnemonic = (char *)strtok(argument," \t,");
    while(mnemonic != NULL)
    {
      
      parse_arg(mnemonic,&arg_type,&arg_index,&arg_i_o);
      switch(arg_i_o)
      {
        case IGNORE :
        {
          break;
        }
        case INPUT :
        {
          if (arg_type == INTEGER)
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_input[arg_index].m = *((int *)arg_value_ptr);
                break;
              }
              case 'n' :
              {
                sizes_input[arg_index].n = *((int *)arg_value_ptr);
                break;
              }
              case 'l' :
              {
                sizes_input[arg_index].l = *((int *)arg_value_ptr);
                break;
              }
            }
          }
          else /* POINTER */
          {
            data_input[arg_index] = arg_value_ptr;
#ifdef STACK_SCHEME_SUN
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_UPF)
              char_pointer[index_char_pointer++] = data_input[arg_index]; 
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_FILE)
              char_pointer[index_char_pointer++] = data_input[arg_index]; 
#endif

#ifdef      STACK_SCHEME_CRAY
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_UPF)
            {
              char *tmp;

              /* Get the length */
              shadow = va_arg(argptr,int);
              tmp = (char *)(data_input[arg_index]);
              data_input[arg_index] = (char *)calloc(shadow+1,sizeof(char));
              strncpy((char *)(data_input[arg_index]),tmp,shadow);
              ((char *)(data_input[arg_index]))[shadow] = '\0';
            }
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_FILE)
            {
              char *tmp;

              /* Get the length */
              shadow = va_arg(argptr,int);
              tmp = (char *)(data_input[arg_index]);
              data_input[arg_index] = (char *)calloc(shadow+1,sizeof(char));
              strncpy((char *)(data_input[arg_index]),tmp,shadow);
              ((char *)(data_input[arg_index]))[shadow] = '\0';
            }
#endif
          }
          break;
        }
        case OUTPUT :
        {
          if (arg_type == POINTER) 
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_output[arg_index].m = (int *)arg_value_ptr;
                break;
              }
              case 'n' :
              {
                sizes_output[arg_index].n = (int *)arg_value_ptr;
                break;
              }
              case 'O' :
              {
                data_output[arg_index] = arg_value_ptr;
#ifdef STACK_SCHEME_SUN
                if (pb_desc->output_obj[arg_index].object_type == NETSOLVE_FILE)
                  char_pointer[index_char_pointer++] = data_output[arg_index]; 
#endif

#ifdef STACK_SCHEME_CRAY
                if (pb_desc->output_obj[arg_index].object_type == NETSOLVE_FILE)
                {
                  char *tmp;

                  /* Get the length */
                  shadow = va_arg(argptr,int);
                  tmp = (char *)(data_output[arg_index]);
                  data_output[arg_index] = (char *)calloc(shadow+1,sizeof(char));
                  strncpy((char *)(data_output[arg_index]),tmp,shadow);
                  ((char *)(data_output[arg_index]))[shadow] = '\0';
                }
#endif
                break;
              }
            }
          }
          else /* INTEGER */
          {
            if (mnemonic[0] == 'l')
              sizes_output[arg_index].l = *((int *)arg_value_ptr);
          }
        }
      }
      mnemonic = (char *)strtok(NULL,"\t ,");
    }
  }
  /* Pop the stuff off the stack in a SUN_SCHEME */
#ifdef STACK_SCHEME_SUN
  /* Pop the length of the real_code */
  shadow = va_arg(argptr,int);
  if (shadow != strlen(real_code)+2)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Invalid number of arguments passed to netsl()",
      __FILE__,__LINE__);
    *return_value = NetSolveBadCallingSequence;
    return;
  }
  for (i=0;i<index_char_pointer;i++)
  {
      char *tmp;
      shadow = va_arg(argptr,int);
      tmp = (char *)(char_pointer[i]);
      char_pointer[i] = (char *)calloc(shadow+1,sizeof(char));
      strncpy((char *)(char_pointer[i]),tmp,shadow);
  }
#endif

  /******************************/
  /* Processing the expressions */
  /******************************/


  status = parse_formulas(pb_desc,sizes_input,sizes_output,data_input);
  if (status != 1)
  {
    /* The Error message is already set */
    *return_value =  NetSolveFailure;
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    free_pb_desc(pb_desc);
    return;
  }

  /*******************************/
  /* Calling solve()             */
  /*******************************/

  status = thread_solve(pb_desc,sizes_input,sizes_output,
           data_input,data_output);

  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    if (sizes_output[i].m != NULL)
      free(sizes_output[i].m);
    if (sizes_output[i].n != NULL)
      free(sizes_output[i].n);
  }
  free_pb_desc(pb_desc);
  free(sizes_input);
  free(sizes_output);
  free(data_input);
  free(data_output);
  *(return_value) = status;
  return;
}
