/*****************************************************************/
/*      fnetslnb.c                                               */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netslnb()                                                    */
/*****************************************************************/

/*
 * netslnb() : FORTRAN interface to gateway()
 */

#include "netsolve.h"
#include <stdio.h>
#ifdef  __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif


#ifdef __STDC__

#ifdef ADD_
void fnetslnb_(char *code,...)
#endif
#ifdef UPCASE
void FNETSLNB(char *code,...)
#endif
#ifdef NOCHANGE
void fnetslnb(char *code,...)
#endif

#else /* not STDC */
 
#ifdef ADD_
void fnetslnb_(va_alist)
va_dcl
#endif
#ifdef UPCASE
void FNETSLNB(va_alist)
va_dcl
#endif
#ifdef NOCHANGE
void fnetslnb(va_alist)
va_dcl
#endif

#endif /* STDC */
{
#ifndef __STDC__
  char *code;
#endif
  char *char_pointer[50];
  int index_char_pointer = 0;
  int status;
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input, **data_output;

  char tmp[CODE_LENGTH+1];
  char *argument;
  int *return_value;
  int slot;
#ifdef THREADS
  thread_input *arguments=NULL;
  pthread_attr_t custom_sched_attr;
#endif

  char *real_code;

  void *thread_do();

  /* The shadow arguments inserted by the FORTRAN    */
  /* to describe the length of the character strings */
  int shadow;

  static already_been_here = 0;
  int i;

  va_list argptr;

  /************************/
  /* Initialization       */
  /************************/

  netslmajorintern(COL_MAJOR);

  if (already_been_here == 0)
  {
#ifdef THREADS
    pthread_mutex_init(&thread_mutex,NULL);
#endif
    for (i = 0;i<MAX_REQUEST;i++)
      request[i] = NULL;
    already_been_here = 1;
  }


  /************************/
  /* Getting the code     */
  /************************/

#ifdef __STDC__
  va_start(argptr,code);
#else
  va_start(argptr);
  code = va_arg(argptr,char *);
#endif

  /**************************/
  /* Possibly pop something */
  /**************************/

#ifdef STACK_SCHEME_CRAY
  shadow = va_arg(argptr, int);
#endif

  /******************/
  /* Return value   */
  /******************/

  return_value = (int *)va_arg(argptr,int *);

  /*******************/
  /* Finding the '(' */
  /*******************/

  for (i=0;i<CODE_LENGTH+1;i++)
  {
    if (code[i] == '(')
      break;
  }
  if (i == CODE_LENGTH+1)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    *return_value = NetSolveBadCode;
    return;
  }
  if (code[i+1] != ')')
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    *return_value = NetSolveBadCode;
    return;
  }
  strncpy(tmp,code,i);
  tmp[i] = '\0';
  real_code = tmp;

  /***********************/
  /* Getting the number  */
  /* of parameters       */
  /***********************/

  pb_desc = (problem_desc *)calloc(1,sizeof(problem_desc));
  status = netsolve_info(real_code,pb_desc);

  if (status <0)
  {
    /* the error message is already set */
    *return_value = status;
    free_pb_desc(pb_desc);
    return;
  }

  /************************/
  /* Allocating the space */
  /************************/

  sizes_input =
     (input_sizes *)calloc(pb_desc->nb_obj_input,sizeof(input_sizes));
  sizes_output =
     (output_sizes *)calloc(pb_desc->nb_obj_output,sizeof(output_sizes));
  data_input = (void **)calloc(pb_desc->nb_obj_input,sizeof(void *));
  data_output = (void **)calloc(pb_desc->nb_obj_output,sizeof(void *));

  /****************************/
  /* Processing the constants */
  /****************************/

  status = parse_constants(pb_desc,sizes_input,sizes_output);
  if (status != 1)
  {
    free_pb_desc(pb_desc);
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    /* The error message is already set */
    *return_value = status;
    return;
  }

  /*************************/
  /* Parsing the arguments */
  /* Using the format      */
  /*************************/

  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    int arg_type;
    int arg_i_o;
    int arg_index;
    void *arg_value_ptr;
    char *mnemonic;

    argument = pb_desc->formats[0].args_desc[i];

    parse_arg(argument,&arg_type,&arg_index,&arg_i_o);

    arg_value_ptr = (void *)va_arg(argptr,void *);

    mnemonic = (char *)strtok(argument," \t,");
    while(mnemonic != NULL)
    {
      parse_arg(mnemonic,&arg_type,&arg_index,&arg_i_o);
      switch(arg_i_o)
      {
        case IGNORE :
        {
          break;
        }
        case INPUT :
        {
          if (arg_type == INTEGER)
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_input[arg_index].m = *((int *)arg_value_ptr);
                break;
              }
              case 'n' :
              {
                sizes_input[arg_index].n = *((int *)arg_value_ptr);
                break;
              }
              case 'l' :
              {
                sizes_input[arg_index].l = *((int *)arg_value_ptr);
                break;
              }
            }
          }
          else /* POINTER */
          {
            data_input[arg_index] = arg_value_ptr;
#ifdef STACK_SCHEME_SUN
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_UPF)
              char_pointer[index_char_pointer++] = data_input[arg_index];
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_FILE)
              char_pointer[index_char_pointer++] = data_input[arg_index];
#endif

#ifdef      STACK_SCHEME_CRAY
            if (pb_desc->input_obj[arg_index].object_type == NETSOLVE_UPF)
            {
              char *tmp;

              /* Get the length */
              shadow = va_arg(argptr,int);
              tmp = (char *)(data_input[arg_index]);
              data_input[arg_index] = (char *)calloc(shadow+1,sizeof(char));
              strncpy((char *)(data_input[arg_index]),tmp,shadow);
              ((char *)(data_input[arg_index]))[shadow] = '\0';
            }
#endif
          }
          break;
        }
        case OUTPUT :
        {
          if (arg_type == POINTER)
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_output[arg_index].m = (int *)arg_value_ptr;
                break;
              }
              case 'n' :
              {
                sizes_output[arg_index].n = (int *)arg_value_ptr;
                break;
              }
              case 'O' :
              {
                data_output[arg_index] = arg_value_ptr;
#ifdef STACK_SCHEME_SUN
                if (pb_desc->output_obj[arg_index].object_type == NETSOLVE_FILE)
                  char_pointer[index_char_pointer++] = data_output[arg_index];
#endif

#ifdef STACK_SCHEME_CRAY
                if (pb_desc->output_obj[arg_index].object_type == NETSOLVE_FILE)
                {
                  char *tmp;

                  /* Get the length */
                  shadow = va_arg(argptr,int);
                  tmp = (char *)(data_output[arg_index]);
                  data_output[arg_index] = (char *)calloc(shadow+1,sizeof(char));
                  strncpy((char *)(data_output[arg_index]),tmp,shadow);
                  ((char *)(data_output[arg_index]))[shadow] = '\0';
                }
#endif
                break;
              }
            }
          }
          else /* INTEGER */
          {
           if (mnemonic[0] == 'l')
              sizes_output[arg_index].l = *((int *)arg_value_ptr);
          }
        }
      }
      mnemonic = (char *)strtok(NULL,"\t ,");
    }
  }
  /* Pop the stuff of the stack is a SUN_SCHEME */
#ifdef STACK_SCHEME_SUN
  /* Pop the length of the real_code */
  shadow = va_arg(argptr,int);
  if (shadow != strlen(real_code)+2)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Invalid number of arguments passed to netsl()",
      __FILE__,__LINE__);
    *return_value = NetSolveBadCallingSequence;
    return;
  }
  for (i=0;i<index_char_pointer;i++)
  {
      char *tmp;
      shadow = va_arg(argptr,int);
      tmp = (char *)(char_pointer[i]);
      char_pointer[i] = (char *)calloc(shadow+1,sizeof(char));
      strncpy((char *)(char_pointer[i]),tmp,shadow);
  }
#endif


  /******************************/
  /* Processing the expressions */
  /******************************/

  status = parse_formulas(pb_desc,sizes_input,sizes_output,data_input);
  if (status != 1)
  {
    /* the Error Message is already set */
    *return_value =  NetSolveFailure;
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    free_pb_desc(pb_desc);
    return;
  }

#ifdef THREADS
  /*******************************/
  /* Crating a Thread            */
  /*******************************/

  /* Beginning of critical section */
  if (pthread_mutex_lock(&thread_mutex))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not lock a mutex",
            __FILE__,__LINE__);
    *return_value =  NetSolveThreadError;
    return;
  }

  /* finding a slot in the thread list */

  slot =-1;
  for (i=0;i<MAX_REQUEST;i++)
    if (request[i] == NULL)
    {
      slot = i;
      break;
    }

  if (slot == -1)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Too many pending requests to create another one",__FILE__,__LINE__);
    if (pthread_mutex_unlock(&thread_mutex))
    {
      sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Could not unlock a mutex",
              __FILE__,__LINE__);
      *return_value =  NetSolveThreadError;
      return;
    }
    *return_value = NetSolveRequestFull;
    return;
  }


  /* Create a new entry in the array */

  request[slot] = (thread_list *)calloc(1,sizeof(thread_list));

  if (pthread_mutex_unlock(&thread_mutex))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not unlock a mutex",
            __FILE__,__LINE__);
    *return_value =  NetSolveThreadError;
    return;
  }
  /* End of critical section */

  request[slot]->command_name = (char *)strdup(pb_desc->code);
  request[slot]->thread_status = THREAD_RUNNING;
  arguments = (thread_input *)calloc(1,sizeof(thread_input));

  arguments->pb_desc = pb_desc;
  arguments->sizes_input = sizes_input;
  arguments->sizes_output = sizes_output;
  arguments->data_input = data_input;
  arguments->data_output = data_output;
  arguments->my_slot = slot;

  if (pthread_create(&(request[slot]->thread_id),NULL,
                thread_do,(void*)arguments))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not create a thread",
            __FILE__,__LINE__);
    *return_value = NetSolveThreadError;
    return;
  }

  *(return_value) = slot;

  /* So that this thread releases control.... ugly, but for now.... */
  sleep(1);
  return;
#endif

#ifdef NOTHREADS
  /****************************/
  /* Calling solve()          */
  /****************************/

  status = nothread_solve(-1,pb_desc,sizes_input,sizes_output,
                         data_input,data_output);

  *(return_value) = status;
  return;
#endif
}

#ifdef THREADS
/*
 * thread_do()
 */
void *thread_do(void *input)
{
  int i;
  thread_input *s;

  s = (thread_input *)input;

  request[s->my_slot]->NetSolve_error_code = thread_solve(
           s->pb_desc,s->sizes_input,
           s->sizes_output,s->data_input,s->data_output);
  request[s->my_slot]->thread_status = THREAD_DONE;
  for (i=0;i<s->pb_desc->nb_obj_output;i++)
  {
    if (s->sizes_output[i].m != NULL)
      free(s->sizes_output[i].m);
    if (s->sizes_output[i].n != NULL)
      free(s->sizes_output[i].n);
  }
  free_pb_desc(s->pb_desc);
  free(s->sizes_input);
  free(s->sizes_output);
  free(s->data_input);
  free(s->data_output);
  free(s);
  return NULL;
}
#endif
