/*****************************************************************/
/*      netsl.c                                                  */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netsl()                                                      */
/*****************************************************************/

/*
 * netsl() : Interface to gateway() with variable number of parameters
 */

#include "netsolve.h"
#include <stdio.h>
#ifdef  __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#ifdef __STDC__
int netsl(char *code,...)
#else /* not STDC */
int netsl(va_alist)
va_dcl
#endif /* STDC */
{
#ifndef __STDC__
  char *code;
#endif
  int info,status;
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input, **data_output;
  char *argument;

  static already_been_here=0;
  int i;
  va_list argptr;
  char real_code[CODE_LENGTH];

  int code_len;


  /********************/
  /* Getting the code */
  /********************/

#ifdef __STDC__
  va_start(argptr,code); 
#else
  va_start(argptr);
  code       = va_arg(argptr,char *);
#endif
  strcpy(real_code,code);

  /*********************************/
  /* Stripping off the parenthesis */
  /*********************************/
  code_len = strlen(real_code);

  if (real_code[code_len-2] != '(')
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return NetSolveBadCode;
  }
  if (real_code[code_len-1] != ')')
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return NetSolveBadCode;
  }
  real_code[code_len-2] = '\0';

  /********************/
  /* Initializing     */
  /********************/

  netslmajorintern(ROW_MAJOR);

  /***********************/
  /* Getting the number  */
  /* of parameters       */
  /***********************/

  pb_desc = (problem_desc *)calloc(1,sizeof(problem_desc));
  status = netsolve_info(real_code,pb_desc);

  if (status <0)
  {
    free_pb_desc(pb_desc);
    /* the error message is already set */
    return status;
  }

  /************************/
  /* Allocating the space */
  /************************/

  sizes_input = 
     (input_sizes *)calloc(pb_desc->nb_obj_input,sizeof(input_sizes));
  sizes_output = 
     (output_sizes *)calloc(pb_desc->nb_obj_output,sizeof(output_sizes));
  data_input = (void **)calloc(pb_desc->nb_obj_input,sizeof(void *));
  data_output = (void **)calloc(pb_desc->nb_obj_output,sizeof(void *));

  /****************************/
  /* Processing the constants */
  /****************************/

  status = parse_constants(pb_desc,sizes_input,sizes_output);
  if (status != 1)
  {
    free_pb_desc(pb_desc);
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    /* The error message is already set */
    return status;
  }

  /*************************/
  /* Parsing the arguments */
  /* Using the format      */
  /*************************/

  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    int arg_type;
    int arg_i_o;
    int arg_index;
    int arg_value_int;
    void *arg_value_ptr;
    char *mnemonic;

    argument = pb_desc->formats[0].args_desc[i];

    parse_arg(argument,&arg_type,&arg_index,&arg_i_o);

    /* Get the argument from the calling sequence */

    if (arg_type == IGNORE)
      arg_value_ptr = (void *)va_arg(argptr,void *);
    if (arg_type == INTEGER)
      arg_value_int = va_arg(argptr,int);
    if (arg_type == POINTER)
      arg_value_ptr = (void *)va_arg(argptr,void *);

    mnemonic = (char *)strtok(argument," \t,");
    while(mnemonic != NULL)
    {
      parse_arg(mnemonic,&arg_type,&arg_index,&arg_i_o);
      switch(arg_i_o)
      {
        case IGNORE :
        {
          break;
        }
        case INPUT :
        {
          if (arg_type == INTEGER)
          {
            switch(mnemonic[0])
            {
              case 'm' :  
              {
                sizes_input[arg_index].m = arg_value_int;
                break;
              }
              case 'n' :  
              {
                sizes_input[arg_index].n = arg_value_int;
                break;
              }
              case 'l' :  
              {
                sizes_input[arg_index].l = arg_value_int;
                break;
              }
            }
          }
          else /* POINTER */
          {
            data_input[arg_index] = arg_value_ptr;
          }
          break;
        }
        case OUTPUT :
        {
          if (arg_type == POINTER)
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_output[arg_index].m = (int *)arg_value_ptr;
                break;
              }
              case 'n' :
              {
                sizes_output[arg_index].n = (int *)arg_value_ptr;
                break;
              }
              case 'O' :
              {
                data_output[arg_index] = arg_value_ptr;
                break;
              }
            }
          }
          else /* INTEGER */
          {
            if (mnemonic[0] == 'l')
              {
                sizes_output[arg_index].l = arg_value_int;
                break;
              }
          }
        }
      }
      mnemonic = (char *)strtok(NULL,"\t ,");
    }
  }
  /******************************/
  /* Processing the expressions */
  /******************************/

  status = parse_formulas(pb_desc,sizes_input,sizes_output,data_input);
  if (status != 1)
  {
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    free_pb_desc(pb_desc);
    /* The error message is already set */
    return status;
  }

  /****************************/
  /* Calling solve()          */
  /****************************/

  status = thread_solve(pb_desc,sizes_input,sizes_output,data_input,data_output);
  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    if (sizes_output[i].m != NULL)
      free(sizes_output[i].m);
    if (sizes_output[i].n != NULL)
      free(sizes_output[i].n);
  }
  free_pb_desc(pb_desc);
  free(sizes_input);
  free(sizes_output);
  free(data_input);
  free(data_output);
  return status;
}

