/*****************************************************************/
/*      netslnb.c                                                */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netsl_nb()                                                   */
/*****************************************************************/

/*
 * netsl_nb() : 
 */

#include "netsolve.h"
#include <stdio.h>
#ifdef  __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#ifdef __STDC__
int netslnb(char *code,...)
#else /* not STDC */
int netslnb(va_alist)
va_dcl
#endif /* STDC */
{
#ifndef __STDC__
  char *code;
#endif
  int slot;
  int status;
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input, **data_output;
  char *argument;
#ifdef THREADS
  thread_input *arguments;
  void *thread_do();
#endif
  static int already_been_here=0;
  int i;
  va_list argptr;
  char real_code[CODE_LENGTH];
  int code_len;


  /********************/
  /* Getting the code */
  /********************/

#ifdef __STDC__
  va_start(argptr,code); 
#else
  va_start(argptr);
  code       = va_arg(argptr,char *);
#endif
  strcpy(real_code,code);

  /*********************************/
  /* Stripping off the parenthesis */
  /*********************************/
  code_len = strlen(real_code);

  if (real_code[code_len-2] != '(')
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return NetSolveBadCode;
  }
  if (real_code[code_len-1] != ')')
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "First argument should be 'XXX()' where XXX is the problem name",
            __FILE__,__LINE__);
    return NetSolveBadCode;
  }
  real_code[code_len-2] = '\0';

  /********************/
  /* Initializing     */
  /********************/

  netslmajorintern(ROW_MAJOR);

  if (already_been_here == 0)
  {
#ifdef THREADS
    pthread_mutex_init(&thread_mutex,NULL);
#endif
    for (i = 0;i<MAX_REQUEST;i++) 
      request[i] = NULL;
    already_been_here = 1;
  }
 
  /***********************/
  /* Getting the number  */
  /* of parameters       */
  /***********************/

  pb_desc = (problem_desc *)calloc(1,sizeof(problem_desc));
  status = netsolve_info(real_code,pb_desc);

  if (status <0)
  {
    free_pb_desc(pb_desc);
    /* the error message is already set */
    return status;
  }

  /************************/
  /* Allocating the space */
  /************************/

  sizes_input = 
     (input_sizes *)calloc(pb_desc->nb_obj_input,sizeof(input_sizes));
  sizes_output = 
     (output_sizes *)calloc(pb_desc->nb_obj_output,sizeof(output_sizes));
  data_input = (void **)calloc(pb_desc->nb_obj_input,sizeof(void *));
  data_output = (void **)calloc(pb_desc->nb_obj_output,sizeof(void *));

  /****************************/
  /* Processing the constants */
  /****************************/

  status = parse_constants(pb_desc,sizes_input,sizes_output);
  if (status != 1)
  {
    free_pb_desc(pb_desc);
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    /* The error message is already set */
    return status;
  }

  /*************************/
  /* Parsing the arguments */
  /* Using the format      */
  /*************************/

  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    int arg_type;
    int arg_i_o;
    int arg_index;
    int arg_value_int;
    void *arg_value_ptr;
    char *mnemonic;

    argument = pb_desc->formats[0].args_desc[i];

    parse_arg(argument,&arg_type,&arg_index,&arg_i_o);

    /* Get the argument from the calling sequence */

    if (arg_type == IGNORE)
      arg_value_ptr = (void *)va_arg(argptr,void *);
    if (arg_type == INTEGER)
      arg_value_int = va_arg(argptr,int);
    if (arg_type == POINTER)
      arg_value_ptr = (void *)va_arg(argptr,void *);

    mnemonic = (char *)strtok(argument," \t,");
    while(mnemonic != NULL)
    {
      parse_arg(mnemonic,&arg_type,&arg_index,&arg_i_o);
      switch(arg_i_o)
      {
        case IGNORE :
        {
          break;
        }
        case INPUT :
        {
          if (arg_type == INTEGER)
          {
            switch(mnemonic[0])
            {
              case 'm' :  
              {
                sizes_input[arg_index].m = arg_value_int;
                break;
              }
              case 'n' :  
              {
                sizes_input[arg_index].n = arg_value_int;
                break;
              }
              case 'l' :  
              {
                sizes_input[arg_index].l = arg_value_int;
                break;
              }
            }
          }
          else /* POINTER */
          {
            data_input[arg_index] = arg_value_ptr;
          }
          break;
        }
        case OUTPUT :
        {
          if (arg_type == POINTER)
          {
            switch(mnemonic[0])
            {
              case 'm' :
              {
                sizes_output[arg_index].m = (int *)arg_value_ptr;
                break;
              }
              case 'n' :
              {
                sizes_output[arg_index].n = (int *)arg_value_ptr;
                break;
              }
              case 'O' :
              {
                data_output[arg_index] = arg_value_ptr;
                break;
              }
            }
          }
          else /* INTEGER */
          {
            if (mnemonic[0] == 'l')
              {
                sizes_output[arg_index].l = arg_value_int;
                break;
              }
          }
        }
      }
      mnemonic = (char *)strtok(NULL,"\t ,");
    }
  }
  /******************************/
  /* Processing the expressions */
  /******************************/

  status = parse_formulas(pb_desc,sizes_input,sizes_output,data_input);
  if (status != 1)
  {
    free(sizes_input);
    free(sizes_output);
    free(data_input);
    free(data_output);
    free_pb_desc(pb_desc);
    /* The error message is already set */
    return status;
  }

#ifdef THREADS
  /****************************/
  /* Creating a thread        */
  /****************************/

  /* Beginning of critical section */
  if (pthread_mutex_lock(&thread_mutex))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not lock a mutex",
            __FILE__,__LINE__);
    return NetSolveThreadError;
  }

  /* finding a slot in the thread list */
  
  slot =-1;
  for (i=0;i<MAX_REQUEST;i++)
    if (request[i] == NULL)
    {
      slot = i;
      break;
    }
  
  if (slot == -1)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Too many pending requests to create another one",__FILE__,__LINE__);
    if (pthread_mutex_unlock(&thread_mutex))
    {
      sprintf(error_message,"%s \n\t\t[%s,%d]",
              "Could not unlock a mutex",
              __FILE__,__LINE__);
      return NetSolveThreadError;
    }
    return NetSolveRequestFull;
  }

  /* Create a new entry in the array */
  request[slot] = (thread_list *)calloc(1,sizeof(thread_list));
  if (pthread_mutex_unlock(&thread_mutex))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not unlock a mutex",
            __FILE__,__LINE__);
    return NetSolveThreadError;
  }
  /* End of critical section */

  request[slot]->command_name = (char *)strdup(pb_desc->code);
  request[slot]->thread_status = THREAD_RUNNING;

  arguments = (thread_input *)calloc(1,sizeof(thread_input));

  arguments->pb_desc = pb_desc;
  arguments->sizes_input = sizes_input; 
  arguments->sizes_output = sizes_output; 
  arguments->data_input = data_input;
  arguments->data_output = data_output;
  arguments->my_slot = slot;

  if (pthread_create(&(request[slot]->thread_id),NULL,
                thread_do,(void*)arguments))
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
            "Could not create a thread",
            __FILE__,__LINE__);
    return NetSolveThreadError;
  }
  sleep(1);
  return slot;
#endif

#ifdef NOTHREADS
  /****************************/
  /* Calling solve()          */
  /****************************/

  status = nothread_solve(-1,pb_desc,sizes_input,sizes_output,
                         data_input,data_output); 

  return status;
#endif
}

#ifdef THREADS
/*
 * thread_do()
 */
void *thread_do(void *input)
{
  int i;
  thread_input *s;

  s = (thread_input *)input;

  request[s->my_slot]->NetSolve_error_code = thread_solve(
           s->pb_desc,s->sizes_input,
           s->sizes_output,s->data_input,s->data_output);

  request[s->my_slot]->thread_status = THREAD_DONE;
  for (i=0;i<s->pb_desc->nb_obj_output;i++)
  {
    if (s->sizes_output[i].m != NULL)
      free(s->sizes_output[i].m);
    if (s->sizes_output[i].n != NULL)
      free(s->sizes_output[i].n);
  }
  free_pb_desc(s->pb_desc);
  free(s->sizes_input);
  free(s->sizes_output);
  free(s->data_input);
  free(s->data_output);
  free(s);
  return NULL;
}
#endif
