/*****************************************************************/
/*      client_f_report.c                                        */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  client_f_report()                                            */
/*  job_completed()                                              */
/*****************************************************************/

#include "netsolve.h"

/*
 * client_f_report()
 *
 * This function generates a failure report message and send it
 * to the communication server previously contacted
 *
 * Return 1 on success, -1 otherwise.
 */

#ifdef __STDC__
int client_f_report(int type,char *agent,unsigned int IPaddr)
#else
int client_f_report(type,agent,IPaddr)
int type;
char *agent;
unsigned int IPaddr;
#endif
{
  int sock;                      
  int message_type;
  communicator *comm;

  sock = connect_to_agent(agent,0);

  if(sock == -3)
  {
    fprintf(stderr,"Impossible to create socket\n");
    return -1;
  }
 if (sock == -2)
 {
    fprintf(stderr,"Impossible to locate host '%s'\n",agent);
    return -1;
  }

 if (sock == -1)
 {
    fprintf(stderr,"No NetSolve agent on '%s'\n",agent);
    return -1;
  }

  message_type = CLIENT_F_REPORT;
  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  write_int(comm,&message_type);
  write_int(comm,&type);
  write_IPaddr(comm,&IPaddr);
  end_message(comm);
  end_com(sock);
  return 1;
}

/*
 * job_completed()
 *
 * This function generates a message sent to the communication server
 * to make it cancel the artificial workload update.
 *
 * Return 1 on success, -1 otherwise.
 */

#ifdef __STDC__
int job_completed(char *agent,unsigned int IPaddr)
#else
int job_completed(agent,IPaddr)
char *agent;
unsigned int IPaddr;
#endif
{
  int sock;
  int message_type;
  communicator *comm;

  sock = connect_to_agent(agent,0);
  if(sock == -3)
  {
    fprintf(stderr,"Impossible to create socket\n");
    return -1;
  }
 if (sock == -2)
 {
    fprintf(stderr,"Impossible to locate host '%s'\n",agent);
    return -1;
  }

 if (sock == -1)
 {
    fprintf(stderr,"No NetSolve agent on '%s'\n",agent);
    return -1;
  }

  message_type = JOB_COMPLETED;
  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  write_int(comm,&message_type);
  write_IPaddr(comm,&IPaddr);
  end_message(comm);
  end_com(sock);
  return 1;
}

