/*****************************************************************/
/*      client_util.c                                            */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  parse_arg()                                                  */
/*  eval()                                                       */
/*  get_op_arg1_arg2()                                           */
/*  get_array_index()                                            */
/*  uppercase()                                                  */
/*  sigpipe_handler()                                            */
/*****************************************************************/


#include "netsolve.h"

/*
 *  parse_arg()
 *
 */
#ifdef __STDC__
int parse_arg(char *string,int *arg_type,int *arg_index,int *arg_i_o)
#else
int parse_arg(string,arg_type,arg_index,arg_i_o)
char *string;
int *arg_type,*arg_index,*arg_i_o;
#endif
{
  if (string == NULL)
    return -1;

  switch(string[0])
  {
    case '?' :
    {
      *arg_type = IGNORE;
      *arg_index = IGNORE;
      *arg_i_o = IGNORE;
      return 1;
    }
    case 'I' :
    {
      *arg_type = POINTER;
      *arg_index = atoi(string+1);
      *arg_i_o = INPUT;
      return 1;
    }
    case 'O' :
    {
      *arg_type = POINTER;
      *arg_index = atoi(string+1);
      *arg_i_o = OUTPUT;
      return 1;
    }
    case 'm' :
    {
      *arg_index = atoi(string+2);
      switch(string[1])
      {
        case 'I' :
        {
          *arg_i_o = INPUT;
          *arg_type = INTEGER;
          return 1;
        }
        case 'O' :
        { 
          *arg_i_o = OUTPUT;
          *arg_type = POINTER;
          return 1;
        }
      }
    }
    case 'n' :
    {
      *arg_index = atoi(string+2);
      switch(string[1])
      {
        case 'I' :
        {
          *arg_i_o = INPUT;
          *arg_type = INTEGER;
          return 1;
        }
        case 'O' :
        {
          *arg_i_o = OUTPUT;
          *arg_type = POINTER;
          return 1;
        }
      }
    }
    case 'l' :
    {
      *arg_index = atoi(string+2);
      *arg_type = INTEGER;
      switch(string[1])
      {
        case 'I' :
        {
          *arg_i_o = INPUT;
          return 1;
        }
        case 'O' :
        {
          *arg_i_o = OUTPUT;
          return 1;
        }
      }
    }
  }
  return 1;
  
}

/*
 * eval() : Evaluate an expression
 *
 */

#ifdef __STDC__
int eval(problem_desc *pb_desc,input_sizes *sizes_input,
     void **data_input,char *string)
#else
int eval(pb_desc,sizes_input,data_input,string)
problem_desc *pb_desc;
input_sizes *sizes_input;
void **data_input;
char *string;
#endif
{

  if (strncmp(string,"op(",3) == 0) /* OPERATOR */
  {
    char *op;
    char *arg1,*arg2;
    int val1,val2,result;

    get_op_arg1_arg2(string,&op,&arg1,&arg2);
    val1 = eval(pb_desc,sizes_input,data_input,arg1);
    val2 = eval(pb_desc,sizes_input,data_input,arg2);
    if (strcmp(op,"+") == 0)
      result = val1+val2;
    else if (strcmp(op,"-") == 0)
      result = val1-val2;
    else if (strcmp(op,"*") == 0)
      result = val1 * val2;
    free(op);free(arg1);free(arg2);
    return result;
  }
  else if (strncmp(string,"array(",6) == 0) /* ARRAY ACCESS */
  {
    char *array;
    char *index;
    int number;
    int index_val,result;

    get_array_index(string,&array,&index);
    number = atoi(array+1);
    index_val = eval(pb_desc,sizes_input,data_input,index);
    if ((index_val > sizes_input[number].m) ||
        (index_val < 0))
      return -1;
   if (pb_desc->input_obj[number].data_type == NETSOLVE_I)
      result = ((int *)(data_input[number]))[index_val];
   if (pb_desc->input_obj[number].data_type == NETSOLVE_CHAR)
    result = (int)(((char *)(data_input[number]))[index_val]);
    free(array);free(index);
    return result;
  }
  else if (strncmp(string,"'",1) == 0) /* CHARACTER CONSTANT */
  {
    if (strlen(string) != 3)
      return -1;
    if (string[2] != '\'')
      return -1;
    return (int)(string[1]);
  }
  else if (strncmp(string,"if(",3) == 0) /* CONDITIONAL */
  {
    char *lhs;
    char *rhs;
    char *ifstmt;
    char *elsestmt;
    char *start = string + 3;
    int lhv,rhv;
    int t;

    if (get_members(start,&lhs,&rhs,&ifstmt,&elsestmt) == -1)
      return -1;


    lhv = eval(pb_desc,sizes_input,data_input,lhs);
    rhv = eval(pb_desc,sizes_input,data_input,rhs);
   
    if ((lhv == -1) || (rhv == -1))
      return -1;

    if (lhv == rhv)
      t =  eval(pb_desc,sizes_input,data_input,ifstmt);
    else
      t =  eval(pb_desc,sizes_input,data_input,elsestmt);

    free(lhs);free(rhs);free(ifstmt);free(elsestmt);
    return t;
  }
  else if (is_number(string) == 0)
  {
    return atoi(string);
  }
  else /* MNEMONIC : must be integer */
  {
    int number;
 
    number = atoi(string+2);
    switch (string[0])
    {
      case 'm' :
      {
        return sizes_input[number].m;
      }
      case 'n' :
      {
        return sizes_input[number].n;
      }
      case 'l' :
      {
        return sizes_input[number].l;
      }
      default :
      {
        return -1;
      }
    }
  }
}

/*
 * get_op_arg1_arg2()
 *
 * extract 'op','arg1','arg2' from an op(x,y,z) expression.
 *
 */
#ifdef __STDC__
int get_op_arg1_arg2(char *string,char **op,char **arg1,char **arg2)
#else
int get_op_arg1_arg2(string,op,arg1,arg2)
char *string;
char **op;
char **arg1;
char **arg2;
#endif
{
  int nb_parent_seen = 0;
  char *buf;
  char *tmp;
  char *to_free;

  string[strlen(string)-1] = '\0';

  to_free = buf = (char *)strdup(string+3);
  /* Get the op */
  tmp = (char *)strchr(buf,',');
  if (tmp == NULL)
  {
    *op = NULL;
    *arg1 = NULL;
    *arg2 = NULL;
    return 1;
  }
  *tmp = '\0';
  *op = (char *)strdup(buf);
  buf = tmp+1;
  tmp = buf;
  while(1)
  {
    if (*tmp == '(')
      nb_parent_seen++;
    if (*tmp == ')')
      nb_parent_seen--;
    if ((*tmp == ',')&&(nb_parent_seen == 0))
      break;
    tmp++;
  }
  *tmp = '\0';
  *arg1 = (char *)strdup(buf);
  buf = tmp+1;
  tmp = buf;
  nb_parent_seen = 0;
  while(1)
  {
    if (*tmp == '\0')
    {
      break;
    }
    if (*tmp == '(')
      nb_parent_seen++;
    if (*tmp == ')')
      nb_parent_seen--;
    if ((*tmp == ',')&&(nb_parent_seen == 0))
      break;
    tmp++;
  }
  *tmp = '\0';
  *arg2 = (char *)strdup(buf);
  buf = tmp+1;
  free(to_free);
  return 1;
}


/*
 * get_array_index()
 *
 * extract 'array','index' from an array(x,y) expression.
 *
 */
#ifdef __STDC__
int get_array_index(char *string,char **array,char **index)
#else
int get_array_index(string,array,index)
char *string;
char **array;
char **index;
#endif
{
  int nb_parent_seen = 0;
  char *buf;
  char *tmp;
  char *to_free;

  string[strlen(string)-1] = '\0';

  to_free = buf = (char *)strdup(string+6);
  tmp = buf;
  while(1)
  {
    if (*tmp == '(')
      nb_parent_seen++;
    if (*tmp == ')')
      nb_parent_seen--;
    if ((*tmp == ',')&&(nb_parent_seen == 0))
      break;
    tmp++;
  }
  *tmp = '\0';
  *array = (char *)strdup(buf);
  buf = tmp+1;
  tmp = buf;
  nb_parent_seen = 0;
  while(1)
  {
    if (*tmp == '\0')
    {
      break;
    }
    if (*tmp == '(')
      nb_parent_seen++;
    if (*tmp == ')')
      nb_parent_seen--;
    if ((*tmp == ',')&&(nb_parent_seen == 0))
      break;
    tmp++;
  }
  *tmp = '\0';
  *index = (char *)strdup(buf);
  buf = tmp+1;
  free(to_free);
  return 1;
}

/*
 * uppercase() : change a string in uppercase
 */
#ifdef __STDC__
int uppercase(char *string)
#else
int uppercase(string)
char *string;
#endif
{
  int length,i;

  length = strlen(string);
  for (i=0;i<length;i++)
    if ((string[i] >= 97)&&(string[i]<=122))
      string[i] = string[i]-32;
}

/*
 * get_members()
 */
#ifdef __STDC__
int get_members(char *string,char **lhs,char **rhs,
                char **ifstmt,char **elsestmt)
#else
int get_members(string,lhs,rhs,ifstmt,elsestmt)
char *string;
char **lhs;
char **rhs;
char **ifstmt;
char **elsestmt;
#endif
{
  int nbpar = 0;
  int length;
  int comma[2];
  int nb=0;
  int i;
  int egal = 0;
  char *start;

  for (i=0;i<2;i++)
    comma[i] = 0;

  start = (char *)strdup(string);
  length = strlen(start);
  start[--length] = '\0';

  /* Looking for ','s */

  for (i=0;i<length;i++)
  {
    if (start[i] == ',')
      if (nbpar == 0)
      {
        if (nb == 2)
        {
          return -1;
        }
        else
          comma[nb++] = i;
      }
    if (start[i] == '(')
      nbpar++;
    if (start[i] == ')')
      nbpar--;
  }
  if ((comma[0] == 0)||
      (comma[1] == 0)||
      (comma[0] == comma[1])||
      (nbpar != 0))
  {
    return -1;
  }

  /* now, we have the 'if' and 'else' */

  *(start+comma[0]) = '\0';
  *(start+comma[1]) = '\0';
  *ifstmt   = (char *)strdup(start+comma[0]+1);
  *elsestmt = (char *)strdup(start+comma[1]+1);

  /* Deal with the expression */

  for (i=0;i<comma[0];i++)
  {
    if (start[i] == '=')
      if (nbpar == 0)
        egal = i;
    if (start[i] == '(')
      nbpar++;
    if (start[i] == ')')
      nbpar--;
  }
  if ((nbpar != 0)||(egal == 0))
  {
    return -1;
  }
  *(start+egal) = '\0';
  *lhs = (char *)strdup(start);
  *rhs = (char *)strdup(start+egal+1);
  free(start);
  return 1;
}

#ifdef __STDC__
void get_sizes(problem_desc *pb_desc,input_sizes *sizes_input,
               int *data_size, int *problem_size, int *result_size)
#else
void get_sizes(pb_desc,sizes_input,data_size,problem_size,result_size)
problem_desc *pb_desc;
input_sizes *sizes_input;
int *data_size;
int *problem_size;
int *result_size;
#endif
{
  int j;

  *data_size = 0; *problem_size = 0; *result_size = 0;
  for (j=0;j<pb_desc->nb_obj_input;j++)
  {
    int m,n;

    m = sizes_input[j].m;
    n = sizes_input[j].n;
    if (m > *problem_size)
      *problem_size = m;
    if (n > *problem_size)
      *problem_size = n;
    switch (pb_desc->input_obj[j].data_type)
    {
      case NETSOLVE_I :
      {
        *data_size += INTL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_B :
      {
        *data_size += sizeof(char)*m*n+2*sizeof(char);
        break;
      }
      case NETSOLVE_CHAR :
      {
        *data_size += sizeof(char)*m*n+2*sizeof(char);
        break;
      }
      case NETSOLVE_S :
      {
        *data_size += FLOATL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_D :
      {
        *data_size += DOUBL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_C :
      {
        *data_size += SCOMPL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_Z :
      {
        *data_size += DCOMPL*m*n+2*INTL;
        break;
      }
    }
  }
}

#ifdef __STDC__
void sigpipe_handler(int sig)
#else
void
sigpipe_handler(sig)
int sig;
#endif
{

#ifdef SYSVSIGNAL
  signal(SIGPIPE,sigpipe_handler);
#endif
}

