/*****************************************************************/
/*      netsolve_info.c                                          */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*    netsolve_info()                                            */
/*****************************************************************/

#include "netsolve.h"

/*
 * netsolve_info()
 *
 *   Get information on a problem
 */

#ifdef __STDERR__
int netsolve_info(char *code,problem_desc *pb_desc)
#else
int netsolve_info(code,pb_desc)
char *code;
problem_desc *pb_desc;
#endif
{
  int sock;                            /* socket descriptor            */
 
  int message_type;
  int status;
  int i;
  char *agent;
  constant *current_constant;
  formula *current_form;
  int dummy1;
  communicator *comm;

  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The NETSOLVE_AGENT environment variable should be set",__FILE__,__LINE__);
    return NetSolveSetAgent;
  }

  sock = connect_to_agent(agent,0);

  if (sock == -3)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "NetSolve internal error",__FILE__,__LINE__);
    return NetSolveFailure;
  }
  if (sock == -2)
  {
    sprintf(error_message,"%s '%s' \n\t\t[%s,%d]",
      "Could not locate host ",agent,__FILE__,__LINE__);
    return NetSolveNoAgent;
  }
  if (sock == -1)
  {
    sprintf(error_message,"%s '%s' \n\t\t[%s,%d]",
      "No agent running on host ",agent,__FILE__,__LINE__);
    return NetSolveNoAgent;
  }

  
  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  message_type = INFO_REQUEST;
  write_int(comm,&message_type);
  write_string(comm,code);
  end_message(comm);

  comm = init_recv(sock);
  if (read_int(comm,&status) == -1)
  {
    end_com(sock);
    return NetSolveFailure;
  }
  if (status == NO_SUCH_PB)
  {
    end_message(comm);
    end_com(sock);
    sprintf(error_message,"%s : %s \n\t\t[%s,%d]",
      code,"no such problem",__FILE__,__LINE__);
    return NetSolveUnknownProblem;
  }
 
  strcpy(pb_desc->code,code);
  read_string(comm,pb_desc->path);
  read_string(comm,pb_desc->desc);
  read_int(comm,&(pb_desc->nb_obj_input));
  pb_desc->input_obj = 
          (object *)calloc(pb_desc->nb_obj_input,sizeof(object));
  for (i=0;i<pb_desc->nb_obj_input;i++)
  {
    read_int(comm,&(pb_desc->input_obj[i].data_type));
    read_int(comm,&(pb_desc->input_obj[i].object_type));
    read_string(comm,pb_desc->input_obj[i].object_desc);
  }
  read_int(comm,&(pb_desc->nb_obj_output));
  pb_desc->output_obj =
          (object *)calloc(pb_desc->nb_obj_output,sizeof(object));
  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    read_int(comm,&(pb_desc->output_obj[i].data_type));
    read_int(comm,&(pb_desc->output_obj[i].object_type));
    read_string(comm,pb_desc->output_obj[i].object_desc);
  }
  /* Read the format */
  /* We only malloc space for one format */
  pb_desc->nb_formats = 1;
  pb_desc->formats = (format_spec *)calloc(1,sizeof(format_spec));
  read_int(comm,&(pb_desc->formats[0].nb_args));
  pb_desc->formats[0].args_desc = 
        (char **)calloc(pb_desc->formats[0].nb_args,sizeof(char **));
  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    char buf[256];

    read_string(comm,buf);
    pb_desc->formats[0].args_desc[i] = (char *)strdup(buf);
  }
  read_int(comm,&(pb_desc->formats[0].nb_constants));
  /* Swapped while received, but don't care */
  for (i=0;i<pb_desc->formats[0].nb_constants;i++)
  {
    char buf[256];

    current_constant = (constant *)malloc(sizeof(constant));
    current_constant->next = pb_desc->formats[0].consts;
    read_string(comm,buf);
    current_constant->mnemonic = (char *)strdup(buf);
    read_int(comm,&(current_constant->value));
    pb_desc->formats[0].consts = current_constant;
  }

  read_int(comm,&(pb_desc->formats[0].nb_formulae));
  /* Swapped while received, but don't care */
  /* IN FACT I DO CARE BECAUSE OF DATA DEPENDENCIES DUMBASS !!!! */
  pb_desc->formats[0].forms = NULL;
  for (i=0;i<pb_desc->formats[0].nb_formulae;i++)
  {
    char buf1[256];
    char buf2[256];
    formula *tmp;

    /* Reading the formula */
    current_form = (formula *)malloc(sizeof(formula));
    read_string(comm,buf1);
    current_form->mnemonic = (char *)strdup(buf1);
    read_string(comm,buf2);
    current_form->expression = (char *)strdup(buf2);

    /* Inserting at the end of the list */
    tmp = pb_desc->formats[0].forms;
    if (pb_desc->formats[0].forms == NULL)
    {
      pb_desc->formats[0].forms = current_form;
      pb_desc->formats[0].forms->next = NULL; 
    }
    else
    {
      while(tmp->next != NULL)
        tmp = tmp->next;
      tmp->next = current_form;
      current_form->next = NULL;
    }
  }

  /* Ignore the MATLAB information */
  read_int(comm,&dummy1);
  for (i=0;i<dummy1;i++)
  {
    int dummy;
    read_int(comm,&dummy);
    read_int(comm,&dummy);
  }

  end_message(comm);
  end_com(sock);
  return NetSolveSuccess;
}
