/*****************************************************************/
/*      nothread_get_result.c                                    */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  nothread_get_result()                                        */
/*  restart()                                                    */
/*****************************************************************/

#include "netsolve.h"

#ifdef NOTHREADS

/*
 * nothread_get_result() : Check the status of a pending request
 *     mode = PROBE  - non-blocking
 *     more = WAIT   - blocking
 */

#ifdef __STDC__
int nothread_get_result(int r,int mode)
#else
int nothread_get_result(r,mode)
int r;
int mode;
#endif
{
  int k;
  int status,i;
  communicator *comm;
  int obj,dat;
  int  stdoutlength;
  char *stdoutbuf;
  char *agent;
  communicator *hbm_comm;
  communicator *server_comm;
  int hbm_sock;
  int hbm_activated = 1;
  int alive=1; /* Set to one in case the HBM is disabled */

  (void)signal(SIGPIPE,sigpipe_handler);


  /* Is the request in existence ? */

  if (request[r] == 0)
  {
#ifdef DEBUG
    fprintf(stderr,"Unknown Request\n");
#endif
    sprintf(error_message,"%s (%d) \n\t\t[%s,%d]",
      "Invalid request number",r,
      __FILE__,__LINE__);
    return NetSolveInvalidRequestNumber;
  }

  /* Get the hostname of the agent */

  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The environment variable NETSOLVE_AGENT should be set",
      __FILE__,__LINE__);
    return NetSolveSetAgent;
  }

  /*  Connect to the HBM */

  hbm_sock = connect_to_socket(agent, 0, HBM_INTERFACE_PORT);
  if (hbm_sock<0)
  {
#ifdef DEBUG
    fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
    hbm_activated = 0;
  }
  else
  {
    hbm_comm = init_send(DATA_XDR,hbm_sock);
    if (write_int(hbm_comm,&(Xserver_pid[r])) == -1)
    {
#ifdef DEBUG
      fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
      hbm_activated = 0;
    }
    end_message(hbm_comm);
  }


  if (mode == PROBE)
  {
    int off = 0;
    int on = 1;
    char buf[1];
    int try;
    int server_status;

    ioctl(Xdesc[r],FIONBIO,&on);
    try = recv(Xdesc[r],buf,1,MSG_PEEK);
    ioctl(Xdesc[r],FIONBIO,&off);

    if (try == -1) /* Not ready */
    {
#ifdef DEBUG
      fprintf(stderr,"Not ready\n");
#endif

      /* Try the HBM to see if still alive */
      if (hbm_activated)
      { 
        if ((hbm_comm = init_recv(hbm_sock)) == NULL)
        {
#ifdef DEBUG
          fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
          hbm_activated = 0;
          end_com(hbm_sock);
        }
      }
      if (hbm_activated)
      {
        if (read_int(hbm_comm,&server_status) == -1)
        {
#ifdef DEBUG
          fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
          hbm_activated = 0;
          end_message(hbm_comm);
          end_com(hbm_sock);
        }
      }
      if (hbm_activated)
      {
        end_message(hbm_comm);
        switch(server_status)
        {
          case 1:
          case 2:
          case 3:
          case 4:
            alive = 1;
            break; 
          default:
            alive = 0;
            break;
        }
      } 
      if (alive == 1)
      {
        sprintf(error_message,"request %d is not completed yet \n\t\t[%s,%d]\n",
          r,__FILE__,__LINE__);
        end_com(hbm_sock);
        return NetSolveNotReady;
      }
      else
        try = 0; /* In fact, there is no more connection */
    }
    if (try == 0) /* Connection doesn't exist anymore */
    {
      int status;
#ifdef DEBUG
      fprintf(stderr,"The computational server has failed !!\n");
#endif
      end_com(hbm_sock);
      end_com(Xdesc[r]);

      status = restart(mode,r);

      if (status != r)
        return status;

      sprintf(error_message,"request %d is not completed yet \n\t\t[%s,%d]\n",
        r,__FILE__,__LINE__);
      return NetSolveNotReady;
    }
    /* At this point, something wass on the socket */
  }
  
  while(hbm_activated)
  {
    int try;
    int on = 1;
    int off =0;
    char buf[1];
    int server_status;

    ioctl(Xdesc[r],FIONBIO,&on);
    try = recv(Xdesc[r],buf,1,MSG_PEEK);
    ioctl(Xdesc[r],FIONBIO,&off);

    if (try == 0) /* No connection anymore !! */
    {
      end_com(hbm_sock);
      end_com(Xdesc[r]);
      /**** RESTART ****/
      status = restart(mode,r); 
      if (status != r) /* Error message already set */
        return status;
      status = nothread_get_result(r,WAIT);
      if (status <0)
        return status;
    }
    if (try != -1)
      break;

    /* Not ready yet */
    if (hbm_activated)
    { /* Check the HBM */
      if ((hbm_comm = init_recv(hbm_sock)) == NULL)
      {
#ifdef DEBUG
        fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      if (read_int(hbm_comm,&server_status) == -1)
      {
#ifdef DEBUG
        fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_message(hbm_comm);
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      end_message(hbm_comm);
      switch(server_status)
      {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
          break; /* just continue in the while loop */

        default:
          end_com(hbm_sock);
          end_com(Xdesc[r]);
          /**** RESTARTING ****/
          status = restart(mode,r); 
          if (status != r) /* Error message already set */
            return status;
          status = nothread_get_result(r,WAIT);
          if (status <0)
            return status;
          break;
      }
    }
    sleep(2);
  }

  end_com(hbm_sock);

  comm = init_recv(Xdesc[r]);
  if (comm == NULL)
  {
    int status;
#ifdef DEBUG
    fprintf(stderr,"The computational server failed\n");
#endif
    status = restart(mode,r);
    if (status != r) /* Error message already set */
    {
      return status;
    }
    if (mode == WAIT)
    {
      status = nothread_get_result(r,WAIT);  /* Recursive Call */
      if (status < 0)
        return status;
    }
    else
    {
      sprintf(error_message,"request %d is not completed yet \n\t\t[%s,%d]\n",
        r,__FILE__,__LINE__);
      return NetSolveNotReady;
    }
  } 
  /*
   * I read the status of the computation !!!
   */
  read_int(comm,&status);
  /*
   * I read the stdout stuff 
   */
  read_int(comm,&stdoutlength);
  stdoutbuf = (char *)calloc(stdoutlength,sizeof(char));
  read_char_array(comm,stdoutbuf,stdoutlength);
  if (stdoutlength > 1)
  {
    fprintf(stdout,"-- stdout from request '%d' --\n",r);
    fprintf(stdout,"%s\n",stdoutbuf);
  }
  if (stdoutbuf != NULL)
    free(stdoutbuf);

  if (status == SV_FAILURE)
  {
    int re;

    end_message(comm);
    end_com(Xdesc[r]);
    re = restart(mode,r);
    if (re != r)
      return re;
    if (mode == WAIT)
    {
      status = nothread_get_result(r,WAIT);  /* Recursive Call */
      if (status < 0)
        status;
    }
    else
    {
      sprintf(error_message,"request %d is not completed yet \n\t\t[%s,%d]\n",
        r,__FILE__,__LINE__);
      return NetSolveNotReady;
    }
  }
  if (status == PB_NO_SOL)
  {
#ifdef VIEW
    fprintf(stdout,"The problem doesn't have a solution\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    job_completed(agent,XIPaddr[r]);
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The numerical problem does not admit any solution",
      __FILE__,__LINE__);
    return NetSolveNoSolution;
  }
  if (status == DM_MISMATCH)
  {
#ifdef VIEW
    fprintf(stdout,"The dimension are incorrect !!!\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    job_completed(agent,XIPaddr[r]);
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Dimensions of algebrix objects are mismatched",
      __FILE__,__LINE__);
    return NetSolveBadDimension;
  }
  if (status == BAD_VALUES)
  {
#ifdef VIEW
    fprintf(stdout,"Invalid argument values\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    job_completed(agent,XIPaddr[r]);
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Invalud input data for this numerical problem",
      __FILE__,__LINE__);
    return NetSolveBadValues;
  }


  for (i=0;i<Xpb_desc[r]->nb_obj_output;i++)
  {
    int total_size;

    obj = Xpb_desc[r]->output_obj[i].object_type;
    dat = Xpb_desc[r]->output_obj[i].data_type;

    switch(obj)
    {
      case NETSOLVE_MATRIX :
      {
        int maj,dummy;

        /* Useless, since the server makes sure it's in
           the right major anyway                       */
        read_int(comm,&dummy);
        read_int(comm,&dummy);
        read_int(comm,&maj);

        if (Xsizes_output[r][i].m == NULL)
        {
          Xsizes_output[r][i].m = (int *)malloc(INTL);
        }
        read_int(comm,Xsizes_output[r][i].m);
       if (Xsizes_output[r][i].n == NULL)
        {
          Xsizes_output[r][i].n = (int *)malloc(INTL);
        }
        read_int(comm,Xsizes_output[r][i].n);

        total_size = (*(Xsizes_output[r][i].m))*(*(Xsizes_output[r][i].n));
        break;
      }
      case NETSOLVE_VECTOR :
      {
       int dummy;

       if (Xsizes_output[r][i].m == NULL)
        {
          Xsizes_output[r][i].m = (int *)malloc(INTL);
        }
        read_int(comm,&dummy);
        read_int(comm,&dummy);
        read_int(comm,Xsizes_output[r][i].m);
        Xsizes_output[r][i].n = (int *)malloc(INTL);
        *(Xsizes_output[r][i].n) = 1;
        (Xsizes_output[r][i].l) = 0;
        total_size = *(Xsizes_output[r][i].m);
        break;
      }
      case NETSOLVE_SCALAR :
      {
        int dummy;

        read_int(comm,&dummy);
        read_int(comm,&dummy);
        Xsizes_output[r][i].m = (int *)malloc(INTL);
        Xsizes_output[r][i].n = (int *)malloc(INTL);
        *(Xsizes_output[r][i].m) = 1;
        *(Xsizes_output[r][i].n) = 1;
        (Xsizes_output[r][i].l) = 0;
        total_size = 1;
        break;
      }
      case NETSOLVE_FILE:
      {
        int dummy,size;
        read_int(comm,&dummy);
        read_int(comm,&dummy);
        read_int(comm,&size);

        Xsizes_output[r][i].m = (int *)malloc(INTL);
        Xsizes_output[r][i].n = (int *)malloc(INTL);
        *(Xsizes_output[r][i].m) = size; 
        *(Xsizes_output[r][i].n) = 1;
        (Xsizes_output[r][i].l) = 0;
        total_size = sizeof(char)*size;
        break;
      }
    }
    switch(dat)
    {
      case NETSOLVE_ASCII:
      {
        int fd;
        /* Write back to the file */
        fd = open(Xdata_output[r][i],O_WRONLY|O_CREAT,0644);
        if (fd < 0)
        {
          fprintf(stderr,"Impossible to open file '%s'\n",Xdata_output[r][i]);
          fprintf(stderr,"Aborting...\n");
          end_message(comm);
          end_com(Xdesc[r]); 
          return NetSolveFailure;
        }
        read_char_array_file(comm,fd,*(Xsizes_output[r][i].m));
        close(fd);
        break; 
      }
      case NETSOLVE_I :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (int *)malloc(INTL*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          read_int_array(comm,Xdata_output[r][i],total_size);
        }
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
          {
            read_int_array(comm,Xdata_output[r][i],total_size);
          }
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_int_array(comm, (int *)((int *)Xdata_output[r][i]+k*
                   Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        } 
        break;
      }
      case NETSOLVE_CHAR :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (char *)malloc(sizeof(char)*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
          read_char_array(comm,Xdata_output[r][i],total_size);
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
            read_char_array(comm,Xdata_output[r][i],total_size);
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_char_array(comm, 
                   (char *)((char *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }
      case NETSOLVE_B :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (char *)malloc(sizeof(char)*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

/*
        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
          read_byte_array(comm,Xdata_output[r][i],total_size);
        else
*/
        {      /* This is a matrix */
/*
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
            read_byte_array(comm,Xdata_output[r][i],total_size);
          else
*/
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_byte_array(comm,
                   (char *)((char *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }
      case NETSOLVE_S :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (float *)malloc(FLOATL*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
          read_single_array(comm,Xdata_output[r][i],total_size);
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
            read_single_array(comm,Xdata_output[r][i],total_size);
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_single_array(comm,(float *)((float *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }
      case NETSOLVE_D :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (double *)malloc(DOUBL*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }

        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
          read_double_array(comm,Xdata_output[r][i],total_size);
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
            read_double_array(comm,Xdata_output[r][i],total_size);
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_double_array(comm,(double *)((double *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }
      case NETSOLVE_C :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (scomplex *)malloc(SCOMPL*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          read_scomplex_array(comm,Xdata_output[r][i],total_size);
        }
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
          {
            read_scomplex_array(comm,Xdata_output[r][i],total_size);
          }
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_scomplex_array(comm,(scomplex *)((scomplex *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }
      case NETSOLVE_Z :
      {
        if (Xdata_output[r][i] == NULL)
        {
          Xdata_output[r][i] = (dcomplex *)malloc(DCOMPL*total_size);
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);
        }
        if (Xsizes_output[r][i].l == 0)
          Xsizes_output[r][i].l = *(Xsizes_output[r][i].m);

        if ((Xpb_desc[r]->output_obj[i].object_type) != NETSOLVE_MATRIX)
          read_dcomplex_array(comm,Xdata_output[r][i],total_size);
        else
        {      /* This is a matrix */
          if (Xsizes_output[r][i].l == *(Xsizes_output[r][i].m))
            read_dcomplex_array(comm,Xdata_output[r][i],total_size);
          else
            for (k=0;k< *(Xsizes_output[r][i].n);k++)
            {
              read_dcomplex_array(comm,(dcomplex *)((dcomplex *)Xdata_output[r][i]+k*Xsizes_output[r][i].l),
                   *(Xsizes_output[r][i].m));
            }
        }
        break;
      }

    }
  }

  end_message(comm);
  end_com(Xdesc[r]);
  job_completed(agent,XIPaddr[r]);
  request[r] = 0;
  
  /* Free the memory for this request */

  /* A LOT OF FREE TO DO HERE */

  for (i=0;i<Xpb_desc[r]->nb_obj_output;i++)
  {
    if (Xsizes_output[r][i].m != NULL)
      free(Xsizes_output[r][i].m);
    if (Xsizes_output[r][i].n != NULL)
      free(Xsizes_output[r][i].n);
  }
  free_pb_desc(Xpb_desc[r]);
  free(Xsizes_output[r]);
  free(Xsizes_input[r]);
  free(Xdata_input[r]);
  free(Xdata_output[r]);
  return NetSolveSuccess;
}

/*
 * Restart because of a server failure
 */

int restart(mode,r)
int mode;
int r;
{

  char *agent;
  int info;

 /*
  * Report the error to the communication server
  */

  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The environment variable NETSOLVE_AGENT should be set",
      __FILE__,__LINE__);
    return NetSolveSetAgent;
  }

  client_f_report(-1,agent,XIPaddr[r]);

 /*
  *  Submit the job again
  */

  info = nothread_solve(r,Xpb_desc[r],Xsizes_input[r],Xsizes_output[r],
          Xdata_input[r],Xdata_output[r]);
  return info;
}

#endif
