/*****************************************************************/
/*      nothread_solve.c                                         */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  nothread_solve()                                             */
/*****************************************************************/

#include "netsolve.h"

/*
 * this function is re-entrant (old gateway())
 */

#ifdef NOTHREADS

#ifdef __STDC__
int nothread_solve( int rentrance_request_nb,problem_desc *pb_desc,
          input_sizes *sizes_input,
          output_sizes *sizes_output,
          void **data_input, void **data_output)
#else
int nothread_solve(rentrance_request_nb,pb_desc,
          sizes_input, sizes_output,data_input,data_output)
int rentrance_request_nb;
problem_desc *pb_desc;
input_sizes *sizes_input;
output_sizes *sizes_output;
void **data_input, **data_output;
#endif
{
  communicator *comm;
  char *other_servers[LEN_RETURN_LIST];
  int expected[LEN_RETURN_LIST];
  unsigned int other_addr[LEN_RETURN_LIST];
  unsigned int current_addr;
  int current_data_format;
  int nb_other_servers;
  int data_format[LEN_RETURN_LIST];
  int my_data_format;
  int cc;
  int sendOK;
  int i,j,jj;
  int problem_size;
  int data_size;
  int result_size;
  int message_type;
  int submit;
  int status;
  int nb_failure;
  int sock; 
  int server_pid;
  int request_nb;
  int upf_index =0;
  int loop;
  int client_type = C_PROGRAM;
  int upf_type;
  FILE *upf;
  int dummy = 14;

  char *agent;

  char *my_arch;
  char full_name[256];

  (void)signal(SIGPIPE,sigpipe_handler);

 /****************************
  * Getting the Architecture *
  ****************************/

  if ((my_arch = (char *)getenv("NETSOLVE_ARCH")) == 0)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The NETSOLVE_ARCH environment variable should be set",__FILE__,__LINE__);
    return NetSolveSetArch;
  }
  my_data_format = arch_to_format(my_arch);


  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The NETSOLVE_AGENT environment variable should be set",__FILE__,__LINE__);
     return NetSolveSetAgent;
  }

 /* Get a request_nb */
  if (rentrance_request_nb == -1)
  {
    for (i=0;i<MAX_REQUEST;i++)
    {
      if (request[i] == 0)
      {
        request[i] = 1;
        request_nb = i;
        break;
      }
    }
  }
  else
    request_nb = rentrance_request_nb;
  
 

 /****************************************************
  * Asses the problem Size, data Size nd result size *
  ****************************************************/

  get_sizes(pb_desc,sizes_input,&data_size,&problem_size,&result_size);

  /**
   ** Main Loop !!
   **/

  while(1)
  {
    /* Assign the memory space for the servers' hostnames */
    for (j=0;j<LEN_RETURN_LIST;j++)
      other_servers[j] = (char *)calloc(MAX_HOST_NAME,sizeof(char));

    /* submit a request to the agent and accept a list of servers */
    submit = submit_job(agent,
               expected,(char **)other_servers,other_addr,&nb_other_servers,
               data_format,C_PROGRAM,problem_size,
               data_size,result_size,pb_desc);

    /* Has the submission been succesful ? */
    if (submit<0)
    {
      for (j=0;j<LEN_RETURN_LIST;j++)
        free(other_servers[j]);
      return submit;
    } 

#ifdef DEBUG
    fprintf(stderr,"I received a list of %d servers\n",nb_other_servers);
    for (j=0;j<nb_other_servers;j++)
      fprintf(stderr,"\t ---> %s\n",(char *)other_servers[j]);
#endif

    /* No servers available ? */
    if (nb_other_servers == 0)
    {
      sprintf(error_message,"%s \n\t\t[%s,%d]",
          "No computational resource for this problem",__FILE__,__LINE__);
      for (j=0;j<LEN_RETURN_LIST;j++)
        free(other_servers[j]);
      return NetSolveNoServers;
    }

    message_type = PROBLEM_SOLVE;
    nb_failure = 0;

    /* contact the servers, in order */
    for (j=0;j<nb_other_servers;j++)
    {
      current_addr = other_addr[j];
      current_data_format = data_format[j];
#ifdef VIEW
      fprintf(stderr,"contacting server %s ...\n",other_servers[j]);
#endif

      /* Create the socket connection */
      sock = connect_to_server(other_servers[j],other_addr[j]);

      /* Has the connection been succesful ? */
      if(sock == -3)
      {
        sprintf(error_message,"%s \n\t\t[%s,%d]",
          "NetSolve internal failure",__FILE__,__LINE__);
        for (j=0;j<LEN_RETURN_LIST;j++)
          free(other_servers[j]);
        return NetSolveFailure;
      }
      if (sock == -2)
      {
        nb_failure++;
        if (nb_failure != nb_other_servers) /* I go to the next server */
          continue;
        else
          break;
      }
      if (sock == -1)
      {
        client_f_report(0,agent,current_addr);
        if (nb_failure != nb_other_servers) /* I go to the next server */
          continue;
        else
          break;
      }
    
      if (nb_failure == nb_other_servers) /* I've exhausted the list */
        break;

#ifdef DEBUG
      fprintf(stderr,"Applying for a computation ...\n");
#endif

      /* XDR or not ? */
      if (current_data_format == my_data_format)
        comm = init_send(DATA_RAW,sock);
      else
        comm = init_send(DATA_XDR,sock);

      sendOK = 1;
      /* Send the application */
      if (write_int(comm,&message_type) == -1)
        sendOK = 0;
      if (write_string(comm,pb_desc->code) == -1)
        sendOK = 0;
      if (write_string(comm,agent) == -1)
        sendOK = 0;
      if (write_int(comm,&client_type) == -1)
        sendOK = 0;
      if (write_int(comm,&(pb_desc->nb_obj_input)) == -1)
        sendOK =0;
      for (loop = 0;loop < pb_desc->nb_obj_input; loop++)
      {
        if (write_int(comm,&(pb_desc->input_obj[loop].data_type)) == -1)
          sendOK=0;
        if (write_int(comm,&(pb_desc->input_obj[loop].object_type)) == -1)
          sendOK = 0;
      } 
      if (write_int(comm,&(pb_desc->nb_obj_output)) == -1)
        sendOK = 0;
      for (loop = 0;loop < pb_desc->nb_obj_output; loop++)
      {
        if (write_int(comm,&(pb_desc->output_obj[loop].data_type)) == -1)
          sendOK = 0;
        if (write_int(comm,&(pb_desc->output_obj[loop].object_type)) == -1)
          sendOK = 0;
      } 
      end_message(comm);
      if (!sendOK)
      {
        end_com(sock); 
        client_f_report(-1,agent,current_addr);
        nb_failure++; 
        continue;
      }

      /* getting the reply from the server */
      comm = init_recv(sock);
      if (read_int(comm,&status) == -1)
      {
        end_message(comm);
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++; 
        continue;
      }
      end_message(comm);

      /* Processing that reply */
      if (status == PB_REFUSED)
      {  /* The server cannot solve the problem */
        end_message(comm);
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++;
        continue; /* Proceed with the next server */
      }
      if (status == SV_FAILURE)
      {    /* A server has failed */
        end_message(comm);
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++;
        continue; /* Proceed with the next server */
      }
      if ((status== NOT_AUTHORIZED)||(status==TOKENS_OVERFLOW))
      {
        end_message(comm);
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++; 
        continue; /* Proceed with the next server */
      } 

      /* The server has accepted the request */

#ifdef DEBUG
      fprintf(stderr,"Sending data...\n");
#endif

      /* Take care of upf's */

      for (i=0;i<pb_desc->nb_obj_input;i++)
      {
        if (pb_desc->input_obj[i].object_type == NETSOLVE_UPF)
        {
          int size;
          struct stat st;
          char *buf;
          char fcnname[256];
          char subroutinename[256];
          char *ptr;
    
          /* Get the file name */
    
          strcpy(fcnname,(char *)data_input[i]);
          sprintf(full_name,"%s.f",fcnname);
#ifdef DEBUG
          fprintf(stderr,"Trying to open '%s'\n",full_name);
#endif 
          upf = fopen(full_name,"r");
          if (upf == NULL)
          {
            sprintf(full_name,"%s.c",fcnname); 
            upf = fopen(full_name,"r");
            if (upf == NULL)
            {
#ifdef DEBUG
              fprintf(stderr,"File '%s.c' or '%s.f' not found.\n",
                             fcnname,fcnname);
#endif
              sprintf(error_message,"'%s.[f|c]'  : %s \n\t\t[%s,%d]",
                fcnname,"no such file",__FILE__,__LINE__);
              for (j=0;j<LEN_RETURN_LIST;j++)
                free(other_servers[j]);
              return NetSolveFileNotFound;
            }
            else
              upf_type = LANG_C;
          }
          else
            upf_type = LANG_FORTRAN;
    
          /* We read the file */
    
          stat(full_name,&st);
          buf = (char *)malloc(sizeof(char)*(st.st_size+1));
          if (fread(buf,1,st.st_size,upf) != st.st_size)
          {
            sprintf(error_message,"%s '%s' \n\t\t[%s,%d]",
              "Read error on file  ",full_name,__FILE__,__LINE__);
            for (j=0;j<LEN_RETURN_LIST;j++)
              free(other_servers[j]);
    
            return NetSolveFailure;
          }
          buf[st.st_size] = '\0';
          fclose(upf);
          size = (int)(st.st_size);
    
          /* We send it on the socket */
          if (upf_index == 0)
          {
            if (current_data_format == my_data_format)
              comm = init_send(DATA_RAW,sock);
            else
              comm = init_send(DATA_XDR,sock);
          }
          write_int(comm,&upf_type);
          /*  get only the last part of the path to the upf */
          strcpy(subroutinename,fcnname);
          ptr = &(subroutinename[strlen(subroutinename)-1]);
          while((ptr!=subroutinename)&&(*ptr!='/'))
          {
            fprintf(stderr,"*prt = '%c'\n",*ptr);
            ptr--;
          }
          if (*ptr=='/')
            ptr++;
          write_string(comm,ptr);
          write_int(comm,&size);
          write_char_array(comm,buf,size);
    
          upf_index++;
        }
      }
    
      /* Wating for the UPF ACK if necessary */
      if (upf_index != 0)
      {
        int message;
    
        end_message(comm);
        init_recv(sock);
        read_int(comm,&message);
        if (message == UPF_FAILED)
        {
          end_message(comm);
          end_com(sock);
          sprintf(error_message,"%s \n\t\t[%s,%d]",
            "The computational server was not able to incorporate your UPF",
            __FILE__,__LINE__);
          for (j=0;j<LEN_RETURN_LIST;j++)
            free(other_servers[j]);
          return NetSolveUPFFailed;
        }
        if (message == UPF_UNSAFE)
        {
          char tmp[256];
          int upf_unsafe_index;
    
          read_int(comm,&upf_unsafe_index);
          read_string(comm,tmp);
          end_message(comm);
          end_com(sock);
          sprintf(error_message,"%s \n\t\t[%s,%d]%s %s",
            "The computational server deemed your %dthUPF unsafe !!",
            __FILE__,__LINE__,"\nInvalid calls to : ",upf_unsafe_index,tmp);
          for (j=0;j<LEN_RETURN_LIST;j++)
            free(other_servers[j]);
          return NetSolveUPFUnsafe;
        }
      }
    
      /* Sending the input objects */

      /* XDR or not ? */
      if (current_data_format == my_data_format)
        comm = init_send(DATA_RAW,sock);
      else
        comm = init_send(DATA_XDR,sock);

      sendOK = 1;
      for (jj = 0; jj< pb_desc->nb_obj_input;jj++)
      {
        int m,n;

        if (pb_desc->input_obj[jj].object_type == NETSOLVE_MATRIX)
        {
#ifdef DEBUG
          fprintf(stderr,"Sending a matrix..\n");
#endif
          /* Sending the struct and the data type */
          if (write_int(comm,&(pb_desc->input_obj[jj].object_type)) == -1)
            sendOK = 0;
          if (write_int(comm,&(pb_desc->input_obj[jj].data_type)) == -1)
            sendOK = 0;

          /*  Sending its major */
          if (write_int(comm,&Xmajor) == -1)
            sendOK = 0;

          /* Sending the size   */
          m = sizes_input[jj].m;
          n = sizes_input[jj].n;
          if (write_int(comm,&m) == -1)
            sendOK = 0;
          if (write_int(comm,&n) == -1)
            sendOK = 0;
        }
        if (pb_desc->input_obj[jj].object_type == NETSOLVE_VECTOR)
        {
#ifdef DEBUG
           fprintf(stderr,"Sending a vector..\n");
#endif
          /* Sending the struct and the data type */
          if (write_int(comm,&(pb_desc->input_obj[jj].object_type)) == -1)
            sendOK = 0;
          if (write_int(comm,&(pb_desc->input_obj[jj].data_type)) == -1)
            sendOK = 0;
          m = sizes_input[jj].m;
          sizes_input[jj].n = 1;
          sizes_input[jj].l = m;
          n = 1;
          if (write_int(comm,&m) == -1)
            sendOK = 0;
        } 
        if (pb_desc->input_obj[jj].object_type == NETSOLVE_SCALAR)
        {
#ifdef DEBUG
          fprintf(stderr,"Sending a scalar..\n");
#endif
          /* Sending the struct and the data type */
          if (write_int(comm,&(pb_desc->input_obj[jj].object_type)) == -1)
            sendOK = 0;
          if (write_int(comm,&(pb_desc->input_obj[jj].data_type)) == -1)
            sendOK = 0;
          sizes_input[jj].n = 1;
          sizes_input[jj].m = 1;
          sizes_input[jj].l = 1;
          m = 1;
          n = 1;
        }
        if (pb_desc->input_obj[jj].object_type == NETSOLVE_FILE)
        {
          struct stat st;
#ifdef DEBUG
           fprintf(stderr,"Sending a file..\n");
#endif
          /* Sending the struct and the data type */
          if (write_int(comm,&(pb_desc->input_obj[jj].object_type)) == -1)
            sendOK = 0;
          if (write_int(comm,&(pb_desc->input_obj[jj].data_type)) == -1)
            sendOK = 0;
          stat(data_input[jj],&st);
          m = (int)st.st_size;
          sizes_input[jj].m = m;
          sizes_input[jj].n = 1;
          sizes_input[jj].l = m;
          n = 1;
          if (write_int(comm,&m) == -1)
            sendOK = 0;
        }

        if (sizes_input[jj].m == sizes_input[jj].l)
        { /* No Leading Dimension problem */
          switch(pb_desc->input_obj[jj].data_type)
          {
            case NETSOLVE_ASCII:
            {
              int fd;

              fd = open(data_input[jj],O_RDONLY);
              if (fd < 0)
              {
                fprintf(stderr,"Impossible to open file '%s'\n",data_input[jj]);
                fprintf(stderr,"Aborting...\n");
                sendOK = 0;
                break;
              }
              write_char_array_file(comm,fd,sizes_input[jj].m);
              close(fd);
              break;
            }
            case NETSOLVE_I :
            {
              if (write_int_array(comm,(int *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            case NETSOLVE_B :
            {
              int k;
              /* SPECIAL CASE !!!! */
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_byte_array(comm,(char *)((char *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_CHAR :
            {
              if (write_char_array(comm,(char *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            case NETSOLVE_S :
            {
              if (write_single_array(comm,(float *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            case NETSOLVE_D :
            {
              if (write_double_array(comm,(double *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            case NETSOLVE_C :
            {
              if (write_scomplex_array(comm,(scomplex *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            case NETSOLVE_Z :
            {
              if (write_dcomplex_array(comm,(dcomplex *)(data_input[jj]),m*n) == -1)
                sendOK = 0;
              break;
            }
            default :
              break;
          } 
        }
        else /* Take of the leading dimensions ..... */
        {
          int k;
    
          switch(pb_desc->input_obj[jj].data_type)
          {
            case NETSOLVE_I :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_int_array(comm,(int *)((int *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_CHAR :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_char_array(comm,(char *)((char *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_B :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_byte_array(comm,(char *)((char *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_S :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_single_array(comm,(float *)((float *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_D :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_double_array(comm,(double *)((double *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                sendOK = 0;
              }
              break;
            }
            case NETSOLVE_C :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_scomplex_array(comm,(scomplex *)((scomplex *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            case NETSOLVE_Z :
            {
              for (k=0;k<sizes_input[jj].n;k++)
              {
                if (write_dcomplex_array(comm,(dcomplex *)((dcomplex *)(data_input[jj])+
                              k*(sizes_input[jj].l)),sizes_input[jj].m) == -1)
                  sendOK = 0;
              }
              break;
            }
            default :
              break;
          }
        }
      }
      end_message(comm);
      if (!sendOK)
      {
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++;
        continue;
      }

      /* Waiting for the 'pid' of the computing process */
      comm = init_recv(sock);
      if (read_int(comm,&server_pid) == -1)
      {
        end_message(comm);
        end_com(sock);
        client_f_report(-1,agent,current_addr);
        nb_failure++;
        continue;
      }
      end_message(comm);
      if (current_data_format == my_data_format)
        comm = init_send(DATA_RAW,sock);
      else
        comm = init_send(DATA_XDR,sock);
      write_int(comm,&dummy);
      end_message(comm);

      /* At this point, all the data is supposedly on its way */
      /* And I exit the 'for' loop.                           */
      break; 

    } /* end for */
  
    /* Freeing some memory */
    for (j=0;j<LEN_RETURN_LIST;j++)
      free(other_servers[j]);

    /* All the servers have failed ??? */
    if (nb_failure == nb_other_servers) 
    {
      if (nb_other_servers < LEN_RETURN_LIST) 
      {       /* have I exhausted the possibilities ?*/
        end_message(comm);
        end_com(sock);
        sprintf(error_message,"%s \n\t\t[%s,%d]",
          "No computational resource for this problem",__FILE__,__LINE__);
        return NetSolveNoServers;
      }
      else /* Submit the problem again */
      {
        end_message(comm);
        end_com(sock);
        continue; /* Go to the beginning of the while loop, resubmit, etc.. */
      }
    }
    /* At this point, a server is actually taking care of the computation */

    /* Return to main thread of control */
    Xpb_desc[request_nb] = pb_desc;
    Xsizes_input[request_nb] = sizes_input;
    Xdata_input[request_nb] = data_input;
    Xsizes_output[request_nb] = sizes_output;
    Xdata_output[request_nb] = data_output;
    Xdesc[request_nb] = sock;
    XIPaddr[request_nb] = current_addr;
    Xserver_pid[request_nb] = server_pid;
    return request_nb;
  
  }  /* end while */
}

#endif
