/*****************************************************************/
/*	socket_util.c                                            */
/*	Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/* int connect_to_agent()                                        */
/* int connect_to_server()                                       */
/* int connect_to_socket()                                       */
/*****************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include "global.h"
#include "bfunc.h"

#define HOSTNAME_LENGTH 256

/*
 * connect_to_agent()
 *
 *   returns -3 if could not create socket
 *   returns -2 if could not locate host
 *   returns -1 if connection could not be established
 */
int connect_to_agent(hostname,IPaddr)
char *hostname;
unsigned int IPaddr;
{
  return connect_to_socket(hostname,IPaddr,AG_PORT);
}


/*
 * connect_to_server()
 *
 *   returns -3 if could not create socket
 *   returns -2 if host could not be located
 *   returns -1 if connection could not be established
 */
int connect_to_server(hostname,IPaddr)
char *hostname;
unsigned int IPaddr;
{
  return connect_to_socket(hostname,IPaddr,SV_PORT);
}


/*
 * connect_to_socket()
 *
 *   returns -3 if could not create socket
 *   returns -2 if host could not be located
 *   returns -1 if connection could not be established
 */

int connect_to_socket(hostname,IPaddr,port)
char *hostname;
unsigned int IPaddr;
int port;
{
  int sock;
  int try;

  struct sockaddr_in inet_addr;
  struct hostent *hp;

  if ((sock = socket(AF_INET,SOCK_STREAM,0)) == -1)
  {
    perror("connect_to_socket() socket()");
    return -3;
  }
  BZERO(&inet_addr,sizeof(inet_addr));

  /* Try to do a gethostbyname() */
  hp = gethostbyaddr((char *)&IPaddr,sizeof(unsigned int),AF_INET); 
  if (hp == NULL)
  {
    /* Try to do a gethostbyaddr() */
    hp = gethostbyname(hostname);
    if (hp == NULL)
    {
      close(sock);
      return -2;
    }
  }
  BCOPY(hp->h_addr,&inet_addr.sin_addr,hp->h_length);
  inet_addr.sin_family = AF_INET;
  inet_addr.sin_port = htons(port);
  
#ifdef DEBUG
  fprintf(stderr,"Connecting to %s on port '%d'\n",hostname,port);
#endif
  if (connect(sock,(struct sockaddr *)&inet_addr,sizeof(inet_addr)) == -1)
  {
    close(sock);
    sock = -1;
  }
#ifdef DEBUG
  fprintf(stderr,"Connected to %s on port '%d'\n",hostname,port);
#endif

  return sock;
}
