/*****************************************************************/
/*      thread_get_result.c                                      */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  thread_get_result()                                          */
/*****************************************************************/

#include "netsolve.h"

#ifdef __STDC__
int thread_get_result(communicator *server_comm,
               char *agent, int sock,int server_pid, 
               problem_desc *pb_desc,
               output_sizes *sizes_output,void **data_output)
#else
int thread_get_result(server_comm,agent,sock,server_pid,pb_desc,
               sizes_output,data_output)
communicator *server_comm;
char *agent;
int sock;
int server_pid;
problem_desc *pb_desc;
output_sizes *sizes_output;
void **data_output;
#endif
{
  int hbm_sock;
  int failed = 0;
  int hbm_activated = 1;
  int server_status;
  int status;
  int readOK;
  int stdoutlength;
  char *stdoutbuf;
  int i,k;
  communicator *hbm_comm;

  (void)signal(SIGPIPE,sigpipe_handler);

  /*  Connect to the HBM */
  hbm_sock = connect_to_socket(agent, 0, HBM_INTERFACE_PORT);
  if (hbm_sock<0)
  {
#ifdef DEBUG
    fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
    hbm_activated = 0;
  }
  else 
  {
    hbm_comm = init_send(DATA_XDR,hbm_sock);
    if (write_int(hbm_comm,&server_pid) == -1)
    {
#ifdef DEBUG
      fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
      hbm_activated = 0;
    }
    end_message(hbm_comm);
  }
  
  while(hbm_activated)
  {
    int try;
    int on = 1;
    int off =0;
    char buf[1];
    
    ioctl(sock,FIONBIO,&on);
    try = recv(sock,buf,1,MSG_PEEK);
    ioctl(sock,FIONBIO,&off);
 
    if (try == 0) /* No connection anymore !! */
    {
      end_com(hbm_sock);
      end_com(sock);
      return NetSolveFailure;
    }
    if (try != -1)
      break;
   
    /* Not ready yet */
    if (hbm_activated)  
    { /* Check the HBM */
      if ((hbm_comm = init_recv(hbm_sock)) == NULL)
      {
#ifdef DEBUG
        fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      if (read_int(hbm_comm,&server_status) == -1)
      { 
#ifdef DEBUG
        fprintf(stderr,"Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_message(hbm_comm);
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      end_message(hbm_comm);
      switch(server_status)
      {
        case 1:
        case 2:
        case 3:
        case 4:
          break; /* just continue in the while loop */

        default: 
          end_com(hbm_sock);
          end_com(sock);
          return NetSolveFailure;
          break;
      }
    }
    sleep(2);
  }
  
  if (hbm_activated)
    end_com(hbm_sock);

  server_comm = init_recv(sock);
  fflush(stderr);

  /* At this point, I am receiveing stuff from the server !!! cool */
  readOK = 1;
  if (read_int(server_comm,&status) == -1)
    readOK=0;
  
  if (read_int(server_comm,&stdoutlength) == -1)
    readOK=0;

  fflush(stderr);

  if (readOK==0)
    stdoutlength = 0;
  stdoutbuf = (char *)calloc(stdoutlength+1,sizeof(char));
  if (read_char_array(server_comm,stdoutbuf,stdoutlength) == -1)
    readOK=0;
  if (!readOK)
  {
    end_message(server_comm);
    end_com(sock);
    return NetSolveFailure;
  }
  if (stdoutlength > 1)
  {
    fprintf(stdout,"-- stdout from request --\n");
    fprintf(stdout,"%s\n",stdoutbuf);
  }
  if (stdoutbuf != NULL)
    free(stdoutbuf);
  
  if (status == SV_FAILURE)
  {
    end_message(server_comm);
    end_com(sock);
    return NetSolveFailure;
  }

  if (status == PB_NO_SOL)
  {
    end_message(server_comm);
    end_com(sock);
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "The numerical problem does not admit any solution",
      __FILE__,__LINE__);
    return NetSolveNoSolution; 
  }
  if (status == DM_MISMATCH)
  {
    end_message(server_comm);
    end_com(sock);
    sprintf(error_message,"%s \n\t\t[%s,%d]",
      "Dimensions of algebrix objects are mismatched",
      __FILE__,__LINE__);
    return NetSolveBadDimension;
  }
 
  /* The server has returned a numerical result !! */

  readOK = 1;

  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    int total_size;
    int obj,dat;

    obj = pb_desc->output_obj[i].object_type;
    dat = pb_desc->output_obj[i].data_type;

    switch(obj)
    {
      case NETSOLVE_MATRIX :
      {
        int maj,dummy;

        /* Useless, since the server makes sure it's in
           the right major anyway                       */
        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        if (read_int(server_comm,&maj) == -1)
          readOK = 0;


        if (sizes_output[i].m == NULL)
          sizes_output[i].m = (int *)calloc(1,INTL);
        if (read_int(server_comm,sizes_output[i].m) == -1)
          readOK =0;

        if (sizes_output[i].n == NULL)
          sizes_output[i].n = (int *)calloc(1,INTL);
        if (read_int(server_comm,sizes_output[i].n) == -1)
          readOK = 0;

        break;
      }
      case NETSOLVE_VECTOR :
      {
       int dummy;

       if (sizes_output[i].m == NULL)
          sizes_output[i].m = (int *)calloc(1,INTL);

        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        if (read_int(server_comm,sizes_output[i].m) == -1)
          readOK = 0;

        sizes_output[i].n = (int *)calloc(1,INTL);
        *(sizes_output[i].n) = 1;
        (sizes_output[i].l) = 0;
        break;
      }
      case NETSOLVE_SCALAR :
      {
        int dummy;

        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        if (read_int(server_comm,&dummy) == -1)
          readOK = 0;
        sizes_output[i].m = (int *)calloc(1,INTL);
        sizes_output[i].n = (int *)calloc(1,INTL);
        *(sizes_output[i].m) = 1;
        *(sizes_output[i].n) = 1;
        (sizes_output[i].l) = 0;
        break;
      }
      case NETSOLVE_FILE:
      {
        int dummy,size;

        read_int(server_comm,&dummy);
        read_int(server_comm,&dummy);
        read_int(server_comm,&size);

        sizes_output[i].m = (int *)malloc(INTL);
        sizes_output[i].n = (int *)malloc(INTL);
        *(sizes_output[i].m) = size;
        *(sizes_output[i].n) = 1;
        (sizes_output[i].l) = 0;
        total_size = sizeof(char)*size;
        break;
      }
    }
    if (!readOK)
      break;
    total_size = (*(sizes_output[i].m))*(*(sizes_output[i].n));
    if (total_size <0)
    {
      readOK=0;
      break;
    }
    switch(dat)
    {
      case NETSOLVE_ASCII:
      {
        int fd;
        /* Write back to the file */
        fd = open(data_output[i],O_WRONLY|O_CREAT,0644);
        if (fd < 0)
        {
          fprintf(stderr,"Impossible to open file '%s'\n",data_output[i]);
          fprintf(stderr,"Aborting...\n");
          end_message(server_comm);
          end_com(sock);
          return NetSolveFailure;
        }
        read_char_array_file(server_comm,fd,*(sizes_output[i].m));
        close(fd);
        break;
      }
      case NETSOLVE_I :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (int *)calloc(1,INTL*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_int_array(server_comm,data_output[i],total_size) == -1)
            readOK = 0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_int_array(server_comm,data_output[i],total_size) == -1)
              readOK = 0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_int_array( server_comm,(int *)((int *)data_output[i]+k*
                   sizes_output[i].l), *(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
      case NETSOLVE_CHAR :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (char *)calloc(1,sizeof(char)*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_char_array(server_comm,data_output[i],total_size) == -1)
            readOK = 0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_char_array(server_comm,data_output[i],total_size) == -1)
              readOK = 0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_char_array(server_comm,
                   (char *)((char *)data_output[i]+k*sizes_output[i].l),
                   *(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
      case NETSOLVE_B :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (char *)calloc(1,sizeof(char)*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        for (k=0;k< *(sizes_output[i].n);k++)
        {
          if (read_byte_array(server_comm,
               (char *)((char *)data_output[i]+k*sizes_output[i].l),
               *(sizes_output[i].m)) == -1)
            readOK = 0;
        }
        break;
      }
      case NETSOLVE_S :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (float *)calloc(1,FLOATL*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_single_array(server_comm,data_output[i],total_size) == -1)
            readOK = 0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_single_array(server_comm,data_output[i],total_size) == -1)
              readOK = 0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_single_array(server_comm,(float *)((float *)data_output[i]+
                   k*sizes_output[i].l),*(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
      case NETSOLVE_D :
      {
        if (data_output[i] == NULL)
        {
          fprintf(stderr,"CALLOCING\n");
          data_output[i] = (double *)calloc(1,DOUBL*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }

        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_double_array(server_comm,data_output[i],total_size) == -1)
            readOK=0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_double_array(server_comm,data_output[i],total_size) == -1)
              readOK =0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_double_array(server_comm,(double *)((double *)data_output[i]+
                   k*sizes_output[i].l), *(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
      case NETSOLVE_C :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (scomplex *)calloc(1,SCOMPL*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_scomplex_array(server_comm,data_output[i],total_size) == -1)
            readOK = 0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_scomplex_array(server_comm,data_output[i],total_size) == -1)
              readOK = 0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_scomplex_array(server_comm,(scomplex *)((scomplex *)data_output[i]+
                         k*sizes_output[i].l), *(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
      case NETSOLVE_Z :
      {
        if (data_output[i] == NULL)
        {
          data_output[i] = (dcomplex *)calloc(1,DCOMPL*total_size);
          sizes_output[i].l = *(sizes_output[i].m);
        }
        if (sizes_output[i].l == 0)
          sizes_output[i].l = *(sizes_output[i].m);

        if ((pb_desc->output_obj[i].object_type) != NETSOLVE_MATRIX)
        {
          if (read_dcomplex_array(server_comm,data_output[i],total_size) == -1)
            readOK = 0;
        }
        else
        {      /* This is a matrix */
          if (sizes_output[i].l == *(sizes_output[i].m))
          {
            if (read_dcomplex_array(server_comm,data_output[i],total_size) == -1)
               readOK = 0;
          }
          else
            for (k=0;k< *(sizes_output[i].n);k++)
            {
              if (read_dcomplex_array(server_comm,(dcomplex *)((dcomplex *)data_output[i]+
                           k*sizes_output[i].l),*(sizes_output[i].m)) == -1)
                readOK = 0;
            }
        }
        break;
      }
    }
  }
 
  end_message(server_comm);
  end_com(sock);

  if (!readOK)
  {
    return NetSolveFailure;
  }

  return NetSolveSuccess;
}
