/**
* CharParser
*   
* This class contains methods to parse character data.  These are
* primarily called from the NetSolveInt{Scalar,Vector,Matrix}
* classes.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.StringTokenizer;

public class CharParser {

  /**
   *
   *  This method parses a line of text into an array of
   * characters.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of characters, cast to Object.
   *
   **/

  public static Object parseLine(String line, StringBuffer err) {
    boolean err_status = false;
    int items = 0;
    String temp = null;
    StringTokenizer st2;
    int tokCount;
    char [] char_arr;

    /*
     * For characters, retokenize the line with a new
     * set of delimiters: space, tab, cr, newline and
     * tick.  this way the user can enter the characters
     * by themselves, or enclose them in tick-marks as
     * implied by the descriptions.
     */
    st2 = new StringTokenizer(line," \t\r\n'");
    tokCount = st2.countTokens();
    char_arr = new char [tokCount];

    while(tokCount-- > 0) {
      temp = st2.nextToken();

      if(temp.length() > 1) {
        // If this token contains more than one character, generate
        // an error message.
        err_status = true;
        err.append(Labels.char_expected[Labels.language]+temp+"\n");
      }

      try {
        char_arr[items] = new Character(temp.charAt(0)).charValue();
      }catch (NumberFormatException e) {
        err_status = true;
        err.append(Labels.invalid_char[Labels.language]+temp+"\n");
      }
      items++;
    }

    if(err_status)
      return null;
    else
      return char_arr;
  }
}
