import java.io.IOException;
import java.io.BufferedReader;
import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.StringBuffer;

public class ComplexParser {
  
  /**
   * 
   * This method parses the input string into an array of floats or
   * doubles depending on the input object's data type.  The numbers may
   * be in standard or scientfic notation.  The parsing is done by
   * scanning through the input string character-by-character to 
   * split the text into distinct numbers and to verify that the correct 
   * format is used.
   * 
   * @param line The line to parse.
   * @param err StringBuffer in which to append error messages.
   *
   * @return Returns an array of numbers parsed from input string.
   *
   **/

  public static double [] parseComplex(String line, StringBuffer err) {
    String input, token;
    StringTokenizer st;
    boolean extra, img_neg, error=false;
    int idx, real_start, real_end, img_start, img_end, count, tok_count;
    char [] tok;
    double [] temp_array = null;
    double [] retval = null;

    st = new StringTokenizer(line,"i");
 
    temp_array = new double[2 * st.countTokens()];

    if(GlobalDefs.DEBUG)
      System.out.println("Allocating " + 2 * st.countTokens() + 
        " elements for temp_array[]");

    count = 0;
    tok_count = 0;

    while(st.hasMoreTokens()) {
      tok_count++;
      token = st.nextToken();
      tok = token.toCharArray();
      if(GlobalDefs.DEBUG)
        System.out.println("Token: " + token);
      idx = 0;
      try {
        try {
          while((tok[idx] == ' ') || (tok[idx] == '\t'))
            idx++; // skip blanks 
        }catch(ArrayIndexOutOfBoundsException e) {
          if(GlobalDefs.DEBUG)
            System.out.println("empty token");
          tok_count--;
          continue;
        }
        if(GlobalDefs.DEBUG)
          System.out.println("first char is: " + tok[idx]);
        real_start = idx;

        if((tok[idx] == '+') || (tok[idx] == '-'))
          idx++;  // If this char is + or -, inc. the index and continue

        while((tok[idx] >= '0') && (tok[idx] <= '9')) 
          idx++;  // Skip past the digits

        if(tok[idx] == '.')
          idx++;  // If this char is a decimal point, inc. and continue

        while((tok[idx] >= '0') && (tok[idx] <= '9')) 
          idx++;  // Skip past any digits after the decimal point

        if((tok[idx] == 'e') || (tok[idx] == 'E')) {
          // If we see 'e' or 'E', then this is in scientific notation
          idx++;
          if((tok[idx] == '+') || (tok[idx] == '-'))
            idx++;  // Continue past the + or -

          while((tok[idx] >= '0') && (tok[idx] <= '9')) 
            idx++;  // Skip all the digits (for the exponent)
        }
        real_end = idx;  // mark the end of the real part

        while((tok[idx] == ' ') || (tok[idx] == '\t'))
          idx++; // skip blanks 

         /*
          * Following the real part, there should be either a + or -
          * followed by the imaginary part.  If there's no + or -, generate
          * an error message.
          */

        if(tok[idx] == '+')
          img_neg = false;
        else if(tok[idx] == '-')
          img_neg = true;
        else {
          if(GlobalDefs.DEBUG)
            System.out.println("Error, expected a + or -");
          error = true;
          err.append(Labels.invalid_complex[Labels.language] + token + "\n");
          continue;
        }

        idx++;
        try {
          while((tok[idx] == ' ') || (tok[idx] == '\t'))
            idx++; // skip blanks 
        }catch(ArrayIndexOutOfBoundsException e) {
          if(GlobalDefs.DEBUG)
            System.out.println("no imaginary part");
          error = true;
          err.append(Labels.no_imaginary[Labels.language] + 
            token + "\n");
          continue;
        } 

        if(GlobalDefs.DEBUG)
          System.out.println("first char is: " + tok[idx]);

        img_start = idx;  // mark beginning of imaginary part
        try {
          while((tok[idx] >= '0') && (tok[idx] <= '9')) 
            idx++;  // Skip digits
        }catch(ArrayIndexOutOfBoundsException e) {
           // if we run out of bounds still reading digits, that's ok

           if(GlobalDefs.DEBUG)
             System.out.println("tok ended with digits...OK");

           img_end = idx;  // Mark the end of the imaginary part

           temp_array[count] = 
              new Double(token.substring(real_start,real_end)).doubleValue();
            
           if(img_neg)
             temp_array[count+1] = new Double("-" + 
                token.substring(img_start,img_end)).doubleValue();
           else
             temp_array[count+1] =
               new Double(token.substring(img_start,img_end)).doubleValue();

           count += 2;
           continue;
        }

        if(tok[idx] == '.')
          idx++;  // Continue past decimal point
        try {
          while((tok[idx] >= '0') && (tok[idx] <= '9')) 
            idx++;  // Skip digits
        }catch(ArrayIndexOutOfBoundsException e) {
          // Again, if we're reading digits and hit the end of the
          // line, that's ok

          if(GlobalDefs.DEBUG)
            System.out.println("at end of line.. OK");

          img_end = idx;  // Mark the end of the imaginary part

          temp_array[count] = 
             new Double(token.substring(real_start,real_end)).doubleValue();
         
          if(img_neg)
            temp_array[count+1] = new Double("-" + 
               token.substring(img_start,img_end)).doubleValue();
          else
            temp_array[count+1] =
              new Double(token.substring(img_start,img_end)).doubleValue();

          count += 2;
          continue;
        }

        if((tok[idx] == 'e') || (tok[idx] == 'E')) {
          // this number is in scientific notation

          idx++;

          if((tok[idx] == '+') || (tok[idx] == '-'))
            idx++;  // Continue past the + or -
          try {
            while((tok[idx] >= '0') && (tok[idx] <= '9')) 
              idx++;  // Skip digits
          }catch(ArrayIndexOutOfBoundsException e) {
            // Here we're hitting the end of the line after reading some
            // digits making up the exponent of the scientific number

            if(GlobalDefs.DEBUG)
              System.out.println("at end of line.. OK");

            img_end = idx;  // Mark the end of the imaginary part

            temp_array[count] = 
               new Double(token.substring(real_start,real_end)).doubleValue();
             
            if(img_neg)
              temp_array[count+1] = new Double("-" + 
                 token.substring(img_start,img_end)).doubleValue();
            else
              temp_array[count+1] =
                new Double(token.substring(img_start,img_end)).doubleValue();
 
            count += 2;
            continue;
          }
        }
         
        img_end = idx;  // Mark the end of the imaginary part

        extra = false;

        while(idx < token.length()) {
          if((tok[idx] != ' ') && (tok[idx] != '\t')) {
            // if there's anything other than spaces or tabs following
            // the token, generate an error
            extra = true;
            break;
          }
          idx++;
        }

        if(extra) {
          if(GlobalDefs.DEBUG)
            System.out.println("Extraneous stuff at end of token");
          error = true;
          err.append(Labels.invalid_complex[Labels.language] + token + " \n");
          continue;
        }
        
        temp_array[count] = 
           new Double(token.substring(real_start,real_end)).doubleValue();
       
        if(img_neg)
          temp_array[count+1] = new Double("-" + 
             token.substring(img_start,img_end)).doubleValue();
        else
          temp_array[count+1] =
            new Double(token.substring(img_start,img_end)).doubleValue();

        count += 2;
       }catch(ArrayIndexOutOfBoundsException e) {
         /*
          * this is sort of a catch-all... if we unexpectedly hit the
          * end of line anywhere in parsing, just generate an
          * 'imcomplete complex number' error.
          */

         if(GlobalDefs.DEBUG)
           System.out.println("Incomplete Complex Number: "+token);
         error = true;
         err.append(Labels.incomplete_complex[Labels.language] + token + "\n");
       }
    } // end while

    if(GlobalDefs.DEBUG) {
      System.out.println("tok_count = " + tok_count);
      System.out.println("Complex array: " + temp_array.length + " elements");
      for(idx=0;idx<temp_array.length;idx+=2)
        System.out.println(temp_array[idx] + " " + temp_array[idx+1]);
    }
    
    retval = new double [tok_count * 2];

    System.arraycopy(temp_array,0,retval,0,tok_count*2);

    if(error) 
      return(null);
    else
      return(retval);
  } // end method parseComplex

}
