/**
* DoubleParser
*   
* This class contains methods to parse double data.  These are
* primarily called from the NetSolveDouble{Scalar,Vector,Matrix}
* classes.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.StringTokenizer;

public class DoubleParser {

  /**
   *
   *  This method parses a line of text into an array of
   * doubles.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of doubles, cast to Object.
   *
   **/

  public static Object parseLine(String line, StringBuffer err) {
    boolean err_status = false, isTok = false;
    int items = 0;
    String temp = null;
    Object return_object = null;

    StringTokenizer st = new StringTokenizer(line);
    int tokCount = st.countTokens();
    double [] double_arr = new double [tokCount];

    while(tokCount-- > 0) {
      try {
        temp = st.nextToken();
        double_arr[items] = Double.valueOf(temp).doubleValue();
        items++;
      }catch (NumberFormatException e) {
        err_status = true;
        err.append(Labels.invalid_double[Labels.language]+temp+"\n");
      }
    }
    if(err_status)
      return null;
    else
      return double_arr;
  }
}
