/**
* FloatParser
*   
* This class contains methods to parse float data.  These are
* primarily called from the NetSolveFloat{Scalar,Vector,Matrix}
* classes.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.StringTokenizer;

public class FloatParser {

  /**
   *
   *  This method parses a line of text into an array of
   * floats.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of floats, cast to Object.
   *
   **/

  public static Object parseLine(String line, StringBuffer err) {
    boolean err_status = false, isTok = false;
    int items = 0;
    String temp = null;
    Object return_object = null;

    StringTokenizer st = new StringTokenizer(line);
    int tokCount = st.countTokens();
    float [] float_arr = new float [tokCount];

    while(tokCount-- > 0) {
      try {
        temp = st.nextToken();
        float_arr[items] = Float.valueOf(temp).floatValue();
        items++;
      }catch (NumberFormatException e) {
        err_status = true;
        err.append(Labels.invalid_float[Labels.language]+temp+"\n");
      }
    }
    if(err_status)
      return null;
    else
      return float_arr;
  }
}
