/**
* HelpScreen
*   
* This class creates a graphical help screen from which the
* user may select topics and view information about them.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see HelpText
**/

import java.awt.*;
import java.awt.event.*;

public class HelpScreen extends Frame
 implements ActionListener, ItemListener {
  private List topic_list;
  private Button  close_button;
  private TextArea message;
  private String [] help_topics, help_strings;
  public int cur_lang = Labels.language;

  /**
   *
   * This is the constructor for the HelpScreen class.
   * It initializes all the graphical components and
   * displays the window.
   * 
   **/

  public HelpScreen() {
    super("NetSolve Help");

    // set up default size, color, and font for the screen
    Color default_color = new Color(190,190,190);
    Font default_font   = new Font("TimesRoman",Font.BOLD,12);
    int default_height  = 20;
    int default_width   = 70;

    HelpText htext;
    int i;

    htext = new HelpText();
    help_topics = htext.getTopics();     // get the topic list
    help_strings = htext.getHelpText(); // get text for each topic

    this.setLayout(new BorderLayout(5,5));
    this.setBackground(default_color);

    topic_list = new List(15,false);
    topic_list.addItemListener(this);
    for(i=0;i<help_topics.length;i++)
      topic_list.add(help_topics[i]);

    // panel to contain the "close" button
    Panel p1 = new Panel();
    p1.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
    close_button = new Button(Labels.close[cur_lang]);
    close_button.addActionListener(this);
    close_button.setFont(new Font("TimesRoman",Font.BOLD,12));
    p1.add(close_button);

    message = new TextArea(default_height,default_width);
    message.setFont(default_font);
    message.setEditable(false);

    // panel to contain the help topic list
    Panel p2 = new Panel();
    // p2.setLayout(new GridLayout(1,1,5,5));
    p2.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
    p2.add(topic_list);
    p2.setSize(100,200);
 
    // panel to contain the help text
    Panel p3 = new Panel();
    p3.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
    p3.add(message);

    this.add("West",p2);
    this.add("East",p3);
    this.add("South",p1);

    this.pack();
    this.setVisible(true);
  }

  /**
   *
   * This method is called whenever an action event is detected 
   * and includes appropriate event-handling code.  The action 
   * event detected is clicking the "Close" button.
   *          
   * @param e The detected action event.
   *
   **/

  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
 
    if(source == close_button) {
      // user clicked on "close" - get rid of this screen
      this.setVisible(false);
      this.dispose(); 
    }
  }

  /**
   *
   * This method is called whenever a List item has been
   * selected.
   *          
   * @param e The detected item event.
   *
   **/

  public void itemStateChanged(ItemEvent e) {

    if((e.getStateChange() == ItemEvent.SELECTED) &&
       (e.getSource() == topic_list))
    {
      // user selected something from the topic list -
      // display the appropriate text

      int selected = topic_list.getSelectedIndex();
      message.setText(help_strings[selected]);
    }
  }
}  // end class HelpScreen
