/**
* InitData
*   
* This class contains methods to prepare the data for
* display to the user.  Essentially it takes a vector of
* results and produces a string representation of the
* data.
* 
* Before reading this code, it may be informative to review
* the way in which data is stored when it is retrieved from
* the computational server.
* 
* Scalars:
* 
* Each NETSOLVE_SCALAR output object is stored
* in a Vector element (see java.util.Vector).  Integers, 
* floats, doubles, and characters are wrapped in the appropriate
* object (Integer, Float, Double, and Character respectively)
* and stored in element j of the Vector, where j is the index
* of the output object.   Complex numbers are stored similarly
* into the Vector, but they are not wrapped in an object. 
* A complex scalar consists of two floating point numbers which
* are placed in an array, then stored in the Vector.
* To access a scalar, just retrieve the correct Vector element
* using the elementAt() method and cast the return object to
* the appropriate wrapper object (in the case of int, float,
* double, and character) or to the appropriate array type (in
* the case of complex numbers).
* 
* Vectors and Matrices:
*  
* NetSolve vectors and matrices are stored similarly to 
* complex scalars.   First, they are placed in an array 
* and then this array is stored into element j of the Vector,
* where j is the index of the output object.  To access a
* vector or matrix, just retrieve the correct Vector element
* using the elementAt() method and cast the return object
* to the appropriate array type (int, float, etc) and
* dimension (one-dimensional for a netsolve-vector or 
* two-dimensional for a netsolve-matrix).
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.Vector;
import java.awt.Frame;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;

public class InitData extends NetSolveThread {
  private String [] str_array;
  private Problem prob;
  private NetSolveObject [] output_obj;
  private StringBuffer temp;
  private OutputData parent;

  /**
   *
   * This method returns the string representation of the
   * data.
   * 
   * @return Returns string array - one element for each
   *   output object.
   *
   **/

  public String [] getStrArray() {
    return str_array;
  }

  /**
   *
   * This is the constructor for the InitData class.
   * It simply copies the parameters to class variables and
   * then starts the class as a separate thread.
   * 
   * @param a The problem that was solved.
   * @param b The array containing the results.
   * @param c The OutputData screen.
   *
   **/

  public InitData(Problem a, NetSolveObject [] b, OutputData c) {
    prob = a;
    output_obj = b;
    parent = c;

    addActionListener(parent);

    temp = new StringBuffer(200);

    if(GlobalDefs.DEBUG)
      System.out.println("Going to start InitData thread");

    if(GlobalDefs.DEBUG)
      System.out.println("before start");

    this.start();

    if(GlobalDefs.DEBUG)
      System.out.println("after start");
  }
  
  /**
   *
   * Since the constructor starts this class as a thread,
   * this method is really the main method in the class.
   * It loops through each output object (that is, each
   * element of the Vector) and prints the values to a
   * StringBuffer.  This StringBuffer may then be 
   * retrieved by another class and displayed however
   * necessary.
   * 
   **/

  public void run() {
    boolean stdoutText = output_obj.length > prob.nb_output;
    int j;

    if(GlobalDefs.DEBUG)
      System.out.println("...here I am");

    if(stdoutText) {
      int last_el = output_obj.length - 1;

      str_array = new String[prob.nb_output+1];
      str_array[last_el] = output_obj[last_el].toString();
    }
    else
      str_array = new String[prob.nb_output];

    for(j=0;j<prob.nb_output;j++) {
      if(GlobalDefs.DEBUG)
        System.out.println("before yield");

      this.yield();   // give other threads a chance to run

      if(GlobalDefs.DEBUG)
        System.out.println("after yield");

      str_array[j] = output_obj[j].toString();

    }


    if(GlobalDefs.DEBUG)
      System.out.println("Done");

    if(GlobalDefs.DEBUG)
      System.out.println("InitData done...calling processEvent");

    processEvent(new ActionEvent(this,(int)AWTEvent.ACTION_EVENT_MASK,"hi there"));

    return;
  }
} // end class InitData
