/**
* InputData
*   
* This class controls the graphical user interface for the
* Data Input Screen.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.Frame;
import java.awt.Button;
import java.awt.Choice;
import java.awt.TextField;
import java.awt.TextArea;
import java.awt.Checkbox;
import java.awt.Label;
import java.awt.Panel;
import java.awt.AWTEvent;
import java.awt.List;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.event.*;

import java.util.Vector;

public class InputData extends Frame 
 implements ActionListener, ItemListener, FocusListener {

  private int last_selected = -1;
  private int [] checked;
  private int subID = 1;
  public  int myID = 0;
  
  private boolean url_last_choice[];

  private String agent;
  private String [] filename;
  private String [] data;

  private Button quit_button;
  private Button compute_button;
  private Button browse_button;
  private Button sample_button;
  private Button jobList_button;

  private Choice lang_choice;
  private int [] lang_checked;

  private List input_list;

  private TextField filename_box;
  private TextArea data_box;
  private TextArea status_box;
  private TextArea description;

  private Checkbox file_chkbox;
  private Checkbox data_chkbox;
  private Checkbox url_chkbox;

  private Label file_label;
  private Label input_label;
  private Label desc_label;
  private Label status_label;

  private Panel data_panel;
  private Panel file_input_panel;
  private Panel file_chk_panel;
  private Panel data_chk_panel;

  private Frame parent;
  private Problem prob;

  private int cur_lang = Labels.language;
  private String title = Labels.input_title[cur_lang];

  private static String C_COMMENT = new String(
     "/* Do not change the function name below! To reset\n" +
     "   the prototype select C or FORTRAN from the\n" + 
     "   language selector and then press 'Reset'. */\n\n");

  private static String FORTRAN_COMMENT = new String(
     "C Do not change the function name below! To reset\n" +
     "C the prototype select C or FORTRAN from the\n" + 
     "C language selector and then press 'Reset'.\n\n");

  private boolean SAMPLE_RAND = false;

  private int [] upf_index;

  public ScreenList screen_list;

  private Vector cdVec;
  private Vector subVec;
  private Vector allVec;

  private ThreadList threadlist = null;

  private static final int STATUS_WIDTH        = 20;
  private static final int STATUS_HEIGHT       =  7;
  private static final int FILENAME_BOX_WIDTH  = 40;
  private static final int DATA_BOX_WIDTH      = 35;
  private static final int DATA_BOX_HEIGHT     = 10;
  private static final int INPUT_LIST_SIZE     = 12;
  private static final int DESC_WIDTH          = 35;
  private static final int DESC_HEIGHT         =  9;

  private static final int SMALL     = 10;
  private static final int LARGE     = 14;

  private static final int CHOICE_FONT_SIZE        = SMALL;
  private static final int STATUS_FONT_SIZE        = SMALL;
  private static final int FILENAME_BOX_FONT_SIZE  = SMALL;
  private static final int DATA_BOX_FONT_SIZE      = SMALL;
  private static final int FILE_CHKBOX_FONT_SIZE   = LARGE;
  private static final int DATA_CHKBOX_FONT_SIZE   = LARGE;
  private static final int URL_CHKBOX_FONT_SIZE    = LARGE;
  private static final int BUTTON_FONT_SIZE        = LARGE;
  private static final int LABEL_FONT_SIZE         = LARGE;
  private static final int DESC_FONT_SIZE          = SMALL;

  /**
   *
   * This is the constructor for the InputData class.  The purpose 
   * of this method is to initialize and layout all the graphical 
   * components in the Data Input Screen.  This is all done with 
   * standard AWT components.
   * 
   * @param f The parent frame.
   * @param hostname The host name of the agent.
   * @param p The problem to be solved.
   *
   **/

  public InputData(FirstFrame f, String hostname, Problem p) {
    super(); 

    // Get the request ID for this window and add it to the title bar
    myID = f.getReqID();
    super.setTitle(Labels.input_title[cur_lang] + 
            p.name + "' ("+ myID + ")"); 

    int i, num_matrix = 0, num_vector = 0, num_scalar = 0;
    int upf_count = 0;

    parent = f;
    prob = p;
    agent = hostname;

    this.setLayout(new BorderLayout(10,10)); 

    screen_list = f.screen_list;
    subVec = new Vector(20);
    cdVec = new Vector(20);
    allVec = new Vector(40);

    // Create arrays to keep track of settings for each input object
    checked = new int[p.nb_input];
    lang_checked = new int[p.nb_input];
    filename = new String[p.nb_input];
    data = new String[p.nb_input];
    url_last_choice = new boolean[p.nb_input];
    upf_index = new int[p.nb_input];

    /* 
     * This loop initializes each UPF's data input box (a TextArea) with
     * a function prototype and a comment mentioning not to change the 
     * name of the function.  Non-UPF input objects get a blank input box.
     */

    for(i=0;i<p.nb_input;i++) {
      // checked[i] = GlobalDefs.NOT_CHECKED;
      checked[i] = GlobalDefs.DATA_INPUT;
      lang_checked[i] = GlobalDefs.LANG_C;
      url_last_choice[i] = false;

      if(p.inputs[i] instanceof NetSolveUPF) {
        data[i] = new String(C_COMMENT + "    upf"+upf_count+"( )\n{\n\n\n\n\n}");
        upf_index[i] = upf_count;
        upf_count++;
      }
      else {
        data[i] = "";
        filename[i] = "";
        upf_index[i] = -1;
      }
    }

    // Initialize all the checkbox and text field arrays
    filename_box = new TextField(FILENAME_BOX_WIDTH);
    filename_box.addFocusListener(this);
    filename_box.setFont(new Font("Helvetica",Font.BOLD,FILENAME_BOX_FONT_SIZE));

    data_box = new TextArea(DATA_BOX_HEIGHT,DATA_BOX_WIDTH);
    data_box.addFocusListener(this);
    data_box.setFont(new Font("default",Font.PLAIN,DATA_BOX_FONT_SIZE));

    file_chkbox = new Checkbox(Labels.file_input[cur_lang]);
    file_chkbox.addItemListener(this);
    file_chkbox.setFont(new Font("Helvetica",Font.BOLD,FILE_CHKBOX_FONT_SIZE));

    data_chkbox = new Checkbox(Labels.data_input[cur_lang]);
    data_chkbox.addItemListener(this);
    data_chkbox.setFont(new Font("Helvetica",Font.BOLD,DATA_CHKBOX_FONT_SIZE));

    url_chkbox = new Checkbox(Labels.url_input[cur_lang]);
    url_chkbox.addItemListener(this);
    url_chkbox.setFont(new Font("Helvetica",Font.BOLD,URL_CHKBOX_FONT_SIZE));

    // Create some buttons
    quit_button = new Button(Labels.dismiss[cur_lang]);
    quit_button.addActionListener(this);
    quit_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    jobList_button = new Button("list threads");
    jobList_button.addActionListener(this);
    jobList_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    compute_button = new Button(Labels.compute[cur_lang]);
    compute_button.addActionListener(this);
    compute_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    browse_button = new Button(Labels.browse[cur_lang]);
    browse_button.addActionListener(this);
    browse_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    sample_button = new Button(Labels.sample_data[cur_lang]);
    sample_button.addActionListener(this);
    sample_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    lang_choice = new Choice();
    lang_choice.addItemListener(this);
    lang_choice.add("C");
    lang_choice.add("FORTRAN");
    lang_choice.setFont(new Font("Helvetica",Font.BOLD,CHOICE_FONT_SIZE));

    // Define panel holding input list:
    Panel input_panel = new Panel();
    input_panel.setLayout(new BorderLayout(5,5));

    input_list = new List(INPUT_LIST_SIZE,false);
    input_list.addItemListener(this);
    input_panel.add("South",input_list);

    input_label = new Label(Labels.inputs[cur_lang]);
    input_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    input_panel.add("North",input_label);
 
    // Define panels holding input areas
    Panel file_input_panel = new Panel();
    file_input_panel.setLayout(new BorderLayout(5,5));
   
    file_chk_panel = new Panel();
    file_chk_panel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
    file_chk_panel.add(file_chkbox);
    file_chk_panel.add(url_chkbox);

    file_input_panel.add("North",file_chk_panel);
    file_input_panel.add("West",filename_box);
    file_input_panel.add("East",browse_button);
 
    // Define description panel
    Panel desc_panel = new Panel();
    desc_panel.setLayout(new BorderLayout(5,5));

    description = new TextArea(DESC_HEIGHT,DESC_WIDTH);
    description.setFont(new Font("Helvetica",Font.BOLD,DESC_FONT_SIZE));
    description.setEditable(false);
   
    desc_label = new Label(Labels.desc[cur_lang]);
    desc_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    desc_panel.add("North",desc_label);
    desc_panel.add("South",description);

    // Define panel for data input
    data_panel = new Panel();
    data_panel.setLayout(new BorderLayout(5,5));

    data_chk_panel = new Panel();
    data_chk_panel.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
    data_chk_panel.add(data_chkbox);
    data_chk_panel.add(sample_button);
    data_chk_panel.add(lang_choice);

    data_panel.add("North",data_chk_panel);
    data_panel.add("South",data_box);

    Panel right_panel = new Panel();
    right_panel.setLayout(new BorderLayout(5,5));

    right_panel.add("North",file_input_panel);
    right_panel.add("Center",data_panel);
    right_panel.add("South",desc_panel);
 
    // Define panel for buttons
    Panel button_panel = new Panel();
    button_panel.setLayout(new GridLayout(0,1,5,5));

    button_panel.add(compute_button);
    button_panel.add(quit_button);
    button_panel.add(jobList_button);
    
    Panel left_panel = new Panel();
    left_panel.setLayout(new BorderLayout(5,5));

    // Define panel for status box
    Panel status_panel = new Panel();
    status_panel.setLayout(new BorderLayout(5,5));

    status_label = new Label(Labels.status[cur_lang]);
    status_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    status_box = new TextArea(STATUS_HEIGHT,STATUS_WIDTH);
    status_box.setFont(new Font("Helvetica",Font.PLAIN,STATUS_FONT_SIZE));
    status_box.setText(Labels.ready[cur_lang]);
    status_box.setEditable(false);

    status_panel.add("North",status_label);
    status_panel.add("South",status_box);

    left_panel.add("North",input_panel);
    left_panel.add("Center",status_panel);
    left_panel.add("South",button_panel);

    this.add("West",left_panel);
    this.add("East",right_panel);

    // Initialize the input list
    for(i=0;i<prob.nb_input;i++) {
      if(prob.inputs[i] instanceof NetSolveUPF) 
        input_list.add("User Provided Function");
      else
        input_list.add( 
          NetSolveObject.datatype2str(prob.inputs[i].data_type) 
           + " - " +
          NetSolveObject.objtype2str(prob.inputs[i].object_type));
    }
    
    /* 
     * The user interface begins with the first input object
     * selected and the data input box enabled by default.
     */

    input_list.select(0);
    last_selected = 0;
    if(prob.inputs[0] instanceof NetSolveUPF) {
      lang_choice.setEnabled(true);
      lang_choice.setVisible(true);
      fillSampleData();
    }
    handleInputListSelection();

    description.setText(prob.inputs[0].description);

    // file_chkbox.setEnabled(false);
    data_chkbox.setEnabled(true);
    data_chkbox.setState(true);
    // url_chkbox.setEnabled(false);
    filename_box.setEditable(false);
    data_box.setEditable(true);
    browse_button.setEnabled(false);
    sample_button.setEnabled(true);

    if(in_applet()) {
      file_chkbox.setVisible(false);
      browse_button.setVisible(false);
    }
    
    /*
     * Another hack to make things look normal under Netscape 3.x
     * Use long dummy initializer to ensure that the textfield is
     * displayed full-size.  Reset textfield below so that it will
     * be blank upon startup.
     */
    filename_box.setText("0123456789012345678901234567890123456789");

    lang_choice.setEnabled(false);

    this.pack();
    this.setVisible(true);

    // Reset textfield as described above.
    filename_box.setText("");

    lang_choice.setEnabled(false);
    data_box.requestFocus();

    /*
     * for some reason, Netscape resizes the screen when lang_choice
     * is hidden here, so I'll just disable it and only setVisible(false) if we're
     * not running as an applet.
     */
//    if(!in_applet())
    if( !(prob.inputs[0] instanceof NetSolveUPF) )
      lang_choice.setVisible(false);
  }

  public synchronized Vector getThreadList() {
    return allVec;
  }

  /**
   *
   * If this method returns true, this program is running as 
   * an applet.  If false, this program is running as an application.
   * This method simply accesses the in_applet method of
   * FirstFrame.
   *
   * @return Returns boolean representing whether this program
   *    is running as an applet or as an application.
   *
   * @see FirstFrame
   *
   **/

  public boolean in_applet() {
    return ((FirstFrame)parent).in_applet();
  }

  /**
   *
   * This method sets the text of the 'status box' contained
   * in the Data Input Screen.  This box is meant to inform
   * the user about the status of the problem submission.
   * 
   * @param stat_str The string to show in the status box.
   *
   **/

  public void setStatusText(String stat_str) {
    status_box.setText(stat_str);
  }

  /**
   *
   * This method saves the current settings in the Data
   * Input Window so that when the user switches back and
   * forth between different input objects, the previous
   * settings can be restored.  Some of the settings that
   * are saved include:
   *
   *  - text in the filename/URL selection box
   *  - text in the data input box
   *  - the language of the UPF (C or FORTRAN)
   *  - the source of this input (file, URL, text)
   *
   **/

  private void saveVals() {

    /* 
     * If there was nothing previously selected, we don't
     * need to save anything.
     */

    if(last_selected != -1) {
      filename[last_selected] = filename_box.getText();
      data[last_selected] = data_box.getText();
      lang_checked[last_selected] = lang_choice.getSelectedIndex();
      if(file_chkbox.getState()) 
        checked[last_selected] = GlobalDefs.FILE_INPUT;
      else if(data_chkbox.getState())
        checked[last_selected] = GlobalDefs.DATA_INPUT;
      else if(url_chkbox.getState())
        checked[last_selected] = GlobalDefs.URL_INPUT;
      else
        checked[last_selected] = GlobalDefs.NOT_CHECKED;
    }
  }

  /**
   * 
   * User clicked the file browse button.  Note that the rest of
   * the GUI is frozen while the user is browsing.
   * 
   **/

  private void browseFiles()
  {
          
    FileDialog fd = new FileDialog(this,"Load a file",FileDialog.LOAD);
    fd.setResizable(false);
    fd.setVisible(true);

    /* 
     * First make sure the user didn't hit cancel by verifying that
     * the filename is non-null.
     */

    if(fd.getFile() != null) {
      if(fd.getDirectory() == null) {
        // If there's no directory, just get the filename...

        filename_box.setText(fd.getFile());
      } 
      else {
        // ....otherwise, get the whole path.

        filename_box.setText(fd.getDirectory() + fd.getFile());
      }
    }
  }

  /**
   * 
   * User clicked the 'sample data' button.  Fill the Data
   * Input Box with the appropriate data.
   * 
   **/

  private void fillSampleData()
  {
    int selected = input_list.getSelectedIndex();
    int obj = prob.inputs[selected].object_type;

    if(prob.inputs[selected] instanceof NetSolveUPF) {
      /* 
       * This object is a UPF, actually what we're doing here
       * is resetting the data input box to its initial state,
       * in which it contained a function prototype and comment.
       */
 
      if(lang_choice.getSelectedIndex() == GlobalDefs.LANG_FORTRAN)
        data_box.setText(FORTRAN_COMMENT +  
          "                 UPF"+upf_index[selected]+"( )"+
          "\n\n\n\n       END");
      else
        data_box.setText(C_COMMENT + 
          "    upf"+upf_index[selected]+"( )\n{\n\n\n\n\n}");
    } else {
      /* 
       * Otherwise, this is a non-UPF data type, so just call
       * SampleData to get some numbers and display them.
       */

      data_box.setText(prob.inputs[selected].sampleData(SAMPLE_RAND));
    }
  }

  /**
   * 
   * User clicked the 'compute' button.  Get ready to solve
   * the problem.
   * 
   **/

  private void handleCompute()
  {
    saveVals();     // ensure that the arrays are up-to-date

    CheckData cd = new CheckData(prob,filename,data,
       lang_checked,checked,this);
    cd.setName("Check " + prob.name + "(" + myID + "." + subID + ")");

    synchronized (cdVec) {
      cdVec.addElement(cd);
      allVec.addElement(cd);
    }
    cd.start();
    
    if(threadlist != null)
      threadlist.handleUpdate();
  }

  private void handleDataSubmit(CheckData checked)
  {
    // CheckData has completed - before submitting the data,
    // check the status.

    if(checked.dataOK())
    {
      // All the input objects have been checked & parsed, now submit the job

      if(GlobalDefs.DEBUG)
        System.out.println("ok... data looks good, submitting the job");

      try {
        SubmitJob sub = new SubmitJob(this,agent,prob,checked.obj,
            myID + "." + subID, GlobalDefs.ROW_MAJOR);

        if(GlobalDefs.DEBUG)
          System.out.println("inserting " + sub.getName() + " into joblist");

        sub.setName("Submit " + prob.name + "(" + myID + "." + subID + ")");
        synchronized(subVec) {
          subVec.addElement(sub);
          allVec.addElement(sub);
        }
        sub.start();
    
        if(threadlist != null)
          threadlist.handleUpdate();
      }catch(NetSolveException e) {
        System.err.println("Could not submit job: "+e);
        NetSolveWarning solve_warn = new NetSolveWarning(this,
          "SubmitJob: "+e.getMessage());
      }
    }
    else
    {
      TextScreen el = new TextScreen(Labels.bad_data[cur_lang],
         checked.getErrorReport());
      if(in_applet())
        screen_list.addElement(el);
    }

    subID++;
  }

  private void handleDataOutput(SubmitJob s)
  {
    if(threadlist != null)
      threadlist.handleUpdate();

    setStatusText(Labels.got_answer[cur_lang]);
    OutputData od = new OutputData(this,prob,s.getOutputVec(),s.getidString());
  }

  /**
   * 
   * User clicked the quit button...kill this window.
   * 
   **/

  private void handleQuit()
  {
    this.setVisible(false);
    this.dispose(); 
  }

  /**
   * 
   * Filename/URL Box:  OFF -> ON
   *
   * Enable the textfield.
   * Disable the data box, the browse button, the sample button,
   * and the language choice.
   * 
   **/

  private void url_off_to_on()
  {
    int selected = input_list.getSelectedIndex();

    url_last_choice[selected] = true;

    data_chkbox.setState(false);
    file_chkbox.setState(false);
    filename_box.setEditable(true);
    filename_box.requestFocus();
    data_box.setEditable(false);
    browse_button.setEnabled(false);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
  }

  /**
   *
   * Filename/URL Box:  ON -> OFF
   *
   * Disable the textField and the browse button.
   * 
   **/

  private void url_on_to_off()
  {
    int selected = input_list.getSelectedIndex();

    url_last_choice[selected] = false;

    filename_box.setEditable(false);
    browse_button.setEnabled(false);
    input_list.requestFocus();
  }

  /**
   * 
   * Filename/URL Box:  OFF -> ON
   *
   * Enable the filename textField and the browse button.
   * Disable the sample button, the language choice, and
   * the data input box.
   * 
   **/

  private void file_off_to_on()
  {
    int selected = input_list.getSelectedIndex();

    url_last_choice[selected] = false;

    data_chkbox.setState(false);
    url_chkbox.setState(false);
    filename_box.setEditable(true);
    filename_box.requestFocus();
    data_box.setEditable(false);
    browse_button.setEnabled(true);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
  }

  /**
   * 
   * Filename/URL Box:  ON -> OFF
   *
   * Disable the filename textField and the browse button.
   * 
   **/

  private void file_on_to_off()
  {
    int selected = input_list.getSelectedIndex();

    url_last_choice[selected] = false;

    filename_box.setEditable(false);
    browse_button.setEnabled(false);
    input_list.requestFocus();
  }

  /**
   * 
   * Data Input Box:  OFF -> ON
   *
   * Enable the data input textArea, the sample button,
   * and the language choice.  Disable the filename
   * textField and the browse button.
   * 
   **/

  private void data_off_to_on()
  {
    file_chkbox.setState(false);
    url_chkbox.setState(false);
    data_box.setEditable(true);
    data_box.requestFocus();
    filename_box.setEditable(false);
    browse_button.setEnabled(false);
    sample_button.setEnabled(true);
    lang_choice.setEnabled(true);
  }

  /**
   *
   * Data Input Box:  ON -> OFF
   *
   * Disable the data input textArea, the sample button,
   * and the language choice.
   *
   **/

  private void data_on_to_off()
  {
    data_box.setEditable(false);
    input_list.requestFocus();
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
  }

  /**
   * 
   * This method enables all checkboxes.  This doesn't depress 
   * any of the checkboxes, it just allows them to be depressed.
   * 
   **/

  private void enableAllCheckboxes()
  {
    file_chkbox.setEnabled(true);
    data_chkbox.setEnabled(true);
    url_chkbox.setEnabled(true);
  }

  /**
   * 
   * If the current input object is a User Provided Function,
   * then the language choice should be displayed.  This allows
   * the user to signify the language in which their UPF is
   * written (currently C or FORTRAN).
   * 
   * @param selected The index of the selected problem.
   * 
   **/

  private void showLanguageSelector(int selected)
  {
    lang_choice.select(lang_checked[selected]);

    /*
     * Warning: the following is another hack to make Netscape 3.x
     * act properly.  Otherwise, the language choice vanishes until
     * the user clicks on it.  
     */

    lang_choice.setEnabled(false);
    lang_choice.setVisible(false);
    lang_choice.setEnabled(true);

    lang_choice.setVisible(true);
    sample_button.setLabel("   Reset   ");
  }

  /**
   * 
   * If there were previously stored values for this
   * input object, display them, otherwise display nothing.
   * 
   * @param selected The index of the selected problem.
   *
   **/

  private void restoreText(int selected)
  {
    if(filename[selected] != null)
      filename_box.setText(filename[selected]);
    else 
      filename_box.setText("");

    if(data[selected] != null)
      data_box.setText(data[selected]);
    else
      data_box.setText("");
  }

  /**
   * 
   * Data input was previously selected - enable the 
   * textArea and depress the 'Data' checkbox.  Enable
   * the sample button and disable the browse button.
   * 
   **/

  private void setDataSelected()
  {
    data_chkbox.setState(true);
    file_chkbox.setState(false);
    url_chkbox.setState(false);
    filename_box.setEditable(false);
    data_box.setEditable(true);
    data_box.requestFocus();
    browse_button.setEnabled(false);
    sample_button.setEnabled(true);
    lang_choice.setEnabled(true);
  }

  /**
   * 
   * File input was previously selected - enable the
   * textField and depress the 'File' checkbox.   Enable
   * the browse button and disable the sample button.
   * 
   **/

  private void setFileSelected()
  {
    data_chkbox.setState(false);
    url_chkbox.setState(false);
    file_chkbox.setState(true);
    filename_box.setEditable(true);
    filename_box.requestFocus();
    data_box.setEditable(false);
    browse_button.setEnabled(true);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
  }

  /**
   * 
   * URL input was previously selected - enable the
   * textField and depress the 'URL' checkbox.  Disable the
   * browse and sample button.
   * 
   **/

  private void setUrlSelected()
  {
    data_chkbox.setState(false);
    file_chkbox.setState(false);
    url_chkbox.setState(true);
    filename_box.setEditable(true);
    filename_box.requestFocus();
    data_box.setEditable(false);
    browse_button.setEnabled(false);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
  }

  /**
   * 
   * Nothing was previously checked, so just start 
   * everything in a disabled state.
   * 
   **/

  private void setNothingSelected(NetSolveObject obj)
  {
    data_chkbox.setState(false);
    file_chkbox.setState(false);
    url_chkbox.setState(false);
    filename_box.setEditable(false);
    data_box.setEditable(false);
    browse_button.setEnabled(false);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
    
    if(obj instanceof NetSolveUPF)
      lang_choice.setVisible(true);
  }

  /**
   * 
   * This method is called when the user has selected an item 
   * from the input list. Before displaying the values for the 
   * selected input object, save the previous values.
   * 
   **/

  private void handleInputListSelection()
  {
    int selected = input_list.getSelectedIndex();

    saveVals();  

    enableAllCheckboxes();

    /* 
     * For UPF, show the language choice so the user
     * can specify the language of their UPF.
     *
     * For non-UPF data types, hide the language choice.  it's still not 
     * perfect since Netscape displays the language choice the first time 
     * the user selects an input object, but then it goes away.
     */

    if(prob.inputs[selected] instanceof NetSolveUPF)
      showLanguageSelector(selected);
    else {
      lang_choice.setVisible(false);
      sample_button.setLabel("Sample Data");
    }

    restoreText(selected);
     
    /* 
     * Determine the previously selected source for this
     * input object (text data, file, or URL) and reset the
     * checkboxes and other components accordingly (so that
     * it appears exactly the same as it previously did).
     */

    if(checked[selected] == GlobalDefs.DATA_INPUT)
      setDataSelected();
    else if(checked[selected] == GlobalDefs.FILE_INPUT)
      setFileSelected();
    else if(checked[selected] == GlobalDefs.URL_INPUT)
      setUrlSelected();
    else
      setNothingSelected(prob.inputs[selected]);

    last_selected = selected;

    description.setText(prob.inputs[selected].description);
  }

  /**
   * 
   * This method is called when the User deselects an item 
   * on the input list.  Disable everything except the input 
   * list.
   * 
   **/

  private void handleInputListDeselection()
  {
    file_chkbox.setEnabled(false);
    data_chkbox.setEnabled(false);
    url_chkbox.setEnabled(false);
    filename_box.setEditable(false);
    data_box.setEditable(false);
    browse_button.setEnabled(false);
    sample_button.setEnabled(false);
    lang_choice.setEnabled(false);
    lang_choice.setVisible(false);
  }

  /**
   * 
   * This method is called when The filename textField has 
   * gained focus - enable the textField and depress one of 
   * the two checkboxes (file or URL), depending on which was
   * last checked.  
   *
   * If we're in an applet, then the file checkbox is hidden 
   * so just select the URL checkbox.
   * 
   **/

  private void filenameBoxGetsFocus()
  {
    int idx = input_list.getSelectedIndex();

    if(in_applet() || url_last_choice[idx]) {
      url_chkbox.setState(true);
      url_off_to_on();
    }
    else {
      file_chkbox.setState(true);
      file_off_to_on();
    }
  }

  /**
   * 
   * This method is called when the data textArea has gained 
   * focus - enable the textArea and depress the data checkbox.
   * 
   **/

  private void dataBoxGetsFocus()
  {
    data_chkbox.setState(true);
    data_off_to_on();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of this event.  Then, execute the code 
   * associated with the event.
   *
   * @param event The action event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();
    int elem;

    if (source == quit_button)
      handleQuit();
    else if (source == browse_button)
      browseFiles();
    else if (source == sample_button)
      fillSampleData();
    else if (source == compute_button)
      handleCompute();
    else if (source == jobList_button)
    {
      if( (threadlist != null) && (threadlist.isShowing()) )
        threadlist.toFront();
      else
        threadlist = new ThreadList(this);
    }
    else if ( (elem = cdVec.indexOf(source)) >= 0 )
      handleDataSubmit( (CheckData) cdVec.elementAt(elem));
    else if ( (elem = subVec.indexOf(source)) >= 0 )
      handleDataOutput( (SubmitJob) subVec.elementAt(elem));
    else 
      System.err.println("InputData: I dont know where this event comes from!");
  }

  /**
   *
   * This is the method which is called whenever an item changes state.
   * First, determine the source of this event.  Then, execute the code 
   * associated with the event.
   *
   * @param event The item event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
    Object source = e.getSource();

    if (source == url_chkbox) {          // User checked URL input  
      if(url_chkbox.getState())
        url_off_to_on();
      else
        url_on_to_off();
    }
    else if (source == file_chkbox) {    // User checked file input
      if(file_chkbox.getState())
        file_off_to_on();
      else
        file_on_to_off();
    }
    else if (source == data_chkbox) {    // User checked data input
      if(data_chkbox.getState())
        data_off_to_on();
      else
        data_on_to_off();
    }
    else if (source == input_list) {     // User selected list item
      if(e.getStateChange() == ItemEvent.SELECTED)
        handleInputListSelection();
      else if(e.getStateChange() == ItemEvent.DESELECTED)
        handleInputListDeselection();
    }
  }

  /**
   *
   * This is the method which is called whenever something gains focus.
   * First, determine the source of this event.  Then, execute the code 
   * associated with the event.
   *
   * @param e The focus event that was generated.
   *
   **/

  public void focusGained(FocusEvent e)
  {
    Object source = e.getSource();

    /* 
     * Something got the focus.  This bit of code allows the
     * user to select the source of their input by just 
     * clicking on the textArea or textField, rather than
     * having to click the checkbox and then click in the
     * text component.
     */

    if(input_list.getSelectedIndex() != -1) {
      if (source == filename_box)
        filenameBoxGetsFocus();
      else if (source == data_box)
        dataBoxGetsFocus();
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The focus event that was generated.
   *
   **/

  public void focusLost(FocusEvent e)
  {
  }
} // end class InputData
