/**
* IntegerParser
*   
* This class contains methods to parse Integer data.  These are
* primarily called from the NetSolveInt{Scalar,Vector,Matrix}
* classes.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.StringTokenizer;

public class IntegerParser {

  /**
   *
   *  This method parses a line of text into an array of
   * integers.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of integers, cast to Object.
   *
   **/

  public static Object parseLine(String line, StringBuffer err) {
    boolean err_status = false, isTok = false;
    int items = 0;
    String temp = null;
    Object return_object = null;

    StringTokenizer st = new StringTokenizer(line);
    int tokCount = st.countTokens();
    int [] int_arr = new int [tokCount];

    while(tokCount-- > 0) {
      try {
        temp = st.nextToken();
        int_arr[items] = Integer.valueOf(temp).intValue();
        items++;
      }catch (NumberFormatException e) {
        err_status = true;
        err.append(Labels.invalid_integer[Labels.language]+temp+"\n");
      }
    }
    if(err_status)
      return null;
    else
      return int_arr;
  }
}
