/**
* NetSolveClient
*   
* This is the 'main' class in the Java interface.  It parses
* any command-line parameters, contacts the agent, and starts up
* the main screen.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.applet.*;

public class NetSolveClient extends Applet {
  GetProblems gp;
  public FirstFrame firstframe;
  public static final String DEFAULT_HOST = "kenner.cs.utk.edu";
  public int cur_lang = Labels.language;
  public static boolean inApplet = true;

  /**
   *
   * This is the constructor for the NetSolveClient class.
   * It initializes the Initial Screen and attempts to
   * contact the agent.
   * 
   * @param host The hostname of the agent.
   *
   **/

  public NetSolveClient(String host){
    firstframe = new FirstFrame(this,host);
   // firstframe.pack();
   // firstframe.show();

    if(host != null) {
      try {
        gp = new GetProblems(host);
        firstframe.updateProbList(gp.problems);

        if(gp.problems.length == 0)   // if agent has no problems
          new NetSolveWarning(firstframe, Labels.no_problems[cur_lang]);
      }
      catch(NetSolveException e) {
        NetSolveWarning conn_err = new NetSolveWarning(firstframe,
          Labels.no_connection[cur_lang] + e.getMessage());
      }
    }
  }

  /**
   *
   * This is the first method called when the interface is
   * started from the command line (e.g. java NetSolveClient).
   * First, it attempts to parse any command-line parameters
   * present.  Currently the user may specify the language and
   * the agent's hostname on the command line.  If either of 
   * these are unspecified, default values are used.
   * Finally, it creates a new instance of NetSolveClient
   * causing the constructor to be called.
   * 
   * @param argv The array of command line parameters.
   *
   **/

  public static void main(String argv[]){
    String host = DEFAULT_HOST;
    
    inApplet = false;

    if(argv.length > 2) {
      System.err.println("Usage: NetSolveClient [-language] [hostname]");
      System.exit(1);
    }

    if(argv.length == 2) {
      if(argv[0].equalsIgnoreCase("-english")) {
        Labels.setLang(Labels.ENGLISH);
        host = new String(argv[1]);
      }
      else if(argv[0].equalsIgnoreCase("-italian")) {
        Labels.setLang(Labels.ITALIAN);
        host = new String(argv[1]);
      }
      else if(argv[0].equalsIgnoreCase("-french")) {
        Labels.setLang(Labels.FRENCH);
        host = new String(argv[1]);
      }
      else {
        if(argv[1].equalsIgnoreCase("-english")) {
          Labels.setLang(Labels.ENGLISH);
          host = new String(argv[0]);
        }
        else if(argv[1].equalsIgnoreCase("-italian")) {
          Labels.setLang(Labels.ITALIAN);
          host = new String(argv[0]);
        }
        else if(argv[1].equalsIgnoreCase("-french")) {
          Labels.setLang(Labels.FRENCH);
          host = new String(argv[0]);
        }
        else {
          System.err.println("Unknown language, try -english or -italian");
          System.exit(1);
        }
      }
    }
    else if(argv.length == 1) {
      if(argv[0].equalsIgnoreCase("-english")) {
        Labels.setLang(Labels.ENGLISH);
      }
      else if(argv[0].equalsIgnoreCase("-italian")) {
        Labels.setLang(Labels.ITALIAN);
      }
      else if(argv[0].equalsIgnoreCase("-french")) {
        Labels.setLang(Labels.FRENCH);
      }
      else
        host = new String(argv[0]);
    }

    NetSolveClient f = new NetSolveClient(host);
  } // End main
} // end class NetSolveClient
