import java.io.IOException;
import java.io.BufferedReader;
import java.util.Vector;
import java.lang.StringBuffer;

public abstract class NetSolveMatrix extends NetSolveObject {

  protected int order;

  public abstract String toString();
  
  public abstract void printData();

  public abstract void readData( NetSolveStream s) throws IOException;

  public abstract void writeData( NetSolveStream s) throws IOException;

  public void setDims(int lines, int last, int total)
  {
    m = lines;
    n = last;
  }

  public void readMatrixHeader( NetSolveStream s ) throws IOException
  {
    // First, read the data type and object type.  we dont
    // care about this because we already know the types.

    s.server_in.readInt();   // read data type
    s.server_in.readInt();   // read object type

    order = s.server_in.readInt();

    if(GlobalDefs.DEBUG)
      System.out.println("order = " + order );

    if(order == GlobalDefs.COL_MAJOR) {
      n = s.server_in.readInt();
      m = s.server_in.readInt();
      if(GlobalDefs.DEBUG)
      {
        System.out.println("m = " + n );
        System.out.println("n = " + m );
      }
    } else {
      m = s.server_in.readInt();
      n = s.server_in.readInt();
      if(GlobalDefs.DEBUG)
      {
        System.out.println("m = " + m );
        System.out.println("n = " + n );
      }
    }
  }

  public void writeMatrixHeader( NetSolveStream s ) 
     throws IOException
  {
    s.sendType(data_type, object_type);

    // send orientation (row or column major)
    s.server_out.writeInt(order);

    /*
     * Next, send m and n.  However, if we are sending
     * a column-major array from the API,
     * we need to switch m and n.
     */

    if(order == GlobalDefs.COL_MAJOR) {
      if(GlobalDefs.DEBUG)
        System.out.println("sending m = " + n);
      s.server_out.writeInt(n);             // send n

      if(GlobalDefs.DEBUG)
        System.out.println("sending n = " + m);
      s.server_out.writeInt(m);             // send m
    } else {
      if(GlobalDefs.DEBUG)
        System.out.println("sending m = " + m);
      s.server_out.writeInt(m);             // send m

      if(GlobalDefs.DEBUG)
        System.out.println("sending n = " + n);
      s.server_out.writeInt(n);             // send n
    }
  }

  /**
   *
   * This method sets the current storage scheme to be used
   * in interpreting the user's matrices.
   *
   * @param major An integer representing the storage scheme
   *   being used.  Acceptable values are:
   *     GlobalDefs.ROW_MAJOR - signifies row-major layout
   *     GlobalDefs.COL_MAJOR - signifies column-major layout
   *
   * @see GlobalDefs
   *
   **/

  public void setMajor(int major)
  {
    if((major != GlobalDefs.ROW_MAJOR) && (major != GlobalDefs.COL_MAJOR))
    {
      System.err.println("setMajor: Warning, unknown type ("+major+")..." +
         "use 0 for col major, 1 for row major.");
      order = GlobalDefs.ROW_MAJOR;
    }
    else
      order = major;
  }

  public NetSolveMatrix() {
    super();
    order = GlobalDefs.ROW_MAJOR;
  }

  public void copy(NetSolveMatrix obj) {
    super.copy(obj);
    order = obj.order;
  }
}
