import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Vector;

public class NetSolveUPF extends NetSolveObject {
  private String val;
  private String upfName;
  private int upfLanguage;
  private Thread parentThread;

  public String sampleData(boolean rand) {
    return("junk");
  }

  public String getval() {
    return val;
  }

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_EXTERNAL;
    object_type = NETSOLVE_UPF;
  }

  public String toString() {
    return val;
  }

  public NetSolveUPF() 
  {
    super();
  }

  public NetSolveUPF(InputStream in, Thread p) 
    throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveUPF(String code, int lang, int upf_code) 
    throws NetSolveException
  {
    super();
    initialize(code);
    setLang(lang);
    setUPFnum(upf_code);
  }

  public NetSolveUPF(String code) throws NetSolveException
  {
    super();
    initialize(code);
  }

  public void initialize (String code ) throws NetSolveException {
    if(code == null)
      throw new NetSolveException("Null UPF code");

    val = code;
    setDims(0,0,0);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    StringBuffer tmp = new StringBuffer(300);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveIntMatrix()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());

    for(int i = 0; i < vec.size(); i++)
      tmp.append( (String) (vec.elementAt(i)) );
    val = tmp.toString();
  }

  public Object parseLine(String line, StringBuffer err)
  {
    return(line + "\n");
  }

  public int lineLength(Object item)
  {
    return 1;
  }

  public void setDims(int lines, int last, int total)
  {
    // we dont really care about dims for the UPF

    m = 0;
    n = 0;
  }

  public void setLang(String filename)
  {
    if((filename.charAt(filename.length() - 1) == 'c') ||
       (filename.charAt(filename.length() - 1) == 'C'))
    {
      if(GlobalDefs.DEBUG)
        System.out.println("lang is C");
      setLang(GlobalDefs.LANG_C);
    }
    else {
      if(GlobalDefs.DEBUG)
        System.out.println("lang is FORTRAN");
      setLang(GlobalDefs.LANG_FORTRAN);
    }
  }

  public void setLang(int lang)
  {
    if((lang != GlobalDefs.LANG_FORTRAN) && 
       (lang != GlobalDefs.LANG_C))
    {
       System.err.println("NetSolveUPF.setLang(): Invalid value for lang.");
       return;
    }

    upfLanguage = lang;
  }

  public void setUPFnum(int num) {
    upfName = new String("upf" + (new Integer(num).toString()));
  }

  public void readData(NetSolveStream s) throws IOException {
    System.err.println("I dont know how to read a UPF.");
  }

  public void writeData(NetSolveStream s) throws IOException {
    int i;

    // Now send the UPF
    s.server_out.writeInt(upfLanguage);         // type = c or fortran
    s.writeString(upfName);                     // send function name
    s.server_out.writeInt(val.length());        // send size of code

    if(s.getEncoding() == GlobalDefs.DATA_RAW)
      s.server_out.writeBytes(val);
    else
      for(i=0;i<val.length();i++)
        s.server_out.writeInt( (int) val.charAt(i) );

    if(GlobalDefs.DEBUG)
      System.out.println("UPF sent");
  }

  public void copy (NetSolveUPF obj) {
    super.copy(obj);
    val = obj.val;
    upfName = obj.upfName;
    upfLanguage = obj.upfLanguage;
    parentThread = obj.parentThread;
  }
}
