/**
* NetSolveWarning
*   
* This class provides a dialog box used to inform the
* user about errors.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.Button;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Event;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Label;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.*;

public class NetSolveWarning extends Frame
 implements ActionListener {

  private Button  ok_button;
  private Frame   parent;
  public int cur_lang = Labels.language;

  /**
   *
   * This is the constructor for the NetSolveWarning class.
   * The purpose of this method is to initialize and layout
   * the graphical components in the dialog box.
   * 
   * @param f The parent frame.
   * @param message The warning message to display.
   *
   **/

  public NetSolveWarning(Frame f,String message) {

    super();
    super.setTitle(Labels.warning_title[cur_lang]);
    parent = f;
    parent.setEnabled(false);
    this.setLayout(new BorderLayout(5,5));
    this.setBackground(new Color(50,200,240));

    // define panel to contain the message
    Panel p1 = new Panel();
    p1.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
    Label label = new Label(message);
    label.setFont(new Font("Courier",Font.BOLD,18));
    p1.add(label);

    ok_button = new Button("OK");
    ok_button.addActionListener(this);
    ok_button.setFont(new Font("TimesRoman",Font.BOLD,20));

    // define panel for the OK button
    Panel p2 = new Panel();
    p2.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
    p2.add(ok_button);

    this.add("Center",p1);
    this.add("South",p2);
    this.pack();
    this.setVisible(true);
    
  } // End NetSolveWarning

  /**
   *
   * This method waits for an action event, determines if
   * the target was the "OK" button and if so, closes the
   * dialog box.
   * 
   * @param e The event.
   *
   **/

  public void actionPerformed(ActionEvent e) {
 
   if (e.getSource() == ok_button) {
     // User hit the OK button, get rid of this window
     this.setVisible(false);
     this.dispose(); 
     parent.setEnabled(true);
   }
  } // action

} // End class NetSolveWarning
