/**
* Problem
*
* This class defines the data structure for a problem.
* This class was grabbed from another Java/NetSolve project
* so some of the methods contained here might not be applicable
* to the needs of the NetSolve Java interface.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

public class Problem {

  // NAME AND PATH SECTION
  public String name;
  public String path;

  /**
   * 
   * Set the path string for this problem.
   * 
   * @param string Contains the path.
   *
   * @return Returns true if path is set, false upon error.
   *
   **/

  public boolean setpath(String string){
    // Check for "//"
    int test;

    if (string.length() == 0)
      return false;
    if (string.indexOf("//") != -1)
      return false;
    if (string.indexOf(" ") != -1)
      return false;

    if (string.charAt(0) != '/')  // Prepend with a '/' if necessary
      string = '/'+string;
    if (string.charAt(string.length()-1) != '/') // Add a '/' if necessary
      string = string+'/';

    path = string;
    return true;
  } // End setpath

  public String description;

  // LANGUAGE SECTION
  public static final int FORTRAN_LANG = 0;
  public static final int C_LANG = 1;
  public int language;

  // Complexity section
  public int complexity_a;
  public int complexity_b;
  
  // INPUT SECTION
  public int nb_input;
  public NetSolveObject[] inputs;

  /**
   * 
   * This method sets the number of inputs this problem has
   * and allocates enough input objects. 
   * 
   * @param nb The number of input objects.
   *
   * @see NetSolveObject
   **/

  public void setinput(int nb){

    nb_input = nb;
    inputs = new NetSolveObject[nb]; 
  } // end setinput

  // OUTPUT SECTION
  public int nb_output;
  public NetSolveObject[] outputs;

  /**
   * 
   * This method sets the number of outputs this problem has
   * and allocates enough output objects. 
   * 
   * @param nb The number of output objects.
   *
   * @see NetSolveObject
   **/

  public void setoutput(int nb){

    nb_output = nb;
    outputs = new NetSolveObject[nb];
  } // end setoutput

  // UPF SECTION
//  public boolean upf;
  public int upf;

  // CALLING SEQUENCE

  String mnemonics[][];
  Constant constants[];
  Formula formulae[];

  /**
   *
   * This is the constructor for this class.  Its purpose is
   * merely to initialize some class variables.
   * 
   **/

  public Problem()
  {
    description = "";
    mnemonics = null;
    constants = null;
    formulae  = null;
    upf = 0;
  }

  /**
   * 
   * This method searches through the mnemonics array for the
   * specified string.
   * 
   * @param s The string to look for.
   *
   * @return Returns true if the string was found, false otherwise.
   *
   **/

  public boolean lookFor(String s)
  {
    int i,j;

    for (i=0;i<mnemonics.length;i++)
      for (j=0;j<mnemonics[i].length;j++)
        if (s.compareTo(mnemonics[i][j]) == 0)
          return true;
    return false;      
  } // End lookFor

  public void copy(Problem p) {
    int i,j;

    name = new String(p.name);
    setpath(new String(p.path));
    description = new String(p.description);
    language = p.language;
    complexity_a = p.complexity_a;
    complexity_b = p.complexity_b;

// copy inputs
    setinput(p.nb_input);
    for(i=0;i<nb_input;i++) {
      inputs[i] = NetSolveObject.newObject(p.inputs[i].data_type,
         p.inputs[i].object_type);
      inputs[i].copy(p.inputs[i]);
    }

// copy outputs
    setoutput(p.nb_output);
    for(i=0;i<nb_output;i++) {
      outputs[i] = NetSolveObject.newObject(p.outputs[i].data_type,
         p.outputs[i].object_type);
      outputs[i].copy(p.outputs[i]);
    }

    upf = p.upf;

// copy mnemonics
   if(p.mnemonics != null) {
     mnemonics = new String[p.mnemonics.length][];
     for (i=0;i<p.mnemonics.length;i++) {
       mnemonics[i] = new String [p.mnemonics[i].length];
       for (j=0;j<mnemonics[i].length;j++)
         mnemonics[i][j] = new String(p.mnemonics[i][j]);
     }
   }

// copy Constants
   if(p.constants != null) {
     constants = new Constant[p.constants.length];
     for(i=0;i<p.constants.length;i++) {
       constants[i] = new Constant();      
       constants[i].copy(p.constants[i]);
     }
   }

// copy Formulae      
   if(p.formulae != null) {
     formulae = new Formula[p.formulae.length];
     for(i=0;i<p.formulae.length;i++) {
       formulae[i] = new Formula();
       formulae[i].copy(p.formulae[i]);
     }
   }
  }
} // End class Problem
