/**
* ServerList
* 
* This class controls the graphical user interface for the
* Server Information Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.Frame;
import java.awt.Event;
import java.awt.TextArea;
import java.awt.Button;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.*;

public class ServerList extends Frame
 implements ActionListener {
  private Button  close_button;
  private Button  update_button;
  private TextArea message;
  private String agent;
  public int cur_lang = Labels.language;

  /**
   *
   * This is the constructor for the ServerList class.  The purpose of
   * this method is to initialize and layout all the graphical components
   * in the Server Information Screen.  This is all done with standard 
   * AWT components.
   * 
   * @param gs The servers to be listed.
   * @param agent_name The host name of the agent.
   *
   **/

  public ServerList(GetServers gs, String agent_name) {

    super();
    super.setTitle(Labels.server_title[cur_lang]);
    agent = agent_name;
    this.setLayout(new BorderLayout(5,5));

    // define panel to hold the two buttons
    Panel p1 = new Panel();
    p1.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));

    close_button = new Button(Labels.close[cur_lang]);
    close_button.addActionListener(this);
    close_button.setFont(new Font("TimesRoman",Font.BOLD,14));
    update_button = new Button(Labels.update[cur_lang]);
    update_button.addActionListener(this);
    update_button.setFont(new Font("TimesRoman",Font.BOLD,14));

    p1.add(update_button);
    p1.add(close_button);

    message = new TextArea(20,90);

    // set the font to 'default' to get a fixed-size
    // so that formatting is easier
    message.setFont(new Font("default",Font.PLAIN,13));
    message.setEditable(false);

    this.add("South",p1);
    this.add("North",message);

    message.setText(gs.statusString());

    this.pack();
    this.setVisible(true);
    
  }

  /**
   *
   * This method is called when the user clicks the 'close' 
   * button.  Get rid of this window.
   *
   **/

  private void handleClose()
  {
    this.setVisible(false);
    this.dispose(); 
  }

  /**
   *
   * This method is called when the user clicks the 'update'
   * button.  Try to get new server list from agent.
   *
   **/

  private void handleUpdate()
  {
    try {
      GetServers gs = new GetServers(agent);
      message.setText(gs.statusString());
    }catch(NetSolveException exc) {
      System.err.println( Labels.info_error[cur_lang]
         + exc.getMessage());
    }
  }

  /**
   *
   * This is the method which is called whenever an event happens.
   * First, determine what kind of event this is.  Next, determine the
   * target of this event.  Finally, execute the code associated with
   * the event.
   *
   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
 
    if (source == close_button)
      handleClose();
    else if (source == update_button)
      handleUpdate();
  }
} // end class ServerList
