/**
* TextScreen
*
* This class handles the graphical user interface for a text screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
* 
**/

import java.awt.Frame;
import java.awt.Event;
import java.awt.TextArea;
import java.awt.Button;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.*;

public class TextScreen extends Frame
 implements ActionListener {

  private Button  close_button;
  private TextArea message;
  private Color default_color = new Color(50,200,240);
  private Font default_font = new Font("TimesRoman",Font.BOLD,12);
  private int default_height = 20;
  private int default_width = 70;
  public int cur_lang = Labels.language;

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   *
   **/

  public TextScreen(String title, String text) {
    super(title);
    makeTextScreen(text,default_color,default_font,default_height,default_width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param height The height of the TextArea. 
   * @param width The width of the TextArea.
   *
   **/

  public TextScreen(String title, String text, int height, int width) {
    super(title);
    makeTextScreen(text,default_color,default_font,height,width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param color The background color to use.
   * @param height The height of the TextArea. 
   * @param width The width of the TextArea.
   *
   **/

  public TextScreen(String title, String text, Color color, int height, int width) {
    super(title);
    makeTextScreen(text,color,default_font,height,width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param color The background color to use.
   * @param textFont The font to use.
   * @param height The height of the TextArea. 
   * @param width The width of the TextArea.
   *
   **/

  public TextScreen(String title, String text, Color color, Font textFont, int height, int width) {
    super(title);
    makeTextScreen(text,color,textFont,height,width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param textFont The font to use.
   * @param height The height of the TextArea. 
   * @param width The width of the TextArea.
   *
   **/

  public TextScreen(String title, String text, Font textFont, int height, int width) {
    super(title);
    makeTextScreen(text,default_color,textFont,height,width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param color The background color to use.
   *
   **/

  public TextScreen(String title, String text, Color color) {
    super(title);
    makeTextScreen(text,color,default_font,default_height,default_width);
  }

  /**
   *
   * This is the constructor for the TextScreen class.  Its purpose is 
   * to initialize the screen.
   * 
   * @param title The title to display on the titlebar.
   * @param text The text to display on the text screen.
   * @param textFont The font to use.
   *
   **/

  public TextScreen(String title, String text, Font textFont) {
    super(title);
    makeTextScreen(text,default_color,textFont,default_height,default_width);
  }

  /**
   *
   * The purpose of this method is to initialize and layout the
   * graphical components of the text screen.
   * 
   * @param text The text to display on the text screen.
   * @param color The background color to use.
   * @param textFont The font to use.
   * @param height The height of the TextArea. 
   * @param width The width of the TextArea.
   *
   **/

  private void makeTextScreen(String text, Color color, Font textfont,
   int height, int width) {

    this.setLayout(new BorderLayout(5,5));
    this.setBackground(color);

    // define panel to hold the 'close' button
    Panel p1 = new Panel();
    p1.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
    close_button = new Button(Labels.close[cur_lang]);
    close_button.addActionListener(this);
    close_button.setFont(new Font("TimesRoman",Font.BOLD,12));
    p1.add(close_button);

    message = new TextArea(height,width);
    message.setFont(textfont);
    message.setEditable(false);

    this.add("South",p1);
    this.add("North",message);

    message.setText(text);

    this.pack();
    this.setVisible(true);
    
  }

  /**
   *
   * This method waits for an action event, determines if
   * the target was the "close" button and if so, closes the
   * text screen.
   *
   * @param e The event.
   *
   **/

  public void actionPerformed(ActionEvent e) {
 
    if (e.getSource() == close_button) {
      // User clicked the 'close' button - get rid of this window
      this.setVisible(false);
      this.dispose(); 
    }
  }
} // end class TextScreen
