/*****************************************************************/
/*      submit_job.c                                             */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*    submit_job()                                               */
/*****************************************************************/

#include "matlab_netsolve.h"

/*
 * This function contacts the default communication server and sends the
 * problem specifications.
 *
 * Returns :
 *  -1 if failure
 *  NetSolveNoAgent if no agent
 *  NetSolveBadInput if the input is not well specified
 *  NetSolveBadOutput if the output is not well specified
 *  NetSolveAgentFailure if the local server failed
 *  NetSolveUnknownProblem if the problem doesn't exist
 *
 * return 1 if there is a list of remote servers
 *   ( this list may be empty if some servers are down)
 *
 */

#ifdef __STDC__
int submit_job(char *agent,
               int* expected, char** other_server,
               unsigned int* other_addr, int *nb_other_servers,
               int *data_format, int client,
               int problem_size,int data_size, int result_size,
               problem_desc *pb_desc)
#else
int submit_job(agent,expected,other_server,other_addr,nb_other_servers,
               data_format,client, problem_size,data_size,
               result_size, pb_desc)               
char *agent;
int *expected;
char **other_server;
unsigned int *other_addr;
int *nb_other_servers;
int *data_format;
int client;
int problem_size;
int data_size;
int result_size;
problem_desc *pb_desc;
#endif
{
  int sock;                            /* socket descriptor            */
 
  int message_type;
  int status=-1;
  communicator *comm;

  int i,j; 

  sock = connect_to_agent(agent,0);

  if (sock == -3)
    return NetSolveFailure;

  if (sock ==-2)
    return NetSolveNoAgent;

  if (sock ==-1)
    return NetSolveNoAgent;

  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  message_type = PROBLEM_REQUEST;
  write_int(comm,&message_type);
  write_int(comm,&client);
  write_string(comm,pb_desc->code);
  write_int(comm,&problem_size);
  write_int(comm,&data_size);
  write_int(comm,&result_size);
  write_int(comm,&(pb_desc->nb_obj_input));
  for (i=0;i<pb_desc->nb_obj_input;i++)
  {
    write_int(comm,&(pb_desc->input_obj[i].data_type));
    write_int(comm,&(pb_desc->input_obj[i].object_type));
  }
  write_int(comm,&(pb_desc->nb_obj_output));
  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    write_int(comm,&(pb_desc->output_obj[i].data_type));
    write_int(comm,&(pb_desc->output_obj[i].object_type));
  }
  /* END OF A SEND PHASE */
  end_message(comm);

  comm = init_recv(sock);
  read_int(comm,&status);
  if (status == NO_SUCH_PB)
  {
    end_message(comm);
    end_com(sock);
    return NetSolveUnknownProblem;
  }
  if (status == BAD_INPUT)
  {
    end_message(comm);
    end_com(sock);
    return NetSolveBadInput;
  }
  if (status == BAD_OUTPUT)
  {
    end_message(comm);
    end_com(sock);
    return NetSolveBadOutput;
  }
  if (status == SV_FAILURE)
  {
    end_message(comm);
    end_com(sock);
    return NetSolveAgentFailure;
  }
 
/*
 * From now, we have the servers listed
 */

  read_int(comm,nb_other_servers);
  for (j=0;j<*nb_other_servers;j++)
  {
    read_string(comm,(char *)other_server[j]);
    read_IPaddr(comm,(unsigned int *)&other_addr[j]);
    read_int(comm,&data_format[j]);
    read_int(comm,&(expected[j]));
  }
  end_message(comm);
  end_com(sock);
  return NetSolveSuccess;
}
