/*****************************************************************/
/*      matlab_client.c                                          */
/*      Version 1.0                      Henri Casanova          */
/*****************************************************************/

#include "mex.h"
#include "interface.h"
#include "matlab_netsolve.h"

#define BYTE(x,y) (((unsigned char *)x)[(y)]&0xff)

int is_sigpipe;

/*
 * This function is the asynchronous version
 */
        
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  char ptr[100];
  char function[20];
  int j,i;
  int tmp_nlhs,tmp_nrhs;
  mxArray **tmp_plhs;
  mxArray **tmp_prhs;
  double *debug;
  communicator *comm;

  

  if (nrhs == 0)
  {
    int sock;                            /* socket descriptor            */
    struct sockaddr_in server_add;       /* Server INET address */
    char *server;
    struct hostent *hp;
    int message_type;
    int status=-1;
    int nb_pb;
    int detail = 0;
    char **list;
    int *index_list;
    char *min;

    int i,j;

    mexPrintf("NetSolve - List of problems available -\n");

    if ((server = (char *)getenv("NETSOLVE_AGENT")) == 0)
    {
      mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
      return;
    }
 

    sock = connect_to_agent(server,0);
 

    if (sock == -3)
    {
      perror("matlab_client() socket()");
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      return;
    }
    if (sock < 0)
    {
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      mexPrintf("Impossible to contact agent on : %s\n",server);
      return ;
    }
    /* Always XDR */
 
    comm = init_send(DATA_XDR,sock);
    message_type = POLL_LIST_PB;
    write_int(comm,&message_type);
    write_int(comm,&detail);
    end_message(comm);

 
    /* Receive the data */
    comm = init_recv(sock);
    read_int(comm,&nb_pb);
    list = (char **)mxCalloc(nb_pb,sizeof(char *));
    index_list = (int *)mxCalloc(nb_pb,sizeof(int));
    for (i=0;i<nb_pb;i++)
    {
      char buf1[256];
      char buf2[256];

 
      read_string(comm,buf1);
      read_string(comm,buf2);
      list[i] = (char *)mxCalloc(strlen(buf1)+strlen(buf2),sizeof(char));
      strcpy(list[i],buf2);strcpy(list[i]+strlen(buf2),buf1);
    }
    /* Sort */
    for (j = 0;j<nb_pb;j++)
    {
      int min_index = -1;

      min = NULL;
      for (i=0;i<nb_pb;i++)
      {
        if (index_list[i] == 0)
        {
          if (min == NULL)
            min = list[i];
          if (min_index == -1)
            min_index = i;

          if (strcmp(list[i],min) <0 )
          {
            min = list[i];
            min_index = i;
          }
        }
      }
      index_list[min_index] = 1;
      mexPrintf("%s\n",min);
    }

    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    /* Help */
    mexPrintf("-----------------------\n");
    mexPrintf("[ output args ] = netsolve(problem name, input args)\n");
    mexPrintf("[ a ] = netsolve('+',b,c)   % a = b + c\n");
    mexPrintf("-----------------------\n");
    mexPrintf("Information on a specific problem : netsolve(problem name)\n");
    mexPrintf("Information on the servers : netsolve('?')\n");
    mexPrintf("-----------------------\n");
    return;
  }
  if (!mxIsChar(prhs[0]))
  {
     mexPrintf("Invalid first parameter : should be a string\n");
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
     return;
  }
  mxGetString(prhs[0],ptr,100);

  /*
   * List of servers
   */

  if (ptr[0] == '?') /* List of problem */
  {
  int sock;                            /* socket descriptor            */
  struct sockaddr_in server_add;       /* Server INET address */
  struct hostent *hp;
  int message_type;
  int status=-1;
  int nb_sv;
  int detail = 3;
  char buf[128];
  char *tmp;
  char *server;
  int k;
  int i,j;

  mexPrintf("NetSolve : - list of available servers - \n");

  if ((server = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
    return;
  }

  sock = connect_to_agent(server,0);

  if (sock == -1)
  {
    mexPrintf("Unable to open socket\n");
    return ;
  }
  if (sock == -2)
  {
    mexPrintf(
            "Impossible to contact agent on %s\n",server);
    return;
  }

  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  message_type = POLL_LIST_SV;
  write_int(comm,&message_type);
  write_int(comm,&detail);
  end_message(comm);

  /* Receive the data */
  comm = init_recv(sock);
  read_int(comm,&nb_sv);
  for (k=0;k<nb_sv;k++)
  {
    int dummy;
    char name[256];
    unsigned int IPaddr;
    int host_status,server_status;
    int number_problems;

    read_string(comm,name);
    read_IPaddr(comm,&IPaddr);
    read_int(comm,&dummy);
    read_int(comm,&dummy);
    read_int(comm,&host_status);
    read_int(comm,&server_status);
    read_int(comm,&dummy);
    read_int(comm,&dummy);
    read_int(comm,&dummy);
    read_int(comm,&number_problems);

    mexPrintf("%s (%u.%u.%u.%u)\n",name,BYTE(&IPaddr,0),
                                             BYTE(&IPaddr,1),
                                             BYTE(&IPaddr,2),
                                             BYTE(&IPaddr,3));
    if (number_problems == 0)
      mexPrintf("\tNetSolve Agent\n");
    else
      mexPrintf("\tHandles %d problems\n",number_problems);
    switch(host_status)
    {
      case 1 : /* UP */
        mexPrintf("\tHost : Up ");
        break;
      case 0 : /* DOWN */
        mexPrintf("\tHost : Down");
    }
    switch(server_status)
    {
      case 1 : /* RUNNING */
        mexPrintf("\tServer : Running\n");
        break;
      case 0 : /* STOPPED */
        mexPrintf("\tServer : Stopped\n");
        break;
      case -2 : /* FAILED */
        mexPrintf("\tServer : Failed\n");
        break;
    }

  }

  for (i=0;i<nlhs;i++)
  {
    plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
  }
  return;
  }

 /*
  * Specific information about a problem
  */

  if ((nlhs == 0)&&(nrhs == 1))
  {
    char code[CODE_LENGTH];
    int j;
    problem_desc pb_desc;
    int status;

  
    mxGetString(prhs[0],code,CODE_LENGTH);
    strcpy(pb_desc.code,code);

    status = mxnetsolve_info(code,&pb_desc);

    if (status < 0)
    {
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      return;
    }
    mexPrintf("%s :  %s\n",pb_desc.code,pb_desc.desc);
    mexPrintf("---------\n");
    mexPrintf("- INPUT -\n");
    mexPrintf("---------\n");
    for (i=0;i<pb_desc.nb_obj_input;i++)
    {
      mexPrintf(" #%d : ",i);
      switch(pb_desc.input_obj[i].data_type)
      {
        case NETSOLVE_ASCII :
          mexPrintf("ASCII ");
          break;
        case NETSOLVE_I :
          mexPrintf("Integer ");
          break;
        case NETSOLVE_B :
          mexPrintf("Byte ");
          break;
        case NETSOLVE_CHAR :
          mexPrintf("Character ");
          break;
        case NETSOLVE_S :
          mexPrintf("Single precision real ");
          break;
        case NETSOLVE_D :
          mexPrintf("Double precision real ");
          break;
        case NETSOLVE_C :
          mexPrintf("Single precision complex ");
          break;
        case NETSOLVE_Z :
          mexPrintf("Double precision complex ");
          break;
        case NETSOLVE_EXTERNAL :
          mexPrintf(" ");
          break;
        default :
          mexPrintf("??? ");
          break;
      }
      switch(pb_desc.input_obj[i].object_type)
      {
        case NETSOLVE_FILE :
          mexPrintf("File.\n");
          break;
        case NETSOLVE_MATRIX :
          mexPrintf("Matrix.\n");
          break;
        case NETSOLVE_VECTOR :
          mexPrintf("Vector.\n");
          break;
        case NETSOLVE_SCALAR :
          mexPrintf("Scalar.\n");
          break;
        case NETSOLVE_UPF :
          mexPrintf("User Supplied Function.\n");
          break;
        default :
          mexPrintf("???.\n");
          break;
      }
      mexPrintf(" %s\n",pb_desc.input_obj[i].object_desc);
    }
    mexPrintf("----------\n");
    mexPrintf("- OUTPUT -\n");
    mexPrintf("----------\n");
    for (i=0;i<pb_desc.nb_obj_output;i++)
    {
      mexPrintf(" #%d : ",i);
      switch(pb_desc.output_obj[i].data_type)
      {
        case NETSOLVE_ASCII :
          mexPrintf("ASCII ");
          break;
        case NETSOLVE_I :
          mexPrintf("Integer ");
          break;
        case NETSOLVE_B :
          mexPrintf("Byte ");
          break;
        case NETSOLVE_CHAR :
          mexPrintf("Character ");
          break;
        case NETSOLVE_S :
          mexPrintf("Single precision real ");
          break;
        case NETSOLVE_D :
          mexPrintf("Double precision real ");
          break;
        case NETSOLVE_C :
          mexPrintf("Single precision complex ");
          break;
        case NETSOLVE_Z :
          mexPrintf("Double precision complex ");
          break;
        default :
          mexPrintf("??? ");
          break;
      }
      switch(pb_desc.output_obj[i].object_type)
      {
        case NETSOLVE_FILE :
          mexPrintf("File.\n");
          break;
        case NETSOLVE_MATRIX :
          mexPrintf("Matrix.\n");
          break;
        case NETSOLVE_VECTOR :
          mexPrintf("Vector.\n");
          break;
        case NETSOLVE_SCALAR :
          mexPrintf("Scalar.\n");
          break;
        default :
          mexPrintf("???.\n");
          break;
      }
      mexPrintf(" %s\n",pb_desc.output_obj[i].object_desc);
    }
    if (pb_desc.nb_matlab != 0)
    {
      matlab_merge *tmp;

      mexPrintf("-----------------------------------------------\n");
      tmp = pb_desc.matlab;
      for (i=0;i<pb_desc.nb_matlab;i++)
      {
        mexPrintf("Output %d and %d can be merged ",
                       tmp->output1,tmp->output2);
        mexPrintf("to form a complex object\n");
        tmp = tmp->next;
      }
      mexPrintf("-----------------------------------------------\n");
    }

    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }

  /*
   * Real Netsolve stuff
   */

  mexnetslmajor(COL_MAJOR);
  tmp_nlhs = 1;
  tmp_plhs = (mxArray **)mxCalloc(1,sizeof(mxArray *));
  tmp_plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
  /*
   * Call to matlab_send
   */
  matlab_send(tmp_nlhs,tmp_plhs, nrhs, prhs);
  tmp_nrhs = 1;
  tmp_prhs = (mxArray **)mxCalloc(1,sizeof(mxArray *));
  tmp_prhs[0] = tmp_plhs[0];
  /*
   * Call to matlab_get(WAIT)
   */
  matlab_get_wait(nlhs,plhs, tmp_nrhs, tmp_prhs);
  mxDestroyArray(tmp_plhs[0]);
  mxFree(tmp_plhs); mxFree(tmp_prhs);
}
