/*****************************************************************/
/*      matlab_client_nb.c                                       */
/*      Version 1.0                      Henri Casanova          */
/*****************************************************************/

#include "mex.h"
#include "interface.h"
#include "matlab_netsolve.h"

#define BYTE(x,y) (((unsigned char *)x)[(y)]&0xff)
#define SEND_CMD 0
#define WAIT_CMD 1
#define PROBE_CMD 2
#define STATUS_CMD 3
#define ERROR_CMD -1

int is_sigpipe;

/*
 * This function is the asynchronous version
 */
        
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  char function[20];
  int j,i;
  int cmd;

  
  /*
   * list of problems
   */
  if (nrhs == 0)
  {
    mexPrintf("Arguments required\n");
    mexPrintf("Should be : '[r] = netsolve_nb(operation,problem name, arguments)' where\n");
    mexPrintf("  - operation can be 'send', 'wait', 'probe' or 'status'\n");
    mexPrintf("  - problem name in a string (e.g. 'eig')\n");
    mexPrintf("  - arguments description is available by typing :\n");
    mexPrintf("              netsolve(problem name) (e.g. netsolve('eig'))\n");
    mexPrintf("  - netsolve_nb('status') gives you information on the pending requests\n");
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }

  if (!mxIsChar(prhs[0]))
  {
    mexPrintf("Invalid first argument : should be a string\n");
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }
  mxGetString(prhs[0],function,CODE_LENGTH);

  cmd = parse_cmd(function);

  switch(cmd)
  {
    case STATUS_CMD :
    {
      matlab_status();
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      return;
    }
    case SEND_CMD :
    {
      matlab_send(nlhs,plhs, nrhs-1, prhs+1);
      return;
    } 
    case WAIT_CMD :
    {
      matlab_get_wait(nlhs,plhs, nrhs-1, prhs+1);
      return;
    }
    case PROBE_CMD :
    {
      matlab_get_probe(nlhs,plhs, nrhs-1, prhs+1);
      return;
    }
    case ERROR_CMD :
    {
      mexPrintf(
             "Invalid first argument,should be 'send', 'wait' or 'probe'\n");
      for (i=0;i<nlhs;i++)
      {
        plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
      }
      return;
    }
  }
}

/*
 * parse_cmd() : returns the function code
 */
int parse_cmd(char *string)
{
  char *buf;

  buf = (char *)strdup(string);
  uppercase(buf);
  if ((strcmp(buf,"S") == 0)   ||
      (strcmp(buf,"SE") == 0)  ||
      (strcmp(buf,"SEN") == 0) ||
      (strcmp(buf,"SEND") == 0))
    return SEND_CMD;

  if ((strcmp(buf,"W") == 0)   ||
      (strcmp(buf,"WA") == 0)  ||
      (strcmp(buf,"WAI") == 0) ||
      (strcmp(buf,"WAIT") == 0))
    return WAIT_CMD;

  if ((strcmp(buf,"P") == 0)    ||
      (strcmp(buf,"PR") == 0)   ||
      (strcmp(buf,"PRO") == 0)  ||
      (strcmp(buf,"PROB") == 0) ||
      (strcmp(buf,"PROBE") == 0))
    return PROBE_CMD;

  if ((strcmp(buf,"ST") == 0)   ||
      (strcmp(buf,"STA") == 0)  ||
      (strcmp(buf,"STAT") == 0) ||
      (strcmp(buf,"STATU") == 0) ||
      (strcmp(buf,"STATUS") == 0))
    return STATUS_CMD;

  return ERROR_CMD;

}
