/*****************************************************************/
/*      matlab_get_wait.c                                        */
/*      Version 1.0                      Henri Casanova          */
/*****************************************************************/

#include "mex.h"
#include "interface.h"
#include "matlab_netsolve.h"

#define BYTE(x,y) (((unsigned char *)x)[(y)]&0xff)

int is_sigpipe;

/*
 * matlab_get_wait()
 */
        
void matlab_get_wait(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
  int client_type = MATLAB;
  char code[CODE_LENGTH];
  int info,status;
  problem_desc *pb_desc;

  int i,j;
  int length;
  double *ptr,*pti;
  int request_nb;
  double *Request;

  if (nrhs != 1)
  {
    mexPrintf("Illegal number of input, should be :\n");
    mexPrintf(" netsolve_get(<request number>)\n");
    mexPrintf("Not ready yet\n");
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }
  Request = mxGetPr(prhs[0]);
  request_nb = (int)(*Request);

  if (request_nb == -1)
  {
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }
  if (request[request_nb] == 0)
  {
    mexPrintf("'%d' : No such pending request\n",request_nb);
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }
  pb_desc = Xpb_desc[request_nb];

  if ((nlhs != pb_desc->nb_obj_output)&&
     (nlhs != (pb_desc->nb_obj_output-pb_desc->nb_matlab)))
  {
    mexPrintf("Illegal number of output, should be :\n");
    for (i=0;i<pb_desc->nb_obj_output;i++)
    {
      mexPrintf("\t #%d : ",i);
      switch(pb_desc->output_obj[i].data_type)
      {
        case NETSOLVE_ASCII :
          mexPrintf("ASCII ");
          break;
        case NETSOLVE_I :
          mexPrintf("Integer ");
          break;
        case NETSOLVE_B :
          mexPrintf("Byte ");
          break;
        case NETSOLVE_CHAR :
          mexPrintf("Character ");
          break;
        case NETSOLVE_S :
          mexPrintf("Single precision real ");
          break;
        case NETSOLVE_D :
          mexPrintf("Double precision real ");
          break;
        case NETSOLVE_C :
          mexPrintf("Single precision complex ");
          break;
        case NETSOLVE_Z :
          mexPrintf("Double precision complex ");
          break;
        default :
          mexPrintf("??? ");
          break;
      }
      switch(pb_desc->output_obj[i].object_type)
      {
        case NETSOLVE_FILE :
          mexPrintf("ASCII.\n");
          break;
        case NETSOLVE_MATRIX :
          mexPrintf("Matrix.\n");
          break;
        case NETSOLVE_VECTOR :
          mexPrintf("Vector.\n");
          break;
        case NETSOLVE_SCALAR :
          mexPrintf("Scalar.\n");
          break;
        default :
          mexPrintf("???.\n");
          break;
      }
    }
    if (pb_desc->nb_matlab != 0)
    {
      matlab_merge *tmp;

      mexPrintf("-----------------------------------------------\n");
      tmp = pb_desc->matlab;
      for (i=0;i<pb_desc->nb_matlab;i++)
      {
        mexPrintf("Output %d and %d can be merged ",
                       tmp->output1,tmp->output2);
        mexPrintf("to form a complex object\n");
        tmp = tmp->next;
      }
      mexPrintf("-----------------------------------------------\n");
    }
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }

  status = nb_mxnetsolve_get(request_nb,WAIT,plhs,nlhs);
  if (status < 0)
  {
    for (i=0;i<nlhs;i++)
    {
      plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL);
    }
    return;
  }

}
