/*****************************************************************/
/*      mxgateway.c                                              */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  mxgateway()                                                  */
/*****************************************************************/

#include "matlab_netsolve.h"

int is_sigpipe;

/*
 * mxgateway() is the gateway from C to NetSolve
 */

int mxgateway(int number,problem_desc *pb_desc,
             input_sizes *sizes_input, output_sizes *sizes_output,
             void **data_input, void **data_output)
{
  char *other_servers[LEN_RETURN_LIST];
  int expected[LEN_RETURN_LIST];
  unsigned int other_addr[LEN_RETURN_LIST];
  unsigned int current_addr;
  int current_data_format;
  int nb_other_servers;
  communicator *comm;
  int data_format[LEN_RETURN_LIST];
  int my_data_format;
  int dummy = 14;
  int server_pid;
  int i,j,jj;
  int problem_size;
  int data_size;
  int result_size;
  int message_type;
  int submit;
  int status;
  int nb_failure;
  int connection;
  int sock; 
  struct sockaddr_in server_INET_add;
  struct hostent *hp;
  int request_nb;
  void handler();
  int loop;
  int client_type = C_PROGRAM;
  int upf_type;
  FILE *upf;

  char *agent;

  int upf_index;

  char *my_arch;

  for (i=0;i<LEN_RETURN_LIST;i++)
    other_servers[i] = (char *)mxCalloc(MAX_HOST_NAME,sizeof(char));
 
  /*****************************
  * Handler try                *
  *****************************/

  is_sigpipe = 0;
  signal(SIGPIPE,handler);

  /*****************************
  * Assign the request number *
  *****************************/

  request_nb = -1;
  if (number == -1) 
  {
    for (i=0;i<MAX_REQUEST;i++)
    {
      if (request[i] == 0)
      {
        request[i] = 1;
        request_nb = i;
        break;
      }
    }
  }
  else
    request_nb = number;

  if (request_nb == -1)
  {
    mexPrintf("Too many pending requests\n");
    request[request_nb] = 0;
    return -1;
  }

  /****************************
  * Save the input parameters *
  ****************************/

  Xpb_desc[request_nb] = pb_desc;
  Xsizes_input[request_nb] = sizes_input;
  Xdata_input[request_nb] = data_input;
  Xserver_name[request_nb] = NULL;

  /****************************
  * Getting the Architecture *
  ****************************/

  if ((my_arch = (char *)getenv("NETSOLVE_ARCH")) == 0)
  {
    mexPrintf("The NETSOLVE_ARCH environment variable should be set\n");
    request[request_nb] = 0;
    return -1;
  }
  my_data_format = arch_to_format(my_arch);


  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
    request[request_nb] = 0;
    return -1;
  }

  /********************************************
   * Asses the problem Size and the data Size *
   ********************************************/

  data_size = 0; problem_size = 0; result_size = 0;

  for (j=0;j<pb_desc->nb_obj_input;j++)
  {
    int m,n;

    m = sizes_input[j].m;
    n = sizes_input[j].n;
    if (m > problem_size)
      problem_size = m;
    if (n > problem_size)
      problem_size = n;
    switch (pb_desc->input_obj[j].data_type)
    {
      case NETSOLVE_I :
      {
        data_size += INTL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_B :
      {
        data_size += sizeof(char)*m*n+2*sizeof(char);
        break;
      }
      case NETSOLVE_CHAR :
      {
        data_size += sizeof(char)*m*n+2*sizeof(char);
        break;
      }
      case NETSOLVE_S :
      {
        data_size += FLOATL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_D :
      {
        data_size += DOUBL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_C :
      {
        data_size += SCOMPL*m*n+2*INTL;
        break;
      }
      case NETSOLVE_Z :
      {
        data_size += DCOMPL*m*n+2*INTL;
        break;
      }
    }
  }

/*****************************************/
/** Real Netsolve Stuff                  */
/*****************************************/

/**
 ** Main Loop !!
 **/

while(1)
{

  submit = submit_job(
          agent,expected,(char **)other_servers,other_addr,&nb_other_servers,
          data_format,C_PROGRAM,problem_size,
          data_size,result_size,pb_desc);

  if (submit<0)
  {
    switch(submit)
    {
      case -1 : mexPrintf("NetSolve failure\n");break;
      case NetSolveNoAgent : mexPrintf("Cannot contact Agent\n");break;
      case NetSolveBadInput : mexPrintf("Invalid Input\n");break;
      case NetSolveBadOutput : mexPrintf("Invalid Output\n");break;
      case NetSolveAgentFailure : mexPrintf("NetSolve failure\n");break;
      case NetSolveUnknownProblem : mexPrintf("Unknown Problem \n");break;
    }
    request[request_nb] = 0;
    return -1;
  }

#ifdef DEBUG
  mexPrintf("I received a list of %d servers\n",nb_other_servers);
  for (j=0;j<nb_other_servers;j++)
    mexPrintf("\t ---> %s\n",(char *)other_servers[j]);
#endif
 if (nb_other_servers == 0)
 {
   mexPrintf("No computational resource available\n");
    request[request_nb] = 0;
   return -1;
 }

/* Here, we have the list of servers to contact, sorted !!! */

  message_type = PROBLEM_SOLVE;

  nb_failure = 0;

  for (j=0;j<nb_other_servers;j++)
  {

/*
 * I connect to the server
 */
   
    current_addr = other_addr[j];
    current_data_format = data_format[j];
#ifdef VIEW
    mexPrintf("Connecting to server '%s'\n",other_servers[j]);
#endif
    sock = connect_to_server(other_servers[j],current_addr);

    if (sock == -3)
    {
      perror("main() socket()");
    request[request_nb] = 0;
      return -1;
    }
    if (sock == -2)
    {
      nb_failure++;
      if (nb_failure != nb_other_servers)
        continue;
      else
        break;
    }
    if (sock == -1)
    {
      nb_failure++;
      mxclient_f_report(0,agent,current_addr);
      if (nb_failure != nb_other_servers)
        continue;
      else
        break;
    }
    if (nb_failure == nb_other_servers)
      break;
 /*
  * I actually send the problem
  */
#ifdef DEBUG
  mexPrintf("I eventually found a server\n");
#endif

  /* BEGIN OF A SEND PHASE */
  /* XDR or RAW */
  if (current_data_format == my_data_format)
    comm = init_send(DATA_RAW,sock);
  else
    comm = init_send(DATA_XDR,sock);
  write_int(comm,&message_type);
  write_string(comm,pb_desc->code);
  write_string(comm,agent);
  write_int(comm,&client_type);
  write_int(comm,&(pb_desc->nb_obj_input));
  for (loop = 0;loop < pb_desc->nb_obj_input; loop++)
  {
    write_int(comm,&(pb_desc->input_obj[loop].data_type));
    write_int(comm,&(pb_desc->input_obj[loop].object_type));
  } 
  write_int(comm,&(pb_desc->nb_obj_output));
  for (loop = 0;loop < pb_desc->nb_obj_output; loop++)
  {
    write_int(comm,&(pb_desc->output_obj[loop].data_type));
    write_int(comm,&(pb_desc->output_obj[loop].object_type));
  } 
  end_message(comm);
  /* END OF A SEND PHASE */
  /*
   * Here I want the ack from the remote server
   */
  /* BEGIN OF A RECEIVE PHASE */
    comm = init_recv(sock);
    read_int(comm,&status);
    end_message(comm);
 /* END OF A RECV PHASE */
    if (status == PB_REFUSED)
    {  /* The server cannot solve the problem */
#ifdef DEBUG
      mexPrintf("\tThe remote server cannot deal with this problem\n");
#endif
      end_com(sock);
      nb_failure++;
      continue;
    }
    if (status == SV_FAILURE)
    {  /* A server has failed */
#ifdef VIEW
      mexPrintf("\t The remote server has failed\n");
#endif
      end_com(sock);
      nb_failure++;
      continue;
   }
   if (status== NOT_AUTHORIZED)
   {
#ifdef VIEW
      mexPrintf("\t Non authorized to access this server\n");
#endif
     end_com(sock);
     nb_failure++;
     continue;
   }
   if (status == TOKENS_OVERFLOW)
   {
#ifdef VIEW
     mexPrintf("\t Computation quotas exceeded\n");
#endif
     end_com(sock);
     nb_failure++;
     continue;
   }

   /* Acknoledge is PB_ACCEPTED */

  /* BEGIN OF A SEND PHASE */
  /* XDR or RAW */
#ifdef VIEW
  mexPrintf("Problem accepted, sending the data...\n");
#endif
  /*
   * Take care of upf's
   */
  upf_index = 0;
  for (i=0;i<pb_desc->nb_obj_input;i++)
  {
    if (pb_desc->input_obj[i].object_type == NETSOLVE_UPF)
    {
      int size;
      struct stat st;
      char *buf;
      char fullname[256];
      char fcnname[256];
      char subroutinename[64];
      char *ptr;

      /* Get the file name */

      strcpy(fcnname,(char *)data_input[i]);
      sprintf(fullname,"%s.f",fcnname);
#ifdef DEBUG
      mexPrintf("Trying to open '%s'\n",fullname);
#endif
      upf = fopen(fullname,"r");
      if (upf == NULL)
      {
        sprintf(fullname,"%s.c",fcnname);
        upf = fopen(fullname,"r");
        if (upf == NULL)
        {
          mexPrintf("File '%s.c' or '%s.f' not found.\n",
                         fcnname,fcnname);
          return NetSolveFileNotFound;
        }
        else
          upf_type = LANG_C;
      }
      else
        upf_type = LANG_FORTRAN;

      /* We read the file */

      stat(fcnname,&st);
      buf = (char *)mxCalloc(1+st.st_size,sizeof(char));
      if (fread(buf,1,st.st_size,upf) != st.st_size)
      {
        mexPrintf("Read error on file %s\n",fullname);
        fclose(upf);
        request[request_nb] = 0;
        return -1; 
      }
      buf[st.st_size] = '\0';
      fclose(upf);
      size = (int)(st.st_size);

      /* We send it on the socket */
      if (upf_index == 0)
      {
        if (current_data_format == my_data_format)
          comm = init_send(DATA_RAW,sock);
        else
          comm = init_send(DATA_XDR,sock);
      }
      write_int(comm,&upf_type);

      /*  get only the last part of the path to the upf */
      strcpy(subroutinename,fcnname);
      ptr = &(subroutinename[strlen(subroutinename)-1]);
      while((ptr!=subroutinename)&&(*ptr!='/'))
        ptr--;
      if (*ptr=='/')
        ptr++;
      write_string(comm,ptr);

      write_int(comm,&size);
      write_char_array(comm,buf,size);
      mxFree(buf);
      upf_index++;
    }
  }

  if (upf_index != 0)
  {
    int message;

    end_message(comm);
    comm = init_recv(sock);
    read_int(comm,&message);
    end_message(comm);
    if (message == UPF_FAILED)
    {
      end_message(comm);
      end_com(sock);
      mexPrintf("Dynamic Linking of UPF failed on computational server\n");
      return NetSolveUPFFailed;
    }
    if (message == UPF_UNSAFE)
    {
      char tmp[256];
      int unsafe_upf_index;

      read_int(comm,&unsafe_upf_index);
      read_string(comm,tmp);
      end_message(comm);
      end_com(sock);
      mexPrintf("The computational server deemed your %dth UPF unsafe !!",
                     unsafe_upf_index);
      mexPrintf("Invalid calls : %s\n",tmp);
      return NetSolveUPFUnsafe;
    }
  }


  if (current_data_format == my_data_format)
    comm = init_send(DATA_RAW,sock);
  else
    comm = init_send(DATA_XDR,sock);

  for (jj = 0; jj< pb_desc->nb_obj_input;jj++)
  {
    int m,n;


    if (pb_desc->input_obj[jj].object_type == NETSOLVE_FILE)
    {
      write_int(comm,&(pb_desc->input_obj[jj].object_type));
      write_int(comm,&(pb_desc->input_obj[jj].data_type));
      m = sizes_input[jj].m;
      write_int(comm,&m);
      n = 1;
      sizes_input[jj].n = 1;
      sizes_input[jj].l = m;
    }

    if (pb_desc->input_obj[jj].object_type == NETSOLVE_MATRIX)
    {
      write_int(comm,&(pb_desc->input_obj[jj].object_type));
      write_int(comm,&(pb_desc->input_obj[jj].data_type));
      write_int(comm,&Xmajor);

      m = sizes_input[jj].m;
      n = sizes_input[jj].n;
      write_int(comm,&m);
      write_int(comm,&n);
    }
    if (pb_desc->input_obj[jj].object_type == NETSOLVE_VECTOR)
    {
      write_int(comm,&(pb_desc->input_obj[jj].object_type));
      write_int(comm,&(pb_desc->input_obj[jj].data_type));
      m = sizes_input[jj].m;
      sizes_input[jj].n = 1;
      sizes_input[jj].l = m;
      n = 1;
      write_int(comm,&m);
    } 
    if (pb_desc->input_obj[jj].object_type == NETSOLVE_SCALAR)
    {
      write_int(comm,&(pb_desc->input_obj[jj].object_type));
      write_int(comm,&(pb_desc->input_obj[jj].data_type));
      sizes_input[jj].n = 1;
      sizes_input[jj].m = 1;
      sizes_input[jj].l = 1;
      m = 1;
      n = 1;
    }

    if (sizes_input[jj].m == sizes_input[jj].l)
    { /* No Leading Dimension problem */
      switch(pb_desc->input_obj[jj].data_type)
      {
        case NETSOLVE_ASCII :
        {
          int fd;

          fd = open(data_input[jj],O_RDONLY);
          if (fd < 0)
          {
            fprintf(stderr,"Impossible to open file '%s'\n",data_input[jj]);
            fprintf(stderr,"Aborting...\n");
            break;
          }
          write_char_array_file(comm,fd,sizes_input[jj].m);
          close(fd);
          break;
        }
        case NETSOLVE_I :
        {
          write_int_array(comm,(int *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_B :
        {
          write_byte_array(comm,(char *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_CHAR :
        {
          write_char_array(comm,(char *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_S :
        {
          write_single_array(comm,(float *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_D :
        {
          write_double_array(comm,(double *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_C :
        {
          write_scomplex_array(comm,(scomplex *)(data_input[jj]),m*n);
          break;
        }
        case NETSOLVE_Z :
        {
          write_dcomplex_array(comm,(dcomplex *)(data_input[jj]),m*n);
          break;
        }
      } 
    }
    else /* Take of the leading dimensions ..... */
    {
      int k;

      switch(pb_desc->input_obj[jj].data_type)
      {
        case NETSOLVE_I :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_int_array(comm,(int *)((int *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_CHAR :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_char_array(comm,(char *)((char *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_B :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_byte_array(comm,(char *)((char *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_S :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_single_array(comm,(float *)((float *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_D :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_double_array(comm,(double *)((double *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_C :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_scomplex_array(comm,(scomplex *)((scomplex *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
        case NETSOLVE_Z :
        {
          for (k=0;k<sizes_input[jj].n;k++)
          {
            write_dcomplex_array(comm,(dcomplex *)((dcomplex *)(data_input[jj])+
                          k*(sizes_input[jj].l)),sizes_input[jj].m);
          }
          break;
        }
      }
    }
  }

  /* Waiting for the 'pid' of the computing process */
  comm = init_recv(sock);
  if (read_int(comm,&server_pid) == -1)
  {
    end_message(comm);
    end_com(sock);
    mxclient_f_report(-1,agent,current_addr);
    continue;
  }
  end_message(comm);
  if (current_data_format == my_data_format)
    comm = init_send(DATA_RAW,sock);
  else
    comm = init_send(DATA_XDR,sock);
  write_int(comm,&dummy);
  end_message(comm);

  /* END OF A SEND PHASE */
  /* BEGINNING OF A RECEIVE PHASE */
  if (is_sigpipe == 0)
    break;
  else
  {
    mxclient_f_report(-1,agent,current_addr);
    is_sigpipe = 0;
#ifdef DEBUG
    mexPrintf("Trying another server\n");
#endif
    end_com(sock);
    continue;
  }
  
/**************/
} /* end FOR  */
/**************/

  if (nb_failure == nb_other_servers) 
  {
    /* Everything has failed or has not been up-to-date !!! */
    if (nb_other_servers < LEN_RETURN_LIST)
    {
      end_message(comm);
      end_com(sock);
      mexPrintf("No computational resource available\n");
    request[request_nb] = 0;
      return -1;
    }
    else /* Submit the problem again */
    {
      end_message(comm);
      end_com(sock);
      continue;
    }
  }
  else
  {
    break;
  }

   /*************/
}  /* End while */
   /*************/

  Xsizes_output[request_nb] = sizes_output;
  Xdata_output[request_nb] = data_output;
  Xdesc[request_nb] = sock;
 
  /* Stat stuff */

  if (Xserver_name[request_nb] != NULL)
  {
    mxFree(Xserver_name[request_nb]);
  }
  Xserver_name[request_nb] = (char *)strdup(other_servers[j]);
  XIPaddr[request_nb] = other_addr[j];
  time((time_t *)(&(Xtime[request_nb])));
  Xexpected[request_nb] = expected[j];

#ifdef VIEW
  mexPrintf("Data sent, computation being performed remotely...\n");
#endif
  return request_nb;
}

void
handler(int s)
{

#ifdef DEBUG
  mexPrintf("Computational server failed\n");
#endif
  is_sigpipe = 1;
#ifdef SYSVSIGNAL
  signal(SIGPIPE,handler);
#endif
}
