/*****************************************************************/
/*      mxnetsolve_info.c                                        */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*    mxnetsolve_info()                                          */
/*****************************************************************/

#include "matlab_netsolve.h"

/*
 * mxnetsolve_info()
 *
 *   Get information on a problem
 */

int mxnetsolve_info(char *code,problem_desc *pb_desc)
{
  int sock;                            /* socket descriptor   */
  struct sockaddr_in server_add;       /* Server INET address */
 
  communicator *comm;
  struct hostent *hp;
  int message_type;
  int status;
  int i;
  char *server;
  constant *current_constant;
  formula *current_form;

  if ((server = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
    return -1;
  }

  sock = connect_to_agent(server,0);


  if (sock == -1)
  {
    perror("mxnetsolve_info() socket()");
    return -1;
  }
  if (sock < 0)
  {
    mexPrintf("Impossible to contact server on '%s'\n",server);
    return -1;
  }

  
  /* Always XDR */
  comm = init_send(DATA_XDR,sock);
  message_type = INFO_REQUEST;
  write_int(comm,&message_type);
  write_string(comm,code);
  end_message(comm);

  comm = init_recv(sock);
  read_int(comm,&status);
  if (status == NO_SUCH_PB)
  {
    end_message(comm);
    end_com(sock);
    mexPrintf("Unknown problem\n");
    return -1;
  }
 
  read_string(comm,pb_desc->path);
  read_string(comm,pb_desc->desc);
  read_int(comm,&(pb_desc->nb_obj_input));
  pb_desc->input_obj = 
          (object *)mxCalloc(sizeof(object),pb_desc->nb_obj_input);
  mexMakeMemoryPersistent(pb_desc->input_obj);
  for (i=0;i<pb_desc->nb_obj_input;i++)
  {
    read_int(comm,&(pb_desc->input_obj[i].data_type));
    read_int(comm,&(pb_desc->input_obj[i].object_type));
    read_string(comm,pb_desc->input_obj[i].object_desc);
  }
  read_int(comm,&(pb_desc->nb_obj_output));
  pb_desc->output_obj =
          (object *)mxCalloc(sizeof(object),pb_desc->nb_obj_output);
  mexMakeMemoryPersistent(pb_desc->output_obj);
  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    read_int(comm,&(pb_desc->output_obj[i].data_type));
    read_int(comm,&(pb_desc->output_obj[i].object_type));
    read_string(comm,pb_desc->output_obj[i].object_desc);
  }
  /* Read the format */
  /* We only malloc space for one format */
  pb_desc->nb_formats = 1;
  pb_desc->formats = (format_spec *)mxCalloc(sizeof(format_spec),1);
  mexMakeMemoryPersistent(pb_desc->formats);
  read_int(comm,&(pb_desc->formats[0].nb_args));
  pb_desc->formats[0].args_desc = 
        (char **)mxCalloc(sizeof(char **),pb_desc->formats[0].nb_args);
  mexMakeMemoryPersistent(pb_desc->formats[0].args_desc);
  for (i=0;i<pb_desc->formats[0].nb_args;i++)
  {
    char buf[256];

    read_string(comm,buf);
    pb_desc->formats[0].args_desc[i] = (char *)mxCalloc(sizeof(char),strlen(buf));
    mexMakeMemoryPersistent(pb_desc->formats[0].args_desc[i]);
    strcpy(pb_desc->formats[0].args_desc[i],buf);
  }
  read_int(comm,&(pb_desc->formats[0].nb_constants));
  /* Swapped while received, but don't care */
  for (i=0;i<pb_desc->formats[0].nb_constants;i++)
  {
    char buf[256];

    current_constant = (constant *)mxCalloc(sizeof(constant),1);
    mexMakeMemoryPersistent(current_constant);
    current_constant->next = pb_desc->formats[0].consts;
    read_string(comm,buf);
    current_constant->mnemonic = (char *)mxCalloc(sizeof(char),strlen(buf));
    mexMakeMemoryPersistent(current_constant->mnemonic);
    strcpy(current_constant->mnemonic,buf);
    read_int(comm,&(current_constant->value));
    pb_desc->formats[0].consts = current_constant;
  }

  read_int(comm,&(pb_desc->formats[0].nb_formulae));
  /* Swapped while received, but don't care */
  for (i=0;i<pb_desc->formats[0].nb_formulae;i++)
  {
    char buf1[256];
    char buf2[256];

    current_form = (formula *)mxCalloc(sizeof(formula),1);
    mexMakeMemoryPersistent(current_form);
    current_form->next = pb_desc->formats[0].forms;
    read_string(comm,buf1);
    current_form->mnemonic = (char *)mxCalloc(sizeof(char),strlen(buf1));
    mexMakeMemoryPersistent(current_form->mnemonic);
    strcpy(current_form->mnemonic,buf1);
    read_string(comm,buf2);
    current_form->expression = (char *)mxCalloc(sizeof(char),strlen(buf2));
    mexMakeMemoryPersistent(current_form->expression);
    strcpy(current_form->expression,buf2);
    pb_desc->formats[0].forms = current_form;
  }
  read_int(comm,&(pb_desc->nb_matlab));
  for (i=0;i<pb_desc->nb_matlab;i++)
  {
    matlab_merge *tmp;
    tmp = (matlab_merge *)mxCalloc(1,sizeof(matlab_merge));
    mexMakeMemoryPersistent(tmp);
    read_int(comm,&(tmp->output1));
    read_int(comm,&(tmp->output2));
    tmp->next = pb_desc->matlab;
    pb_desc->matlab = tmp;
  }


  end_message(comm);
  end_com(sock);
  return 1;
}
