/*****************************************************************/
/*      nb_mxnetsolve_get.c                                      */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  nb_mxnetsolve_get()                                          */
/*  restart()                                                    */
/*****************************************************************/

#include "mex.h"
#include "matlab_netsolve.h"

/*
 * mxnetsolve_get() : Check the status of a pending request
 *     mode = PROBE  - non-blocking
 *     more = WAIT   - blocking
 */

int nb_mxnetsolve_get(int r,int mode,mxArray *plhs[],int nlhs)
{
  int k;
  communicator *comm;
  int status,i;
  char buf[1];
  int ok;
  int cc;
  int obj,dat;
  double *ptr,*pti;
  int obj_index = 0;
  int  stdoutlength;
  char *stdoutbuf;
  char *agent;
  communicator *hbm_comm;
  int hbm_sock;
  int hbm_activated = 1;
  int alive=1; /* Set to one in case the HBM is disabled */


  if ((agent = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
    return -1;
  }

  
#ifdef DEBUG
  mexPrintf("Probing request number %d in ",r);
  if (mode == WAIT)
    mexPrintf("blocking mode : ");
  if (mode == PROBE)
    mexPrintf("non-blocking mode : ");
#endif
  if (request[r] == 0)
  {
    mexPrintf("Unknown Request\n");
    return -1;
  }

  /* connect to the HBM */

  hbm_sock = connect_to_socket(agent, 0, HBM_INTERFACE_PORT);
  if (hbm_sock<0)
  {
#ifdef DEBUG
    mexPrintf("Impossible to contact the HBM interface.\n");
#endif
    hbm_activated = 0;
  }
  else
  {
    hbm_comm = init_send(DATA_XDR,hbm_sock);
    if (write_int(hbm_comm,&(Xserver_pid[r])) == -1)
    {
#ifdef DEBUG
      mexPrintf("Impossible to contact the HBM interface.\n");
#endif
      hbm_activated = 0;
    }
    end_message(hbm_comm);
  }

  if (mode == PROBE)
  {
    int off = 0;
    int on = 1;
    int try;
    int server_status;

    try = ioctl(Xdesc[r],FIONBIO,&on);
    ok = recv(Xdesc[r],buf,1,MSG_PEEK);
    ioctl(Xdesc[r],FIONBIO,&off);
    if (ok == -1) /* Not ready */
    {
            /* Try the HBM to see if still alive */
      if (hbm_activated)
      {
        if ((hbm_comm = init_recv(hbm_sock)) == NULL)
        {
#ifdef DEBUG
          mexPrintf("Impossible to contact the HBM interface.\n");
#endif
          hbm_activated = 0;
          end_com(hbm_sock);
        }
      }
      if (hbm_activated)
      {
        if (read_int(hbm_comm,&server_status) == -1)
        {
#ifdef DEBUG
          mexPrintf("Impossible to contact the HBM interface.\n");
#endif
          hbm_activated = 0;
          end_message(hbm_comm);
          end_com(hbm_sock);
        }
      }
     if (hbm_activated)
      {
        end_message(hbm_comm);
        switch(server_status)
        {
          case 1:
          case 2:
          case 3:
          case 4:
            alive = 1;
            break;
          default:
            alive = 0;
            break;
        }
      }
      if (alive == 1)
      {
        end_com(hbm_sock);
        return NetSolveNotReady;
      }
      else
        ok = 0;
    }
    if (ok == 0) /* Connection doesn't exist anymore */
    {
      int status;
#ifdef DEBUG
      mexPrintf("The computational server has failed !!\n");
#endif
      end_com(hbm_sock);
      end_com(Xdesc[r]);
      status = restart(mode,r);
      if (status != r)
        return -1;
      return NetSolveNotReady;
    }
  }

  while(hbm_activated)
  {
    int try;
    int on = 1;
    int off =0;
    char buf[1];
    int server_status;

    ioctl(Xdesc[r],FIONBIO,&on);
    try = recv(Xdesc[r],buf,1,MSG_PEEK);
    ioctl(Xdesc[r],FIONBIO,&off);

    if (try == 0) /* No connection anymore !! */
    {
      end_com(hbm_sock);
      end_com(Xdesc[r]);
      /**** RESTART ****/
      status = restart(mode,r);
      if (status != r) /* Error message already set */
        return status;
      status = nb_mxnetsolve_get(r,WAIT,plhs,nlhs);
      if (status <0)
        return status;
    }
    if (try != -1)
      break;

    /* Not ready yet */
    if (hbm_activated)
    { /* Check the HBM */
      if ((hbm_comm = init_recv(hbm_sock)) == NULL)
      {
#ifdef DEBUG
        mexPrintf("Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      if (read_int(hbm_comm,&server_status) == -1)
      {
#ifdef DEBUG
        mexPrintf("Impossible to contact the HBM interface.\n");
#endif
        hbm_activated = 0;
        end_message(hbm_comm);
        end_com(hbm_sock);
      }
    }
    if (hbm_activated)
    {
      end_message(hbm_comm);
      switch(server_status)
      {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
          break; /* just continue in the while loop */

        default:
          end_com(hbm_sock);
          end_com(Xdesc[r]);
          /**** RESTARTING ****/
          status = restart(mode,r);
          if (status != r) /* Error message already set */
            return status;
          status = nb_mxnetsolve_get(r,WAIT,plhs,nlhs);
          if (status <0)
            return status;
          break;
      }
    }
    sleep(1);
  }

  end_com(hbm_sock);

  comm = init_recv(Xdesc[r]);
  if (comm == NULL)
  {
    int status;
#ifdef DEBUG
    mexPrintf("The computational server failed\n");
#endif
    status = restart(mode,r);
    if (status != r) /* Error message already set */
    {
      return status;
    }
    if (mode == WAIT)
    {
      status = nb_mxnetsolve_get(r,WAIT,plhs,nlhs);  /* Recursive Call */
      if (status < 0)
        return status;
    }
    else
    {
      sprintf(error_message,"request %d is not completed yet \n\t\t[%s,%d]\n",
        r,__FILE__,__LINE__);
      return NetSolveNotReady;
    }
  }

  /*
   * I read the status of the computation !!!
   */
  read_int(comm,&status);
  /*
   * I read the  stuff
   */
  read_int(comm,&stdoutlength);
  stdoutbuf = (char *)mxCalloc(stdoutlength,sizeof(char));
  read_char_array(comm,stdoutbuf,stdoutlength);
  if (stdoutlength > 1)
  {
    mexPrintf("-- stdout from request '%d' --\n",r);
    mexPrintf("%s\n",stdoutbuf);
    mxFree(stdoutbuf);
  }

  if (status == SV_FAILURE)
  {
    int re;

    end_message(comm);
    end_com(Xdesc[r]);
    re = restart(mode,r);
    if (re != r)
    {
      return -1;
    }
    if (mode == WAIT)
    {
      status = nb_mxnetsolve_get(r,WAIT,plhs,nlhs);  /* Recursive Call */
      if (status < 0)
        return status;
    }
    else
    {
      return NetSolveNotReady;
    }
  }
  if (status == PB_NO_SOL)
  {
#ifdef VIEW
    mexPrintf("The problem doesn't have a solution\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    mxjob_completed(agent,XIPaddr[r]);
    mexPrintf("No solution for this problem\n");
    return -1;
  }
  if (status == DM_MISMATCH)
  {
#ifdef VIEW
    mexPrintf("The dimension are incorrect !!!\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    mxjob_completed(agent,XIPaddr[r]);
    mexPrintf("Dimension mismatch\n");
    return -1;
  }
  if (status == BAD_VALUES)
  {
#ifdef VIEW
    mexPrintf("Invalid argument values\n");
#endif
    end_message(comm);
    end_com(Xdesc[r]);
    mxjob_completed(agent,XIPaddr[r]);
    mexPrintf("Invalid parameter values\n");
    return -1;
  }


  for (i=0;i<nlhs;i++)
  {
    int m,n,total_size;
    int merge = 0;
    matlab_merge *tmp;

    obj = Xpb_desc[r]->output_obj[obj_index].object_type;
    dat = Xpb_desc[r]->output_obj[obj_index].data_type;

    mxDestroyArray(plhs[i]);

    /* Merge ? */
    if (nlhs != Xpb_desc[r]->nb_obj_output)
    {
      int j;

      tmp = Xpb_desc[r]->matlab;
      for(j=0;j<Xpb_desc[r]->nb_matlab;j++);
      {
        if (tmp->output1 == obj_index)
        {
          merge = 1;
        }
        tmp = tmp->next;
      }
    }

    switch(obj)
    {
      case NETSOLVE_FILE :
      {
        int dummy;

        read_int(comm,&dummy);
        read_int(comm,&dummy);

        read_int(comm,&m);
        n = 1;
        total_size = m*n;
        break;
      }
      case NETSOLVE_MATRIX :
      {
        int major,dummy;

        /* Useless since the server makes sure it's 
           in the right major                       */

        read_int(comm,&dummy);
        read_int(comm,&dummy);
        read_int(comm,&major);

        read_int(comm,&m);
        read_int(comm,&n);

        total_size = m*n;
        break;
      }
      case NETSOLVE_VECTOR :
      {
        int dummy;

        read_int(comm,&dummy);
        read_int(comm,&dummy);
        read_int(comm,&m);
        n = 1;
        total_size = m;
        break;
      }
      case NETSOLVE_SCALAR :
      {
        int dummy;

        read_int(comm,&dummy);
        read_int(comm,&dummy);
        m = n = 1;
        total_size = 1;
        break;
      }
    }

    switch(dat)
    {
      case NETSOLVE_I :
      {
        if (!merge)
        {
          int *tmp;
          int k;
  
          tmp = (int *)mxCalloc(sizeof(int),total_size);
          
          read_int_array(comm,tmp,total_size);
          plhs[i] = mxCreateDoubleMatrix(m,n,mxREAL);
          ptr = mxGetPr(plhs[i]);
          for (k=0;k<total_size;k++)
             ptr[k] = (double)(((int *)tmp)[k]);
          mxFree(tmp); 
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_ASCII :
      {
        if (!merge)
        {
          int fd;
          char filename[256];
          char message[256];

          sprintf(filename,"%s.output.%d.%d",Xpb_desc[r]->code,r,i);

          sprintf(message,"Output #%d saved as file '%s'\n",i,filename);
          plhs[i] = mxCreateString(message);

          fd = open(filename,O_WRONLY|O_CREAT,0644);
          if (fd < 0)
          {
            mexPrintf("Impossible to open file '%s'\n",filename);
            mexPrintf("Aborting...\n");
            end_message(comm);
            end_com(Xdesc[r]);
            return -1;
          } 
          read_char_array_file(comm,fd,total_size);
          close(fd);
          break;
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_B :
      {
        if (!merge)
        {
          char *tmp;
          int k;

          tmp = (char *)mxCalloc(sizeof(char),total_size);

          read_byte_array(comm,tmp,total_size);
          plhs[i] = (mxArray *)mxCreateString(tmp);
          mxFree(tmp);
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_CHAR :
      {
        if (!merge)
        {
          char *tmp;
          int k;

          tmp = (char *)mxCalloc(sizeof(char),total_size);

          read_char_array(comm,tmp,total_size);
          plhs[i] = (mxArray *)mxCreateString(tmp);
          mxFree(tmp);
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_S :
      {
        if (!merge)
        {
          float *tmp;
          int k;

          tmp = (float *)mxCalloc(sizeof(float),total_size);

          read_single_array(comm,tmp,total_size);
          plhs[i] = mxCreateDoubleMatrix(m,n,mxREAL);
          ptr = mxGetPr(plhs[i]);
          for (k=0;k<total_size;k++)
             ptr[k] = (double)(((float *)tmp)[k]);
          mxFree(tmp);
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_D :
      {
        if (!merge)
        {
          double *tmp;
          int k;

          tmp = (double *)mxCalloc(sizeof(double),total_size);
  
          read_double_array(comm,tmp,total_size);
          plhs[i] = mxCreateDoubleMatrix(m,n,mxREAL);
          ptr = mxGetPr(plhs[i]);
          for (k=0;k<total_size;k++)
             ptr[k] = (double)(((double *)tmp)[k]);
          mxFree(tmp);
        }
        else
        {
          double *tmp1, *tmp2;
          int dummy;
          
          plhs[i] = mxCreateDoubleMatrix(m,n,mxCOMPLEX);
          ptr = mxGetPr(plhs[i]);
          pti = mxGetPi(plhs[i]);
          
          tmp1 = (double *)mxCalloc(sizeof(double),total_size);
          tmp2 = (double *)mxCalloc(sizeof(double),total_size);
          read_double_array(comm,tmp1,total_size);
          if (obj == NETSOLVE_MATRIX)
          {
            read_int(comm,&dummy);
            read_int(comm,&dummy);
            /* Reading the major */
            read_int(comm,&dummy);
            /* Reading the dimensions */
            read_int(comm,&dummy);
            read_int(comm,&dummy);
          }
          else if (obj == NETSOLVE_VECTOR)
          {
            read_int(comm,&dummy);
            read_int(comm,&dummy);
            /* Reading the dimension */
            read_int(comm,&dummy);
          }
          else if (obj == NETSOLVE_SCALAR)
          {
            read_int(comm,&dummy);
            read_int(comm,&dummy);
          }
          read_double_array(comm,tmp2,total_size);
          for (k=0;k<m*n;k++)
          {
            ptr[k] = (double)(((double *)tmp1)[k]);
            pti[k] = (double)(((double *)tmp2)[k]);
          }
          obj_index++;
          mxFree(tmp1); mxFree(tmp2);

        }
        break;
      }
      case NETSOLVE_C :
      {
        if (!merge)
        {
          scomplex *tmp;
          int k;

          tmp = (scomplex *)mxCalloc(sizeof(scomplex),total_size);
  
          read_scomplex_array(comm,tmp,total_size);
          plhs[i] = mxCreateDoubleMatrix(m,n,mxCOMPLEX);
          ptr = mxGetPr(plhs[i]);
          pti = mxGetPi(plhs[i]);
          for (k=0;k<total_size;k++)
          {
             ptr[k] = (double)(((scomplex *)tmp)[k].r);
             pti[k] = (double)(((scomplex *)tmp)[k].i);
          }
          mxFree(tmp);
        }
        else
        {
        }
        break;
      }
      case NETSOLVE_Z :
      {
        if (!merge)
        {
          dcomplex *tmp;
          int k;

          tmp = (dcomplex *)mxCalloc(sizeof(dcomplex),total_size);

          read_dcomplex_array(comm,tmp,total_size);
          plhs[i] = mxCreateDoubleMatrix(m,n,mxCOMPLEX);
          ptr = mxGetPr(plhs[i]);
          pti = mxGetPi(plhs[i]);
          for (k=0;k<total_size;k++)
          {
             ptr[k] = (double)(((dcomplex *)tmp)[k].r);
             pti[k] = (double)(((dcomplex *)tmp)[k].i);
          }
          mxFree(tmp);
        }
        else
        {
        }
        break;
      }
    }
    obj_index++;
  }

  end_message(comm);
  end_com(Xdesc[r]);
  mxjob_completed(agent,XIPaddr[r]);
  request[r] = 0;
  
  /* A LOT OF FREE TO DO HERE */
  for (i=0;i<Xpb_desc[r]->nb_obj_input;i++)
    mxFree(Xdata_input[r][i]);
  for (i=0;i<Xpb_desc[r]->nb_obj_output;i++)
    mxFree(Xdata_output[r][i]);
  /* Should be more !!! */
  mxFree(Xpb_desc[r]);
  mxFree(Xsizes_input[r]);
  mxFree(Xsizes_output[r]);
  mxFree(Xdata_input[r]);
  mxFree(Xdata_output[r]);

  return NetSolveSuccess;
}

/*
 * Restart because of a server failure
 */

int restart(int mode,int r)
{

  char *server;
  int info;

 /*
  * Report the error to the communication server
  */

  if ((server = (char *)getenv("NETSOLVE_AGENT")) == 0)
  {
    mexPrintf("The NETSOLVE_AGENT environment variable should be set\n");
    return -1;
  }

  mxclient_f_report(-1,server,XIPaddr[r]);

 /*
  *  Submit the job again
  */

  info = mxgateway(r,Xpb_desc[r],Xsizes_input[r],Xsizes_output[r],
          Xdata_input[r],Xdata_output[r]);

  return info;
}
