/*
 * Test program for NetSolve.
 *
 *    This program is testing the C interface to NetSolve
 *
 */

#define NB_REQUESTS 5
#define SIZE 2
#include <stdio.h>
#include "netsolve.h"

main(argc,argv)
int argc;
char **argv;
{
  int m;              /* Size of the matrix and right-hand side           */
  double *a1,*b1;     /* Matrix and right-hand side for the linear system */
  int *pivot;         /* Vector of pivots returned by 'dgesv'             */
  int ierr;           /* 'dgesv' error code                               */

  int i;              /* Loop index                                  */
  int init=1325;      /* Seed of the random number generator         */
  int info;           /* NetSolve error code                         */
  int request;        /* NetSolve request handler                    */

  int requests[NB_REQUESTS];
  int go_on;
  
  int A,B;
  int stuff = 1;


  fprintf(stdout,"** Test #0 : Blocking call to 'stupidtest' **\n");

  /***********/
  /* TEST #0 */
  /***********/
  /* Calling Netsolve for 'dgesv' in a blocking fashion */
  /* For 'dgesv', the right-hand side is overwritten    */
  /* with the solution                                  */

  info = netsl("stupidtest()",&stuff);
  if (info <0)
  {
    netslerr(info);
    fprintf(stdout,"** Failure **\n");
    exit(0);
  }
  else
  {
    if (stuff == 2)
    {
      fprintf(stdout,"*************\n");
      fprintf(stdout,"** Success **\n");
      fprintf(stdout,"*************\n");
    }
    else
      fprintf(stdout,"** Failure **\n");
  }
  /*
   * Generating the random mxm matrices, as well as the
   * random right hand sides.
   */

  fprintf(stdout,"** Test #1 : Blocking call to 'dgesv' **\n");

  m = 100;
  a1 = (double *)malloc(m*m*sizeof(double));
  for (i=0;i<m*m;i++) {
    init = 2315*init % 65536;
    a1[i] = (double)((double)init - 32768.0) / 16384.0;
  }

  b1 = (double *)malloc(m*sizeof(double));
  for (i=0;i<m;i++) {
    init = 2315*init % 65536;
    b1[i] = (double)((double)init - 32768.0) / 16384.0; 
  }
  pivot = (int *)malloc(m*sizeof(double));

  netslmajor("col");


  /***********/
  /* TEST #1 */
  /***********/

  info = netsl("dgesv()",m,1,a1,m,pivot,b1,m,&ierr);
  if (info <0)
  {
    netslerr(info);
    fprintf(stdout,"** Failure **\n");
  }
  else
  {
    fprintf(stdout,"*************\n");
    fprintf(stdout,"** Success **\n");
    fprintf(stdout,"*************\n");
  }
  

  /***********/
  /* TEST #2 */
  /***********/
  /* Calling Netsolve for 'dgesv' in a non-blocking fashion */
  /* For 'dgesv', the right-hand side is overwritten        */
  /* with the solution                                      */

  fprintf(stdout,"** Test #2 : Non-blocking call to 'dgesv' and probing\n"); 
  request = netslnb("dgesv()",m,1,a1,m,pivot,b1,m,&ierr);
  if (request <0)
  {
    netslerr(request);
    fprintf(stdout,"** Failure **\n");
  }
  else
  {
    while(1) 
      {
      info = netslpr(request);
      if (info != NetSolveNotReady)
        break;
      sleep(1);
    }
    if (info <0)
    {
      netslerr(info);
      fprintf(stdout,"** Failure **\n");
    }
    else
    {
      fprintf(stdout,"*************\n");
      fprintf(stdout,"** Success **\n");
      fprintf(stdout,"*************\n");
    }
  }


  /***********/
  /* TEST #3 */
  /***********/
  /* Calling Netsolve for 'dgesv' in a non-blocking fashion */
  /* For 'dgesv', the right-hand side is overwritten        */
  /* with the solution                                      */

  fprintf(stdout,"** Test #3 : Non-blocking call to 'dgesv' and waiting\n");
  request = netslnb("dgesv()",m,1,a1,m,pivot,b1,m,&ierr);
  if (request <0)
  {
    netslerr(request);
    fprintf(stdout,"** Failure **\n");
  }
  else
  {
    info = netslwt(request);
    if (info <0)
    {
      netslerr(info);
      fprintf(stdout,"** Failure **\n");
    }
    else
    {
      fprintf(stdout,"*************\n");
      fprintf(stdout,"** Success **\n");
      fprintf(stdout,"*************\n");
    }
  }

  /***********/
  /* TEST #4 */
  /***********/
  /*
   * A bunch of requests
   */

  fprintf(stdout,"** Test #4 : %d non-blocking request to",NB_REQUESTS);
  fprintf(stdout," 'dgesv' and probing\n");

  request = 1;
  for (i=0;i<NB_REQUESTS;i++)
  {
    requests[i] = netslnb("dgesv()",m,1,a1,m,pivot,b1,m,&ierr);
    if (requests[i] <0)
    {
      request = 0;
      netslerr(requests[i]);
    }
  }
  go_on = 1;
  while(go_on)
  {
    go_on = 0;
    for (i=0;i<NB_REQUESTS;i++)
    {
      if (requests[i] <0)
        continue;
      else
        go_on = 1;
      info = netslpr(requests[i]);
      if (info <0)
      {
        request = 0;
        netslerr(info);
        requests[i] = -1;
      }
      if (info > 0)
        requests[i] = -1; 
    }
    sleep(2);
  }

  if (request == 0)
    fprintf(stdout,"** Failure **\n");
  else
  {
    fprintf(stdout,"*************\n");
    fprintf(stdout,"** Success **\n");
    fprintf(stdout,"*************\n");
  }

  /***********/
  /* TEST #5 */
  /***********/
  /* 
   * Test all data types/structures
   */

  fprintf(stdout,"** Test #5 : All data structures/types\n");

  {
    int ok = 1;
    int i,j;

    int intscalar;
    char charscalar;
    float floatscalar;
    double doublescalar;
    scomplex scomplexscalar;
    dcomplex dcomplexscalar;
    char bytescalar;

    int *intvector;
    int intvectorm=2;
    char *charvector;
    int charvectorm=2;
    float *floatvector;
    int floatvectorm=2;
    double *doublevector;
    int doublevectorm=2;
    scomplex *scomplexvector;
    int scomplexvectorm=2;
    dcomplex *dcomplexvector;
    int dcomplexvectorm=2;
    char *bytevector;
    int bytevectorm=2;

    int *intmatrix;
    int intmatrixm=3;
    int intmatrixn=2;
    int intmatrixl=4;
    char *charmatrix;
    int charmatrixm=3;
    int charmatrixn=2;
    int charmatrixl=4;
    float *floatmatrix;
    int floatmatrixm=3;
    int floatmatrixn=2;
    int floatmatrixl=4;
    double *doublematrix;
    int doublematrixm=3;
    int doublematrixn=2;
    int doublematrixl=4;
    dcomplex *dcomplexmatrix;
    int dcomplexmatrixm=3;
    int dcomplexmatrixn=2;
    int dcomplexmatrixl=4;
    scomplex *scomplexmatrix;
    int scomplexmatrixm=3;
    int scomplexmatrixn=2;
    int scomplexmatrixl=4;
    char *bytematrix;
    int bytematrixm=3;
    int bytematrixn=2;
    int bytematrixl=4;

    /* Assigning the initial values */

    /* SCALARS */
    intscalar = 2;
    charscalar = 'a';
    floatscalar = 2.0;
    doublescalar = 2.0;
    scomplexscalar.r = 2.0;
    scomplexscalar.i = 2.0;
    dcomplexscalar.r = 2.0;
    dcomplexscalar.i = 2.0;
    bytescalar = 'a';

    /* VECTORS */
    intvector = (int *)calloc(intvectorm,sizeof(int));
    for (i=0;i<intvectorm;i++)
      intvector[i] = 2;
    charvector = (char *)calloc(charvectorm,sizeof(char));
    for (i=0;i<charvectorm;i++)
      charvector[i] = 'a';
    floatvector = (float *)calloc(floatvectorm,sizeof(float));
    for (i=0;i<floatvectorm;i++)
      floatvector[i] = 2.0;
    doublevector = (double *)calloc(doublevectorm,sizeof(double));
    for (i=0;i<doublevectorm;i++)
      doublevector[i] = 2.0;
    scomplexvector = (scomplex *)calloc(scomplexvectorm,sizeof(scomplex));
    for (i=0;i<scomplexvectorm;i++)
    {
      scomplexvector[i].r = 2.0;
      scomplexvector[i].i = 2.0;
    }
    dcomplexvector = (dcomplex *)calloc(dcomplexvectorm,sizeof(dcomplex));
    for (i=0;i<dcomplexvectorm;i++)
    {
      dcomplexvector[i].r = 2.0;
      dcomplexvector[i].i = 2.0;
    }
    bytevector = (char *)calloc(bytevectorm,sizeof(char));
    for (i=0;i<bytevectorm;i++)
      bytevector[i] = 'a';
    
    /* MATRICES */
    intmatrix =
      (int *)calloc(intmatrixn*intmatrixl,sizeof(int));
    for (i=0;i<intmatrixn;i++)
      for (j=0;j<intmatrixm;j++)
        intmatrix[i*intmatrixl+j] = 2; 
    charmatrix =
      (char *)calloc(charmatrixn*charmatrixl,sizeof(char));
    for (i=0;i<charmatrixn;i++)
      for (j=0;j<charmatrixm;j++)
        charmatrix[i*charmatrixl+j] = 'a'; 
    floatmatrix =
      (float *)calloc(floatmatrixn*floatmatrixl,sizeof(float));
    for (i=0;i<floatmatrixn;i++)
      for (j=0;j<floatmatrixm;j++)
        floatmatrix[i*floatmatrixl+j] = 2.0; 
    doublematrix =
      (double *)calloc(doublematrixn*doublematrixl,sizeof(double));
    for (i=0;i<doublematrixn;i++)
      for (j=0;j<doublematrixm;j++)
        doublematrix[i*doublematrixl+j] = 2.0; 
    scomplexmatrix =
      (scomplex *)calloc(scomplexmatrixn*scomplexmatrixl,sizeof(scomplex));
    for (i=0;i<scomplexmatrixn;i++)
      for (j=0;j<scomplexmatrixm;j++)
      {
        scomplexmatrix[i*scomplexmatrixl+j].r = 2.0; 
        scomplexmatrix[i*scomplexmatrixl+j].i = 2.0; 
      }
    dcomplexmatrix =
      (dcomplex *)calloc(dcomplexmatrixn*dcomplexmatrixl,sizeof(dcomplex));
    for (i=0;i<dcomplexmatrixn;i++)
      for (j=0;j<dcomplexmatrixm;j++)
      {
        dcomplexmatrix[i*dcomplexmatrixl+j].r = 2.0;
        dcomplexmatrix[i*dcomplexmatrixl+j].i = 2.0;
      }
    bytematrix =
      (char *)calloc(bytematrixn*bytematrixl,sizeof(char));
    for (i=0;i<bytematrixn;i++)
      for (j=0;j<bytematrixm;j++)
        bytematrix[i*bytematrixl+j] = 'a';


   
    /* Calling NetSolve */

    info = netsl("totaltest()",
                    &intscalar,
                    intvector,
                    intvectorm,
                    intmatrix,
                    intmatrixm,
                    intmatrixn,
                    intmatrixl,
                    &charscalar,
                    charvector,
                    charvectorm,
                    charmatrix,
                    charmatrixm,
                    charmatrixn,
                    charmatrixl,
                    &floatscalar,
                    floatvector,
                    floatvectorm,
                    floatmatrix,
                    floatmatrixm,
                    floatmatrixn,
                    floatmatrixl,
                    &doublescalar,
                    doublevector,
                    doublevectorm,
                    doublematrix,
                    doublematrixm,
                    doublematrixn,
                    doublematrixl,
                    &scomplexscalar,
                    scomplexvector,
                    scomplexvectorm,
                    scomplexmatrix,
                    scomplexmatrixm,
                    scomplexmatrixn,
                    scomplexmatrixl,
                    &dcomplexscalar,
                    dcomplexvector,
                    dcomplexvectorm,
                    dcomplexmatrix,
                    dcomplexmatrixm,
                    dcomplexmatrixn,
                    dcomplexmatrixl,
                    &bytescalar,
                    bytevector,
                    bytevectorm,
                    bytematrix,
                    bytematrixm,
                    bytematrixn,
                    bytematrixl
               );

    if (info <0)
    {
      netslerr(info);
      ok = 0;
    }
    else
    {

      /* Checking the validity of the data */
  
      /* SCALARS */
  
      if (intscalar != 3)
      {
        fprintf(stdout,"  WARNING : Error on the 'integer scalar' value\n");
        fprintf(stdout,"            (should be 3 and is %d)\n",intscalar);
        ok = 0;
      }
      if (bytescalar != 'b')
      {
        fprintf(stdout,"  WARNING : Error on the 'byte scalar' value\n");
        fprintf(stdout,"            (should be 'b' and is '%c')\n",bytescalar);
        ok = 0;
      }
      if (charscalar != 'b')
      {
        fprintf(stdout,"  WARNING : Error on the 'character scalar' value\n");
        fprintf(stdout,"            (should be 'b' and is '%c')\n",charscalar);
        ok = 0;
      }
      if (floatscalar != 3.0)
      {
        fprintf(stdout,"  WARNING : Error on the 'float scalar' value\n");
        fprintf(stdout,"            (should be 3.0 and is %f)\n",floatscalar);
        ok = 0;
      }
      if (doublescalar != 3.0)
      {
        fprintf(stdout,"  WARNING : Error on the 'double scalar' value\n");
        fprintf(stdout,"            (should be 3.0 and is %f)\n",doublescalar);
        ok = 0;
      }
      if ((scomplexscalar.r != 3.0)||(scomplexscalar.i != 3.0))
      {
        fprintf(stdout,
          "  WARNING : Error on the 'single complex scalar' value\n");
        fprintf(stdout,"            (should be 3.0+i3.0 and is %f+i%f)\n",
                                    scomplexscalar.r,scomplexscalar.i);
        ok = 0;
      }
      if ((dcomplexscalar.r != 3.0)||(dcomplexscalar.i != 3.0))
      {
        fprintf(stdout,
          "  WARNING : Error on the 'double complex scalar' value\n");
        fprintf(stdout,"            (should be 3.0+i3.0 and is %f+i%f)\n",
                                    dcomplexscalar.r,dcomplexscalar.i);
        ok = 0;
      }

  
      /* VECTORS */
  
      for (i=0;i<intvectorm;i++)
        if (intvector[i] != 3)
        {
          fprintf(stdout,"  WARNING : Error on the 'integer vector' value\n");
          fprintf(stdout,"            (should be 3 and is %d)\n",intvector[i]);
          ok = 0;
        }
      
      for (i=0;i<bytevectorm;i++)
        if (bytevector[i] != 'b')
        {
          fprintf(stdout,"  WARNING : Error on the 'byetacter vector' value\n");
          fprintf(stdout,
            "            (should be 'b' and is '%c')\n",bytevector[i]);
          ok = 0;
        }

      for (i=0;i<charvectorm;i++)
        if (charvector[i] != 'b')
        {
          fprintf(stdout,"  WARNING : Error on the 'character vector' value\n");
          fprintf(stdout,
            "            (should be 'b' and is '%c')\n",charvector[i]);
          ok = 0;
        }

      for (i=0;i<floatvectorm;i++)
        if (floatvector[i] != 3.0)
        {
          fprintf(stdout,"  WARNING : Error on the 'float vector' value\n");
          fprintf(stdout,
            "            (should be 3.0 and is %f)\n",floatvector[i]);
          ok = 0;
        }

      for (i=0;i<doublevectorm;i++)
        if (doublevector[i] != 3.0)
        {
          fprintf(stdout,"  WARNING : Error on the 'double vector' value\n");
          fprintf(stdout,
            "            (should be 3.0 and is %f)\n",doublevector[i]);
          ok = 0;
        }

      for (i=0;i<scomplexvectorm;i++)
        if ((scomplexvector[i].r != 3.0)||(scomplexvector[i].i != 3.0))
        {
          fprintf(stdout,
                   "  WARNING : Error on the 'single complex vector' value\n");
          fprintf(stdout,
                   "            (should be 3.0+i3.0 and is %f+i%f)\n",
                    scomplexvector[i].r,scomplexvector[i].i);
          ok = 0;
        }

      for (i=0;i<dcomplexvectorm;i++)
        if ((dcomplexvector[i].r != 3.0)||(dcomplexvector[i].i != 3.0))
        {
          fprintf(stdout,
                   "  WARNING : Error on the 'double complex vector' value\n");
          fprintf(stdout,
                   "            (should be 3.0+i3.0 and is %f+i%f)\n",
                   dcomplexvector[i].r,dcomplexvector[i].i);
          ok = 0;
        }

      /* No TEST for Bytes */

    /* MATRICES */
  
      for (i=0;i<intmatrixn;i++)
        for (j=0;j<intmatrixm;j++)
          if (intmatrix[i*intmatrixl+j] != 3)
          {
            fprintf(stdout,
                    "  WARNING : Error on the 'integer matrix' value\n");
            fprintf(stdout,
                    "            (should be 3 and is %d)\n",intmatrix[i]);
            ok = 0;
          }

      for (i=0;i<bytematrixn;i++)
        for (j=0;j<bytematrixm;j++)
          if (bytematrix[i*bytematrixl+j] != 'b')
          {
            fprintf(stdout,
                  "  WARNING : Error on the 'byteacter matrix' value\n");
            fprintf(stdout,
                  "            (should be 'b' and is '%c')\n",bytematrix[i]);
            ok = 0;
          }

      for (i=0;i<charmatrixn;i++)
        for (j=0;j<charmatrixm;j++)
          if (charmatrix[i*charmatrixl+j] != 'b')
          {
            fprintf(stdout,
                  "  WARNING : Error on the 'character matrix' value\n");
            fprintf(stdout,
                  "            (should be 'b' and is '%c')\n",charmatrix[i]);
            ok = 0;
          }

      for (i=0;i<floatmatrixn;i++)
        for (j=0;j<floatmatrixm;j++)
          if (floatmatrix[i*floatmatrixl+j] != 3.0)
          {
            fprintf(stdout,
                 "  WARNING : Error on the 'float matrix' value\n");
            fprintf(stdout,
                 "            (should be 3.0 and is %f)\n",floatmatrix[i]);
            ok = 0;
          }

      for (i=0;i<doublematrixn;i++)
        for (j=0;j<doublematrixm;j++)
          if (doublematrix[i*doublematrixl+j] != 3.0)
          {
            fprintf(stdout,
                 "  WARNING : Error on the 'double matrix' value\n");
            fprintf(stdout,
                 "            (should be 3.0 and is %f)\n",doublematrix[i]);
            ok = 0;
          }

      for (i=0;i<scomplexmatrixn;i++)
        for (j=0;j<scomplexmatrixm;j++)
          if ((scomplexmatrix[i*scomplexmatrixl+j].r != 3.0) ||
              (scomplexmatrix[i*scomplexmatrixl+j].i != 3.0))
          {
            fprintf(stdout," WARNING : Error on the 'single complex matrix'\n");
            fprintf(stdout,"            (should be 3.0+i3.0 and is %f+i%f)\n",
                         scomplexmatrix[i].r,scomplexmatrix[i].i);
            ok = 0;
          }

      for (i=0;i<dcomplexmatrixn;i++)
        for (j=0;j<dcomplexmatrixm;j++)
          if ((dcomplexmatrix[i*dcomplexmatrixl+j].r != 3.0) ||
              (dcomplexmatrix[i*dcomplexmatrixl+j].i != 3.0))
          {
            fprintf(stdout,
                  "  WARNING : Error on the 'double complex matrix' value\n");
            fprintf(stdout,
                  "            (should be 3.0+i3.0 and is %f+i%f)\n",
                  dcomplexmatrix[i].r,dcomplexmatrix[i].i);
            ok = 0;
          }

       /* No test for bytes */
    }
    /* So ? */
  
    if (!ok)
      fprintf(stdout,"** Failure **\n");
    else
    {
      fprintf(stdout,"*************\n");
      fprintf(stdout,"** Success **\n");
      fprintf(stdout,"*************\n");
    }
  }

  /***********/
  /* TEST #6 */
  /***********/
  /*
   * Test files
   */
  fprintf(stdout,"** Test #6 : Files\n");
  {
    char command[256];
    char *inputfilename;
    FILE *inputfile;
    char *outputfilename;
    FILE *outputfile;
 
    /* Creating a test file */
    inputfilename = (char*)strdup("./NetSolveTestInput");
    fprintf(stderr,"Creating file '%s'\n",inputfilename);
    inputfile = fopen(inputfilename,"w");
    if (inputfile == NULL)
    {
      fprintf(stderr,"Impossible to create file '%s'.\n",inputfilename);
      exit(0);
    }
    fprintf(inputfile,"Test\n");
    fclose(inputfile);
 
    outputfilename = (char*)strdup("./NetSolveTestOutput");

    /* calling NetSolve */
    info = netsl("filetest()",inputfilename,outputfilename);
    if (info <0)
    {
      netslerr(info);
      fprintf(stdout,"** Failure **\n");
    }
    else
    { /* check the output */
       outputfile = fopen(outputfilename,"r");
       if (outputfile == NULL)
       {
         fprintf(stderr,"Output file has not been created.\n");
         fprintf(stdout,"** Failure **\n");
       } 
       else
       {
         char buf[256];
         fread(buf,1,9,outputfile);
         buf[9] = '\0';
         if (strcmp(buf,"Test\nDONE"))
         {
           fprintf(stderr,"Output file does not contain the expected data\n");
           fprintf(stdout,"** Failure **\n");
         }
         else
         {
           fprintf(stdout,"*************\n");
           fprintf(stdout,"** Success **\n");
           fprintf(stdout,"*************\n");  
         }
       }
    }
    /* Cleaning up the files */
    fprintf(stderr,"Cleaning up the files\n");
    sprintf(command,"rm -f %s %s",inputfilename,outputfilename);
    system(command);
  } 
   

}
